﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net.Sockets;
using System.IO;
using System.Threading;
using System.Xml.Serialization;
using System.Windows.Forms;

namespace MoyariClient
{
    public class DownloadManager
    {
        // タスクキュー
        private static List<DownloadTaskInfo> taskQueue = new List<DownloadTaskInfo>();
        // リジューム情報
        private static ResumeInfo resumeInfo = new ResumeInfo();

        // 衝突回避用
        public static DownloadTaskInfo[] getTaskAry(){
            DownloadTaskInfo[] result = null;
            lock (taskQueue)
            {
                result = taskQueue.ToArray();
            }
            return result;
        }

        // ダウンロードタスクの追加検索窓からのみ
        public static void addDownloadTaskFromSearchWindow(ServerInfo serverInfo, SearchResult result)
        {
            addDownloadTask(serverInfo, result);
            FormTransfer.downloadTabActivate();
        }

        // ダウンロードタスクの追加
        public static void addDownloadTask(ServerInfo serverInfo, SearchResult result, bool enterRemoQflg = false, bool autoRetryflg = false)
        {
            DownloadTaskInfo task = FormTransfer.initDownLoadTask(serverInfo, result);
            if (task == null)
            {
                return;
            }
            task.remoteQueueflg = enterRemoQflg;
            task.autoRetryflg = autoRetryflg;
            lock (taskQueue)
            {
                taskQueue.Add(task);
            }
            FormTransfer.refleshDownloadListALL();

            // 可能ならDL開始
            startDownloadIfAble(false);
        }

        // ダウンロードタスクの削除
        public static void removeDownloadTasks(DownloadTaskInfo removeTask, bool reflesh)
        {
            removeDownloadTasks(new DownloadTaskInfo[] { removeTask }, reflesh);
        }
        public static void removeDownloadTasks(DownloadTaskInfo[] removeTaskAry, bool reflesh)
        {
            lock (taskQueue)
            {
                foreach (DownloadTaskInfo rt in removeTaskAry)
                {
                    rt.cancel();
                    taskQueue.Remove(rt);
                }
            }

            if(reflesh)
            {
                FormTransfer.refleshDownloadListALL(); //リスト作り直し
            }
        }

        // キューからDL可能なタスクがあればDL開始する
        public static void startDownloadIfAble(bool relativeTask)
        {
            int activeDLCounter = 0;
            Dictionary<String, int> activeDLNickMap = new Dictionary<String, int>();

            DownloadTaskInfo[] taskAry = getTaskAry();
            foreach (DownloadTaskInfo task in taskAry)
            {
                if (isActiveTask(task))
                {
                    activeDLCounter += 1;
                    if (activeDLNickMap.ContainsKey(task.searchResult.nick))
                    {
                        activeDLNickMap[task.searchResult.nick] += 1;
                    }
                    else
                    {
                        activeDLNickMap.Add(task.searchResult.nick, 1);
                    }
                    if (activeDLCounter >= AppCommon.setting.maxDownloadNum)
                    {
                        return; //DL最大数超え
                    }
                }
            }

            foreach (DownloadTaskInfo task in taskAry)
            {
                if (isWaitingTask(task)== false)
                {
                    continue;
                }
                bool flg = true;
                if (activeDLNickMap.ContainsKey(task.searchResult.nick))
                {
                    if (activeDLNickMap[task.searchResult.nick] >= AppCommon.setting.maxDownloadNumAtUser)
                    {
                        flg = false;
                    }
                }
                if (flg)
                {
                    if (relativeTask)
                    {
                        FormTransfer.addConsoleMessage("次処理待機中(" + (AppCommon.DL_SPAN_SLEEP / 1000) + "秒) ");
                        Thread.Sleep(AppCommon.DL_SPAN_SLEEP);
                    }
                    String msg = "ダウンロード要求します：";
                    if (task.remoteQueueflg)
                    {
                        msg = "リモートキュー登録要求します：";
                    }
                    FormTransfer.addConsoleMessage(msg + task.logInfo());
                    
                    task.downloading = true;
                    task.state = DownloadTaskState.REQUESTING;
                    if (task.remoteQueueflg)
                    {
                        ServerManager.wantQueueRequest(task); // "//WantQueue"
                    }
                    ServerManager.donwloadRequest(task);
                    
                    if (activeDLNickMap.ContainsKey(task.searchResult.nick))
                    {
                        activeDLNickMap[task.searchResult.nick] += 1;
                    }
                    else
                    {
                        activeDLNickMap.Add(task.searchResult.nick, 1);
                    }

                    FormTransfer.refleshDownloadListViewItem(task);
                    if (task.remoteQueueflg == false) //リモキュータスクはタイムアウトさせない
                    {
                        downloadTimeOutCheck(task);
                    }
                }
            }
        }

        // タイムアウトスレッド
        private static void downloadTimeOutCheck(DownloadTaskInfo task)
        {
            Task reqTimeoutThread = Task.Factory.StartNew(() =>
            {
                Thread.Sleep(AppCommon.setting.requestTimeoutSec * AppCommon.SEC_TO_MILLSEC_RATE);
                if (task.downloading == false || task.state != DownloadTaskState.REQUESTING)
                {
                    // 状態が動いていたらなにもしない
                    return;
                }

                if (task.autoRetryflg || AppCommon.setting.timeout2retry)
                {
                    // リトライタスク
                    autoRetryProc(task);
                }
                else
                {
                    // 通常タスク
                    task.downloading = false;
                    task.state = DownloadTaskState.DEAD;
                    task.errorString = "タイムアウト";
                    FormTransfer.addConsoleMessage("タイムアウトしました：" + task.logInfo());
                    FormTransfer.refleshDownloadListViewItem(task);
                    startDownloadIfAble(false);
                }                
            });
        }

        // オートリトライ：リトライスレッド
        private static void autoRetryProc(DownloadTaskInfo task)
        {
            Task autoRetryThread = Task.Factory.StartNew(() =>
            {
                //リトライ状態にして描画更新
                task.autoRetrySleeping = true;
                FormTransfer.addConsoleMessage("リトライ待機(" + (AppCommon.DL_AUTO_RETRY_SLEEP) + "秒)：" + task.logInfo());
                FormTransfer.refleshDownloadListViewItem(task);

                //スリープ
                Thread.Sleep(AppCommon.DL_AUTO_RETRY_SLEEP*AppCommon.SEC_TO_MILLSEC_RATE);
                if (task.downloading == false || task.state != DownloadTaskState.REQUESTING)
                {
                    // 状態が動いていたらなにもしない
                    return;
                }

                //自分は止めて削除：
                task.downloading = false;
                task.state = DownloadTaskState.DEAD;
                task.errorString = "リトライ削除";
                removeDownloadTasks(task, false);

                //リトライフラグを引き継いで再突入
                addDownloadTask(task.serverInfo, task.searchResult, task.remoteQueueflg , task.autoRetryflg);
            });
        }

        // ack待ちのtaskを探す
        private static DownloadTaskInfo checkAckWaitTask(ServerInfo serverInfo, String nick, String filepath)
        {
            DownloadTaskInfo task = null;
            DownloadTaskInfo[] taskAry = getTaskAry();
            foreach (DownloadTaskInfo t in taskAry)
            {
                // リクエスト中 & 鯖一致 & ユーザ一致 & リモートファイルパス一致
                if (t.state == DownloadTaskState.REQUESTING &&
                    t.serverInfo.index == serverInfo.index &&
                    t.searchResult.nick == nick &&
                    t.searchResult.filepath == filepath)
                {
                    task = t;
                    break;
                }
            }
            if (task == null)
            {
                Console.WriteLine("ack対応タスクがない[" + serverInfo.name + "][" + nick + "][" + filepath + "]");
                return null;
            }
            if (task.downloading == false)
            {
                Console.WriteLine("キャンセル済み[" + filepath + "]");
                return null;
            }
            return task;
        }

        // DL_ACKのかわりに来ることがある
        public static void downloadQueueLimit(ServerInfo serverInfo, String nick, String filepath, String filesize, String digit)
        {
            DownloadTaskInfo task = checkAckWaitTask(serverInfo, nick, filepath);
            if (task == null)
            {
                return;
            }
            
            if (task.autoRetryflg){ // 優先度　リトライ > リモキュー
                // リトライタスク
                autoRetryProc(task);
            }
            else if (task.remoteQueueflg)
            {
                //リモキュータスク
                task.remoteQueueSize = digit;
                FormTransfer.refleshDownloadListViewItem(task);
            }            
            else
            {
                // 通常タスクの場合
                FormTransfer.addConsoleMessage("エラー：キュー限界(Server Busy)：" + task.logInfo());
                if (AppCommon.setting.remoteQueueAuto)
                {
                    FormTransfer.addConsoleMessage("->自動リモートキュー登録します");
                    //自分は止めて削除
                    task.downloading = false;
                    task.state = DownloadTaskState.DEAD;
                    task.errorString = "リトライ削除";
                    removeDownloadTasks(task, false);                    

                    //リモートキューとして再突入
                    addDownloadTask(task.serverInfo, task.searchResult, true, task.autoRetryflg); 
                }
                else
                {
                    FormTransfer.addConsoleMessage("->このファイルはリモートキュー登録可能です");
                    task.downloading = false;
                    task.state = DownloadTaskState.DEAD;
                    task.errorString = "キュー限界(Server Busy)";
                    task.queueLimitFlg = true;
                    FormTransfer.refleshDownloadListViewItem(task);
                }

                startDownloadIfAble(true);// 次へ
            }
        }

        // DL_ACK取得時
        public static void downloadAcked(ServerInfo serverInfo, DownloadAckInfo ackInfo) 
        {
            DownloadTaskInfo task = checkAckWaitTask(serverInfo, ackInfo.nick, ackInfo.filepath); ;
            if (task == null)
            {
                return;
            }

            FormTransfer.addConsoleMessage("ダウンロードを開始します：" + task.logInfo());
            if (task.isResume)
            {
                FormTransfer.addConsoleMessage("->リジュームします " + task.resumeLength + "bytes");
            }

            // 通常DL/ポト0DLの分岐
            ackInfo.portNum = int.Parse(ackInfo.port);

            if (ackInfo.portNum != 0)
            {
                // 通常ダウンロード
                normalDownloadThread(serverInfo, ackInfo, task);
            }
            else
            {
                // ポート0ダウンロード
                task.portZeroDownloadFlg = true;
                ServerManager.altDownloadRequest(serverInfo, ackInfo);
                return;
            }
        }

        // 通常ダウンロードスレッド
        public static void normalDownloadThread(ServerInfo serverInfo, DownloadAckInfo ackInfo, DownloadTaskInfo task)
        {
            Task downloadThread = Task.Factory.StartNew(() =>
            {
                NetworkStream stream = null;
                FileStream fsLocal = null;
                try
                {
                    //コマンド：DLスタート
                    ServerManager.donwloadStart(task);

                    //対象を取得:dl中
                    task.state = DownloadTaskState.DONWLOADING;

                    // 保存先ファイルを確定、stream開いてlock                    
                    if (task.isResume)
                    {
                        fsLocal = new FileStream(task.localWorkFilePath, FileMode.Open);
                        fsLocal.Seek(0, SeekOrigin.End);
                        task.completeSize = task.resumeLength;
                    }
                    else
                    {
                        fsLocal = new FileStream(task.localWorkFilePath, FileMode.Create);
                    }

                    addResumeInfo(task.searchResult.filename, task.searchResult.sizeNum, task.localWorkFileName);

                    //接続
                    //long ipNum = long.Parse(ackInfo.ip);
                    //byte[] ipBa = BitConverter.GetBytes(ipNum);
                    //String ipStr = (int)ipBa[0] + "." + (int)ipBa[1] + "." + (int)ipBa[2] + "." + (int)ipBa[3];
                    String hostStr = AppCommon.ipStr2hostStr(ackInfo.ip);

                    TcpClient sock = new TcpClient(hostStr, ackInfo.portNum);
                    sock.SendTimeout = AppCommon.setting.sockTimeoutSec * AppCommon.SEC_TO_MILLSEC_RATE;
                    sock.ReceiveTimeout = AppCommon.setting.sockTimeoutSec * AppCommon.SEC_TO_MILLSEC_RATE;
                    stream = sock.GetStream();

                    // ヘッダ確認
                    int headbyte = stream.ReadByte();
                    if (headbyte != AppCommon.DOWNLOAD_START_BYTE)
                    {
                        //ackbyte不正
                        task.state = DownloadTaskState.DEAD;
                        task.errorString = "エラー中断";
                        FormTransfer.addConsoleMessage("エラー中断します(不正な接続先)：" + task.logInfo());
                        return;
                    }

                    // コマンド送信
                    byte[] sendBytes = AppCommon.DOWNLOAD_GET_BYTES;
                    stream.Write(sendBytes, 0, sendBytes.Length);
                    stream.Flush();

                    //<mynick> "<filename>" <offset> 
                    String getCmd = serverInfo.user + " \"" + task.searchResult.filepath + "\" ";
                    if (task.isResume)
                    {
                        getCmd += task.resumeLength.ToString();
                    }
                    else
                    {
                        getCmd += "0";
                    }
                    sendBytes = AppCommon.ENC_SJIS.GetBytes(getCmd);
                    stream.Write(sendBytes, 0, sendBytes.Length);
                    stream.Flush();

                    // サイズ取得
                    int searchResultSizeNumDigit = task.searchResult.sizeNum.ToString().Length;
                    int sizeNumDigitCount = 0;
                    String sizeStrTmp = "";
                    byte ncByte = 0x00;
                    while (task.downloading)
                    {
                        int sizeByte = stream.ReadByte();
                        if (48 <= sizeByte && sizeByte <= 57 && sizeNumDigitCount < searchResultSizeNumDigit) //0-9
                        {
                            sizeStrTmp += ((char)sizeByte);
                            sizeNumDigitCount += 1;
                        }
                        else
                        {
                            ncByte = (byte)(0xff & sizeByte);
                            break;
                        }
                    }

                    task.searchResult.sizeNum = long.Parse(sizeStrTmp); //ファイル長更新                            
                    task.searchResult.sizeStr = task.searchResult.sizeNum.ToString(AppCommon.FILE_SITE_FOMMAT);
                    fsLocal.WriteByte(ncByte);
                    task.completeSize += 1;

                    normalDownloadBody(task, stream, fsLocal);

                }
                catch (Exception e)
                {
                    if (task.downloading)
                    {
                        task.state = DownloadTaskState.DEAD;
                        task.errorString = "エラー";
                        FormTransfer.addConsoleMessage("エラー発生。中断します：" + task.logInfo() + "\n" + e.Message + "\n" + e.StackTrace);
                    }
                }
                finally
                {
                    task.downloading = false;
                    try
                    {
                        if (stream != null)
                        {
                            stream.Close();
                            stream = null;
                        }
                    }
                    catch (Exception)
                    {
                        //do nothing
                    }

                    try
                    {
                        if (fsLocal != null)
                        {
                            fsLocal.Close();
                            fsLocal = null;
                        }
                    }
                    catch (Exception)
                    {
                        //do nothing
                    }

                    ServerManager.donwloadComplete(task);// コマンド：DL完了 (成否問わず
                    startDownloadIfAble(true);// 次へ
                }
            });
        }

        //　ダウンロード共通部
        public static void normalDownloadBody(DownloadTaskInfo task, NetworkStream stream, FileStream fsLocal)
        {
            byte[] buffer = new byte[AppCommon.DL_BUFFER_SIZE];
            bool flgComplete = false;
            int len = -1;
            long startTime = DateTime.Now.ToFileTime();
            long oldTime = startTime;
            // 本体取得
            while (task.downloading)
            {
                len = stream.Read(buffer, 0, buffer.Length);
                if (len <= 0)
                {
                    break;
                }
                else
                {
                    fsLocal.Write(buffer, 0, len);
                    fsLocal.Flush();
                    long nowTime = DateTime.Now.ToFileTime();
                    task.completeSize += len;
#if DEBUG
                    // Console.WriteLine("★" + task.completeSize +  "/" + task.searchResult.sizeNum);
                    Thread.Sleep(30);
#endif
                    if (task.completeSize >= task.searchResult.sizeNum)
                    {
                        flgComplete = true; //正常完了
                        break;
                    }
                    if ((nowTime - oldTime) > AppCommon.DL_REFLESH_RATE_NS)
                    {
                        float pastSec = (nowTime - startTime) / AppCommon.DL_REFLESH_RATE_NS;
                        float kbPerSec = (((float)task.completeSize) / pastSec) / 1000f;
                        task.dlSpeed = kbPerSec.ToString("0.00");
                        task.compPct = ((float)task.completeSize) / ((float)task.searchResult.sizeNum);
                        task.compPctString = (((int)(task.compPct * 100)).ToString() + "%");
                        FormTransfer.refleshDownloadListViewItem(task);
                        oldTime = nowTime;
                    }
                }
            }

            if (flgComplete)
            {
                long diffTime = DateTime.Now.ToFileTime() - startTime;
                float pastSec = diffTime / AppCommon.DL_REFLESH_RATE_NS;
                if (pastSec < 1)
                {
                    pastSec = 1;
                }
                float kbPerSec = (((float)task.completeSize) / pastSec) / 1000f;
                task.dlSpeed = kbPerSec.ToString("0.00");

                //正常終了の場合
                fsLocal.Close();
                File.Move(task.localWorkFilePath, task.localTrueFilePath); //例外時に落としたいので捕まえない
                delResumeInfo(task.searchResult.filename, task.searchResult.sizeNum);
                
                //コマンド：DL共有追加 正常時
                task.state = DownloadTaskState.COMPLETE;
                FormTransfer.addConsoleMessage("ダウンロード完了しました：" + task.logInfo());
                UploadManager.addShareLocalFilePath(task.localTrueFilePath);
                ServerManager.notifyShareFile(task.serverInfo, task.localTrueFilePath, task.completeSize);
            }
            else
            {
                if (task.state == DownloadTaskState.CANCELED)
                {
                    //キャンセルされてる場合は何もしない
                }
                else
                {
                    // 不完全の場合
                    task.state = DownloadTaskState.DEAD;
                    task.errorString = "切断されました";
                    FormTransfer.addConsoleMessage("ダウンロードが切断されました：" + task.logInfo() + " " + task.completeSize + "bytes");
                }
            }
            FormTransfer.refleshDownloadListViewItem(task);
        }

        // active判定
        public static bool isActiveTask(DownloadTaskInfo qt)
        {
            return (qt.state == DownloadTaskState.REQUESTING || qt.state == DownloadTaskState.DONWLOADING);
        }

        // 実行待ち判定
        public static bool isWaitingTask(DownloadTaskInfo qt)
        {
            return (qt.state == DownloadTaskState.WAIT);
        }

        // リジューム情報ロード(App起動時のみ)
        public static void initResumeInfo()
        {
            if (File.Exists(AppCommon.RESUME_FILE))
            {
                try
                {
                    XmlSerializer serializer = new XmlSerializer(typeof(ResumeInfo));
                    FileStream fs = new FileStream(AppCommon.RESUME_FILE, FileMode.Open);
                    resumeInfo = (ResumeInfo)serializer.Deserialize(fs);
                    fs.Close();
                }
                catch (Exception e)
                {
                    //初期化はMsgBox
                    MessageBox.Show("リジューム情報読み込みに失敗しました：" + e.Message, "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

                // ファイルの確認
                String tmpDirPath = AppCommon.getDownloadDirWithSceparator();
                List<ResumeTaskInfo> taskList = new List<ResumeTaskInfo>();
                foreach (ResumeTaskInfo rt in resumeInfo.tasks)
                {
                    if (File.Exists(tmpDirPath + rt.workFileName))
                    {
                        taskList.Add(rt);
                    }
                }
                resumeInfo.tasks = taskList.ToArray();
            }
        }

        // リジューム情報更新(追加/削除共通)
        private static void saveResumeInfo()
        {
            try
            {
                XmlSerializer serializer = new XmlSerializer(typeof(ResumeInfo));
                FileStream fs = new FileStream(AppCommon.RESUME_FILE, FileMode.Create);
                serializer.Serialize(fs, resumeInfo);
                fs.Close();
            }
            catch (Exception e)
            {
                FormTransfer.addConsoleMessage("リジューム情報保存に失敗しました：" + e.Message);
                return;
            }
        }

        // リジューム情報追加
        public static void addResumeInfo(String srcFileName, long srcFileSize, String workFileName)
        {
            ResumeTaskInfo resumeInf = new ResumeTaskInfo();
            resumeInf.srcFileName = srcFileName;;
            resumeInf.srcFileSize = srcFileSize;
            resumeInf.workFileName = workFileName;

            lock (resumeInfo)
            {
                List<ResumeTaskInfo> taskList = new List<ResumeTaskInfo>(resumeInfo.tasks);
                taskList.Add(resumeInf);
                resumeInfo.tasks = taskList.ToArray();
                saveResumeInfo();
            }
        }

        // リジューム情報削除
        private static void delResumeInfo(String srcFileName, long srcFileSize)
        {
            lock (resumeInfo)
            {
                List<ResumeTaskInfo> taskList = new List<ResumeTaskInfo>();
                foreach (ResumeTaskInfo rt in resumeInfo.tasks)
                {
                    if (rt.srcFileName != srcFileName || rt.srcFileSize != srcFileSize)// local名は見ない
                    {
                        taskList.Add(rt);
                    }
                }
                resumeInfo.tasks = taskList.ToArray();
                saveResumeInfo();
            }
        }

        // リジューム情報リスト取得(使う時
        public static ResumeTaskInfo[] getResumeTaskInfoAry()
        {
            List<ResumeTaskInfo> resultList = new List<ResumeTaskInfo>();
            lock (resumeInfo)
            {
                foreach(ResumeTaskInfo rt in resumeInfo.tasks){
                    resultList.Add(rt);
                }
            }
            return resultList.ToArray();
        }
    }
    
    public class DownloadTaskInfo
    {
        private static int indexCounter = 10;
        public int index = 0;
        public bool downloading = false; //リクエスト(203)で上げる。タスク生存期間判定フラグ
        public DownloadTaskState state = DownloadTaskState.DEAD;
        public ServerInfo serverInfo;
        public SearchResult searchResult;
        public long completeSize = 0;
        public String dlSpeed = null;
        public String errorString = null;
        public String compPctString = null;
        public float compPct = 0;//0-1.0

        public bool isResume = false;
        public long resumeLength = 0;
        public String localTrueFilePath = null;
        public String localTrueFileName = null;
        public String localWorkFilePath = null;
        public String localWorkFileName = null;
        
        public bool queueLimitFlg = false; //キューリミットを受けた後、リモキューを許可するフラグ
        public bool remoteQueueflg = false; //リモキューとして開始する時のフラグ
        public String remoteQueueSize = null; //リモキュー時の表示用待ち数
        public bool autoRetryflg = false; //オートリトライ
        public bool autoRetrySleeping = false; //オートリトライ(待機中

        public bool portZeroDownloadFlg = false;

        public DownloadTaskInfo(ServerInfo serverInfo, SearchResult searchResult)
        {
            this.serverInfo = serverInfo;
            this.searchResult = searchResult;
            this.state = DownloadTaskState.WAIT;
            indexCounter += 1;
            this.index = indexCounter;
        }

        //同一ユーザ判定
        public bool eqCheck(DownloadTaskInfo op)
        {
            if (this.serverInfo.index == op.serverInfo.index)
            {
                if (this.searchResult.nick == op.searchResult.nick)
                {
                    return true;
                }
            }
            return false;
        }

        public void cancel()
        {
            this.downloading = false;
            this.state = DownloadTaskState.CANCELED;
        }
        
        public String logInfo()
        {
            return "  " + searchResult.nick + "[" + serverInfo.name + "] " + searchResult.filename;
        }
    }
    
    public class ResumeInfo
    {
        public ResumeTaskInfo[] tasks = new ResumeTaskInfo[0];
    }

    public class ResumeTaskInfo
    {
        //元ファイル名　元ファイルサイズ　現ローカルファイル名(work名
        public String srcFileName;
        public long srcFileSize;
        public String workFileName;
    }

    public enum DownloadTaskState
    {
        //増やす場合は判定も考慮
        DEAD,
        CANCELED,
        WAIT ,
        REQUESTING , //active
        DONWLOADING, //active
        COMPLETE ,
    }
}
