﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MoyariClient
{
    public partial class FormConfig : Form
    {
        private MainForm parent = null;
        private ColorDialog cdBox = new ColorDialog();

        private bool colorChanged = false;
        private bool oldShareFlg = false;
        private int oldSharePort = 0;
        private String oldLinkType = null;

        private bool oldShareSubDirFlg = false;
        private String oldDownloadFolder = null;
        private String[] oldShareDirs = null;

        public FormConfig(MainForm parent)
        {
            this.AutoScaleMode = AutoScaleMode.None;
            InitializeComponent();
            this.parent = parent;

            this.cdBox.AllowFullOpen = true;
            this.cdBox.SolidColorOnly = false;

            SettingInfo info = AppCommon.setting;

            // 基本項目
            this.checkBoxTopMost.Checked = info.topMost;
            this.checkBoxTaskTray.Checked = info.taskTray;
            this.checkBoxLogEnable.Checked = info.enableLog;
            this.textBoxLogDir.Text = info.logDir;
            this.checkBoxAutoRetry.Checked = info.autoRetry;
            this.checkBoxSaveWindow.Checked = info.saveWindowPos;
            this.checkBoxTrams.Checked = info.enableTrans;
            if (this.transVolume.Minimum < info.transVolume && info.transVolume <this.transVolume.Maximum){
                this.transVolume.Value = info.transVolume;
            }
            checkBoxTramsChange();
            this.listBoxIgnore.Items.Clear();
            this.listBoxIgnore.Items.AddRange(info.ignoreList.ToArray());
            
            //フォント
            buttonFont.Text = info.txtFont.Name;
            buttonFont.Font = info.txtFont;
                        
            //ﾒｯｾｰｼﾞ
            buttonColor1.BackColor = AppCommon.setting.colorInfo.colorBackground;
            buttonColor1.ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
            //背景
            buttonColor2.BackColor = AppCommon.setting.colorInfo.colorBackground;
            buttonColor2.ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
            //時刻
            buttonColor3.BackColor = AppCommon.setting.colorInfo.colorBackground;
            buttonColor3.ForeColor = AppCommon.setting.colorInfo.colorTime;
            //名前
            buttonColor4.BackColor = AppCommon.setting.colorInfo.colorBackground;
            buttonColor4.ForeColor = AppCommon.setting.colorInfo.colorNick;
            //ｼｽﾃﾑ
            buttonColor5.BackColor = AppCommon.setting.colorInfo.colorBackground;
            buttonColor5.ForeColor = AppCommon.setting.colorInfo.colorSysMsg;
            //入室
            buttonColor6.BackColor = AppCommon.setting.colorInfo.colorBackground;
            buttonColor6.ForeColor = AppCommon.setting.colorInfo.colorJoinMsg;
            //退室
            buttonColor7.BackColor = AppCommon.setting.colorInfo.colorBackground;
            buttonColor7.ForeColor = AppCommon.setting.colorInfo.colorPartMsg;
            //ﾌｫｰｶｽ(背)
            buttonColor8.BackColor = AppCommon.setting.colorInfo.colorFocusChBack;
            buttonColor8.ForeColor = AppCommon.setting.colorInfo.colorFocusChFore;
            //ﾌｫｰｶｽ(前)
            buttonColor9.BackColor = AppCommon.setting.colorInfo.colorFocusChBack;
            buttonColor9.ForeColor = AppCommon.setting.colorInfo.colorFocusChFore;

            // httpd関連
            checkBoxHttpdEnabled.Checked = AppCommon.setting.httpdEnabled;
            textBoxHttpdPort.Text = AppCommon.setting.httpdPort.ToString();
            textBoxHttpdUserName.Text = AppCommon.setting.httpdUserName;
            textBoxHttpdPassword.Text = AppCommon.setting.httpdPassword;
            checkBoxHttpdViewport.Checked = AppCommon.setting.httpdViewPort;
            textBoxHttpdUpperFrame.Text = AppCommon.setting.httpdUpperFrameSize;
            textBoxHttpdViewport.Text = AppCommon.setting.httpdViewPortValue;
            checkBoxHttpdEnabled_CheckedChanged(null, null);

            //呼び出し小窓
            checkBoxCalling.Checked = AppCommon.setting.enableCalling;
            checkBoxCallByNick.Checked = AppCommon.setting.callByNick;
            textBoxCallingSoundFile.Text = AppCommon.setting.callingSoundFilePath;
            listBoxAddKeyword.Items.Clear();
            if (AppCommon.setting.appendKeywords != null)
            {
                foreach (String kwd in AppCommon.setting.appendKeywords)
                {
                    listBoxAddKeyword.Items.Add(kwd);
                }
            }
            callingSetEnableChange();

            // 共有
            oldShareFlg = AppCommon.setting.enableShare;
            checkBoxEnableShare.Checked = oldShareFlg;
            try
            {
                comboBoxLinkType.SelectedIndex = int.Parse(AppCommon.setting.linktype);
            }
            catch (Exception)
            {
                comboBoxLinkType.SelectedIndex = 0;
                //デフォルトunknown
            }
            textBoxSharedPort.Text = AppCommon.setting.shareWaitPort.ToString();
            textBoxDownLoadFolder.Text = AppCommon.setting.downloadFolder;
            checkBoxShareSubfolder.Checked = AppCommon.setting.enableSubFolder;
            oldShareSubDirFlg = AppCommon.setting.enableSubFolder;
            oldShareDirs = AppCommon.setting.shareFolderList;
            oldDownloadFolder = AppCommon.setting.downloadFolder;
        
            listBoxShareDirList.Items.Clear();
            if (AppCommon.setting.shareFolderList != null)
            {
                foreach (String dir in AppCommon.setting.shareFolderList)
                {
                    listBoxShareDirList.Items.Add(dir);
                }
            }
            oldSharePort = AppCommon.setting.shareWaitPort;
            oldLinkType = AppCommon.setting.linktype;
            //numericMaxDL.Value = AppCommon.setting.maxDownloadNum;
            //numericMaxUL.Value = AppCommon.setting.maxUploadNum;
            shareCheckChange();
            
        }

        //保存
        private void button1_Click(object sender, EventArgs e)
        {
            SettingInfo info = AppCommon.setting;

            // vridation
            int httpdportnum = AppCommon.DEFAULT_HTTPD_PORT;
            if (checkBoxHttpdEnabled.Checked)
            {
                if (textBoxHttpdUserName.Text.Length == 0 || textBoxHttpdPassword.Text.Length == 0)
                {
                    MessageBox.Show("簡易HTTP機能を使用する際は認証情報が必須です", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                if (textBoxHttpdUserName.Text.Length == 0 || textBoxHttpdPassword.Text.Length == 0)
                {
                    MessageBox.Show("簡易HTTP機能を使用する際はポート番号が必須です", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }
            try
            {
                httpdportnum = int.Parse(textBoxHttpdPort.Text);
            }
            catch (FormatException)
            {
                MessageBox.Show("httpdポートの設定値が不正です。", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            int sharePort = AppCommon.DEFAULT_SHARE_PORT;
            try
            {
                sharePort = int.Parse(textBoxSharedPort.Text);
            }
            catch (FormatException)
            {
                MessageBox.Show("共有ポートの設定値が不正です。", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            // -------------------------------------
            // 基本項目
            info.topMost = this.checkBoxTopMost.Checked;
            info.taskTray = this.checkBoxTaskTray.Checked;
            info.enableLog = this.checkBoxLogEnable.Checked;
            info.logDir = this.textBoxLogDir.Text;
            info.autoRetry = this.checkBoxAutoRetry.Checked;
            info.saveWindowPos = this.checkBoxSaveWindow.Checked;
            if (info.saveWindowPos)
            {
                info.windowPosX = this.parent.Location.X;
                info.windowPosY = this.parent.Location.Y;
                info.windowWidth = this.parent.Width;
                info.windowHeight = this.parent.Height;
            }
            info.enableTrans = this.checkBoxTrams.Checked;
            info.transVolume = (int)this.transVolume.Value;
            info.ignoreList.Clear();
            foreach (String s in this.listBoxIgnore.Items)
            {
                info.ignoreList.Add(s);
            }
            

            // フォント
            info.fontName = buttonFont.Font.Name;
            info.fontSize = (int)buttonFont.Font.Size;
            info.txtFont = buttonFont.Font;

            //色             
            //ﾒｯｾｰｼﾞ
            AppCommon.setting.colorInfo.colorNormalMsg = buttonColor1.ForeColor;
            //背景
            AppCommon.setting.colorInfo.colorBackground = buttonColor2.BackColor;            
            //時刻
            AppCommon.setting.colorInfo.colorTime = buttonColor3.ForeColor;
            //名前
            AppCommon.setting.colorInfo.colorNick = buttonColor4.ForeColor;
            //ｼｽﾃﾑ
            AppCommon.setting.colorInfo.colorSysMsg = buttonColor5.ForeColor;
            //入室
            AppCommon.setting.colorInfo.colorJoinMsg = buttonColor6.ForeColor;
            //退室
            AppCommon.setting.colorInfo.colorPartMsg = buttonColor7.ForeColor;
            //ﾌｫｰｶｽ(背)
            AppCommon.setting.colorInfo.colorFocusChBack = buttonColor8.BackColor;
            //ﾌｫｰｶｽ(前)
            AppCommon.setting.colorInfo.colorFocusChFore = buttonColor9.ForeColor;
            
            AppCommon.setting.colorNormalMsgHtml = ColorTranslator.ToHtml(AppCommon.setting.colorInfo.colorNormalMsg);
            AppCommon.setting.colorBackgroundHtml = ColorTranslator.ToHtml(AppCommon.setting.colorInfo.colorBackground);
            AppCommon.setting.colorTimeHtml = ColorTranslator.ToHtml(AppCommon.setting.colorInfo.colorTime);
            AppCommon.setting.colorNickHtml = ColorTranslator.ToHtml(AppCommon.setting.colorInfo.colorNick);
            AppCommon.setting.colorSysMsgHtml = ColorTranslator.ToHtml(AppCommon.setting.colorInfo.colorSysMsg);
            AppCommon.setting.colorJoinMsgHtml = ColorTranslator.ToHtml(AppCommon.setting.colorInfo.colorJoinMsg);
            AppCommon.setting.colorPartMsgHtml = ColorTranslator.ToHtml(AppCommon.setting.colorInfo.colorPartMsg);
            AppCommon.setting.colorFocusChBackHtml = ColorTranslator.ToHtml(AppCommon.setting.colorInfo.colorFocusChBack);
            AppCommon.setting.colorFocusChForeHtml = ColorTranslator.ToHtml(AppCommon.setting.colorInfo.colorFocusChFore);

            //httpd
            AppCommon.setting.httpdEnabled = checkBoxHttpdEnabled.Checked;
            AppCommon.setting.httpdPort = httpdportnum;
            AppCommon.setting.httpdUserName = textBoxHttpdUserName.Text;
            AppCommon.setting.httpdPassword = textBoxHttpdPassword.Text;
            AppCommon.setting.httpdViewPort = checkBoxHttpdViewport.Checked;
            AppCommon.setting.httpdUpperFrameSize = textBoxHttpdUpperFrame.Text;
            AppCommon.setting.httpdViewPortValue = textBoxHttpdViewport.Text;

            //呼び出し小窓
            AppCommon.setting.enableCalling = checkBoxCalling.Checked;
            AppCommon.setting.callByNick = checkBoxCallByNick.Checked;
            AppCommon.setting.callingSoundFilePath = textBoxCallingSoundFile.Text;
            AppCommon.setting.appendKeywords = new String[listBoxAddKeyword.Items.Count];
            for (int i = 0; i < listBoxAddKeyword.Items.Count;i++)
            {
                AppCommon.setting.appendKeywords[i] = listBoxAddKeyword.Items[i].ToString();
            }
            
            // 共有
            AppCommon.setting.enableShare = checkBoxEnableShare.Checked;
            String linktype = comboBoxLinkType.SelectedIndex.ToString();
            AppCommon.setting.linktype = linktype;

            AppCommon.setting.enableSubFolder = checkBoxShareSubfolder.Checked;
            AppCommon.setting.shareWaitPort = sharePort;
            AppCommon.setting.downloadFolder = textBoxDownLoadFolder.Text;
            AppCommon.setting.shareFolderList = new String[listBoxShareDirList.Items.Count];
            for (int i = 0; i < listBoxShareDirList.Items.Count;i++ )
            {
                AppCommon.setting.shareFolderList[i] = listBoxShareDirList.Items[i].ToString();
            }

            //AppCommon.setting.maxDownloadNum = (int)numericMaxDL.Value;
            //AppCommon.setting.maxUploadNum = (int)numericMaxUL.Value;

            try
            {
                AppCommon.save();

                // 設定後の反映処理↓

                // 色とフォント
                if (colorChanged)
                {
                    this.parent.reflectColorEdit();//設定反映

                    MessageBox.Show("色・フォント設定変更を反映させる為には、ウィンドウを開き直す必要があります。");
                }

                // 常に前面
                this.parent.TopMost = info.topMost;

                // 半透明
                this.parent.setTransParent();

                // トレイアイコン
                this.parent.setTrayIconVisible();

                // httpd
                if (AppCommon.setting.httpdEnabled)
                {
                    Httpd.getInstance(this.parent.httpdLogStore).start();
                }
                else
                {
                    Httpd.getInstance(this.parent.httpdLogStore).stop();
                }

                // 共有
                ServerInfo[] serverInfoList = ServerManager.getLoginedServerInfoList();

                if (oldShareFlg == false)
                {
                    if (AppCommon.setting.enableShare)
                    {
                        // なし→あり
                        UploadManager.initSharedFileInfo();//ロードから             

                        UploadManager.startUploadServer();
                    }
                    else
                    {
                        // なし→なし
                    }
                }
                else if (oldShareFlg == true && AppCommon.setting.enableShare == false)
                {
                    // あり→なし
                    foreach (ServerInfo serverInfo in serverInfoList)
                    {
                        ServerManager.unShareAllFiles_(serverInfo);
                    }

                    FormTransfer.stopALL(); //止め

                    UploadManager.stopUploadServer();
                    UploadManager.unshareALL();
                } 
                else if (oldShareFlg == true && AppCommon.setting.enableShare)
                {
                    // あり→あり
                    bool chgFlg = false;
                    if (oldDownloadFolder != AppCommon.setting.downloadFolder)
                    {
                        chgFlg = true;
                    }
                    if (oldShareSubDirFlg != AppCommon.setting.enableSubFolder)
                    {
                        chgFlg = true;
                    }
                    if (chgFlg == false)
                    {
                        List<String> oldList = new List<String>(oldShareDirs);
                        List<String> newList = new List<String>(AppCommon.setting.shareFolderList);
                        oldList.Sort();
                        newList.Sort();
                        String oldListStr = "";
                        String newListStr = "";
                        foreach (String s in oldList)
                        {
                            oldListStr += s;
                        }
                        foreach (String s in newList)
                        {
                            newListStr += s;
                        }
                        if (oldListStr != newListStr)
                        {
                            chgFlg = true;
                        }
                    }
                    if (chgFlg)
                    {
                        UploadManager.initSharedFileInfo();
                    }

                    foreach (ServerInfo serverInfo in serverInfoList)
                    {
                        if (oldSharePort != AppCommon.setting.shareWaitPort)
                        {
                            //703	change data port [CLIENT] <port>
                            ServerManager.changeDataPort(serverInfo);                        
                        }
                        if (oldLinkType != AppCommon.setting.linktype)
                        {
                            //700	change link speed [CLIENT] <speed>                        
                            ServerManager.changeLinkSpeed(serverInfo);
                        }
                    }
                }

                this.parent.reflectSharedMode(); //サーバ止めた後
            }
            catch (Exception ex)
            {
                MessageBox.Show("設定保存に失敗しました：" + ex.Message+"\n"+ex.StackTrace, "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            Close();
        }

        //閉じる
        private void button2_Click(object sender, EventArgs e)
        {
            Close();
        }

        // フォント
        private void buttonFont_Click(object sender, EventArgs e)
        {
            Console.Write("Font:"+AppCommon.setting.txtFont.Name);
            FontDialog fd = new FontDialog();
            fd.Font = AppCommon.setting.txtFont;
            fd.FontMustExist = true;//エラーメッセージ表示
            fd.AllowVerticalFonts = false;//横書きフォントなし
            fd.ShowColor = false; //色は別枠
            fd.ShowEffects = false;//オプション指定不可
            fd.FixedPitchOnly = false;//固定ピッチフォント以外も表示する
            fd.AllowVectorFonts = true;//ベクタ フォントを選択できるようにする

            if (fd.ShowDialog() != DialogResult.Cancel)
            {
                buttonFont.Text = fd.Font.Name;
                buttonFont.Font = fd.Font;
                colorChanged = true;
            }
        }

        //共通のやつ
        private void buttonColorCmn_Click(object sender, EventArgs e)
        {
            Button targetBtn = (Button)sender;
            cdBox.Color = targetBtn.ForeColor;
            if (cdBox.ShowDialog() == DialogResult.OK)
            {
                targetBtn.ForeColor = cdBox.Color;
                colorChanged = true;
            }
        }

        // メッセージ
        private void buttonColor1_Click(object sender, EventArgs e)
        {
            buttonColorCmn_Click(sender, e);
            buttonColor2.ForeColor = buttonColor1.ForeColor;
        }

        // 背景
        private void buttonColor2_Click(object sender, EventArgs e)
        {
            cdBox.Color = buttonColor2.BackColor;
            if (cdBox.ShowDialog() == DialogResult.OK)
            {
                buttonColor1.BackColor = cdBox.Color;
                buttonColor2.BackColor = cdBox.Color;
                buttonColor3.BackColor = cdBox.Color;
                buttonColor4.BackColor = cdBox.Color;
                buttonColor5.BackColor = cdBox.Color;
                buttonColor6.BackColor = cdBox.Color;
                buttonColor7.BackColor = cdBox.Color;
                colorChanged = true;
            }

        }

        //フォーカス背景
        private void buttonColor8_Click(object sender, EventArgs e)
        {
            cdBox.Color = buttonColor8.BackColor;
            if (cdBox.ShowDialog() == DialogResult.OK)
            {
                buttonColor8.BackColor = cdBox.Color;
                buttonColor9.BackColor = cdBox.Color;
                colorChanged = true;
            }
        }

        //フォーカス文字色
        private void buttonColor9_Click(object sender, EventArgs e)
        {
            buttonColorCmn_Click(sender, e);
            buttonColor8.ForeColor = buttonColor9.ForeColor;
        }

        // 色セット反映(白
        private void buttonColorSet2_Click(object sender, EventArgs e)
        {
            buttonColor1.ForeColor = ColorSetting.SET1colorNormalMsg;
            buttonColor2.ForeColor = ColorSetting.SET1colorNormalMsg;
            buttonColor3.ForeColor = ColorSetting.SET1colorTime;
            buttonColor4.ForeColor = ColorSetting.SET1colorNick;
            buttonColor5.ForeColor = ColorSetting.SET1colorSysMsg;
            buttonColor6.ForeColor = ColorSetting.SET1colorJoinMsg;
            buttonColor7.ForeColor = ColorSetting.SET1colorPartMsg;
            buttonColor8.ForeColor = ColorSetting.SET1colorFocusChFore;
            buttonColor9.ForeColor = ColorSetting.SET1colorFocusChFore;

            buttonColor1.BackColor = ColorSetting.SET1colorBackground;
            buttonColor2.BackColor = ColorSetting.SET1colorBackground;
            buttonColor3.BackColor = ColorSetting.SET1colorBackground;
            buttonColor4.BackColor = ColorSetting.SET1colorBackground;
            buttonColor5.BackColor = ColorSetting.SET1colorBackground;
            buttonColor6.BackColor = ColorSetting.SET1colorBackground;
            buttonColor7.BackColor = ColorSetting.SET1colorBackground;
            buttonColor8.BackColor = ColorSetting.SET1colorFocusChBack;
            buttonColor9.BackColor = ColorSetting.SET1colorFocusChBack;
        }

        // 色セット反映(黒板
        private void buttonColorSet1_Click(object sender, EventArgs e)
        {
            buttonColor1.ForeColor = ColorSetting.SET2colorNormalMsg;
            buttonColor2.ForeColor = ColorSetting.SET2colorNormalMsg;
            buttonColor3.ForeColor = ColorSetting.SET2colorTime;
            buttonColor4.ForeColor = ColorSetting.SET2colorNick;
            buttonColor5.ForeColor = ColorSetting.SET2colorSysMsg;
            buttonColor6.ForeColor = ColorSetting.SET2colorJoinMsg;
            buttonColor7.ForeColor = ColorSetting.SET2colorPartMsg;
            buttonColor8.ForeColor = ColorSetting.SET2colorFocusChFore;
            buttonColor9.ForeColor = ColorSetting.SET2colorFocusChFore;

            buttonColor1.BackColor = ColorSetting.SET2colorBackground;
            buttonColor2.BackColor = ColorSetting.SET2colorBackground;
            buttonColor3.BackColor = ColorSetting.SET2colorBackground;
            buttonColor4.BackColor = ColorSetting.SET2colorBackground;
            buttonColor5.BackColor = ColorSetting.SET2colorBackground;
            buttonColor6.BackColor = ColorSetting.SET2colorBackground;
            buttonColor7.BackColor = ColorSetting.SET2colorBackground;
            buttonColor8.BackColor = ColorSetting.SET2colorFocusChBack;
            buttonColor9.BackColor = ColorSetting.SET2colorFocusChBack;
        }

        // HTTPdチェックボックス変更
        private void checkBoxHttpdEnabled_CheckedChanged(object sender, EventArgs e)
        {
            textBoxHttpdPort.Enabled = checkBoxHttpdEnabled.Checked;
            textBoxHttpdUserName.Enabled = checkBoxHttpdEnabled.Checked;
            textBoxHttpdPassword.Enabled = checkBoxHttpdEnabled.Checked;
            checkBoxHttpdViewport.Enabled = checkBoxHttpdEnabled.Checked;            
            textBoxHttpdUpperFrame.Enabled = checkBoxHttpdEnabled.Checked;
            labelhttpd1.Enabled = checkBoxHttpdEnabled.Checked;
            labelhttpd2.Enabled = checkBoxHttpdEnabled.Checked;
            labelhttpd3.Enabled = checkBoxHttpdEnabled.Checked;
            labelhttpd4.Enabled = checkBoxHttpdEnabled.Checked;
            labelhttpd5.Enabled = checkBoxHttpdEnabled.Checked;
            labelhttpd6.Enabled = checkBoxHttpdEnabled.Checked;
            checkBoxHttpdViewport_CheckedChanged(null, null);
        }

        // viewportチェックボックス変更
        private void checkBoxHttpdViewport_CheckedChanged(object sender, EventArgs e)
        {
            textBoxHttpdViewport.Enabled = (checkBoxHttpdEnabled.Checked && checkBoxHttpdViewport.Checked);
        }

        // ポートの数値制限
        private void textBoxHttpdPort_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (char.IsDigit(e.KeyChar) || e.KeyChar == '\b')
            {
                e.Handled = false;
                return;
            }
            e.Handled = true;
        }

        // 呼び出し小窓：有効無効設定
        private void callingSetEnableChange()
        {
            bool flg = checkBoxCalling.Checked;
            checkBoxCallByNick.Enabled = flg; 
            labelCallByNick.Enabled = flg; 
            labelAddKeywords.Enabled = flg; 
            listBoxAddKeyword.Enabled = flg; 
            textBoxAddKeyword.Enabled = flg; 
            buttonKWAdd.Enabled = flg; 
            buttonKWDell.Enabled = flg; 
        }

        // 呼び出し小窓：チェックボックス変更
        private void checkBoxCalling_CheckedChanged(object sender, EventArgs e)
        {
            callingSetEnableChange();
        }

        // 呼び出し小窓：キーワード追加
        private void buttonKWAdd_Click(object sender, EventArgs e)
        {
            if (textBoxAddKeyword.Text != null && textBoxAddKeyword.Text.Length > 0)
            {
                listBoxAddKeyword.Items.Add(textBoxAddKeyword.Text);
                textBoxAddKeyword.Text = "";
            }
        }

        // 呼び出し小窓：キーワード削除
        private void buttonKWDell_Click(object sender, EventArgs e)
        {
            int idx = listBoxAddKeyword.SelectedIndex;
            if (idx >= 0 &&idx < listBoxAddKeyword.Items.Count )
            {
                listBoxAddKeyword.Items.RemoveAt(idx);
            }
        }

        // 呼び出し小窓：呼び出し音選択ボタン
        private void buttonCallingSoundFile_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog1 = new OpenFileDialog();
            openFileDialog1.Title = "ダイアログのタイトルをココに書く";
            openFileDialog1.InitialDirectory = @".";
            openFileDialog1.FileName = "";
            openFileDialog1.Filter = "wav ファイル|*.wav";
            openFileDialog1.FilterIndex = 1;
            openFileDialog1.RestoreDirectory = true;
            openFileDialog1.Multiselect = false;
            openFileDialog1.ShowHelp = false;
            openFileDialog1.ShowReadOnly = false;
            openFileDialog1.ReadOnlyChecked = false;
            openFileDialog1.CheckFileExists = true;
            openFileDialog1.CheckPathExists = true;
            openFileDialog1.AddExtension = true;
            openFileDialog1.ValidateNames = true;

            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                textBoxCallingSoundFile.Text = openFileDialog1.FileName;
            }
            openFileDialog1.Dispose();
        }

        // 窓半透明チェック変更
        private void checkBoxTramsChange()
        {
            bool flg = checkBoxTrams.Checked;
            transVolume.Enabled = flg;
            labelTrans1.Enabled = flg;
            labelTrans2.Enabled = flg;
        }
        private void checkBoxTrams_CheckedChanged(object sender, EventArgs e)
        {
            checkBoxTramsChange();
        }

        // ダウンロードフォルダ追加
        private void buttonDownLoadFolder_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fbd = new FolderBrowserDialog();
            fbd.Description = "ダウンロードフォルダ選択";
            fbd.RootFolder = Environment.SpecialFolder.Desktop;
            fbd.ShowNewFolderButton = true;
            if (fbd.ShowDialog(this) == DialogResult.OK)
            {
                textBoxDownLoadFolder.Text = fbd.SelectedPath;
            }
        }

        // フォルダリスト追加
        private void buttonAddShareDir_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fbd = new FolderBrowserDialog();
            fbd.Description = "共有フォルダ選択";
            fbd.RootFolder = Environment.SpecialFolder.Desktop;
            fbd.ShowNewFolderButton = true;
            if (fbd.ShowDialog(this) == DialogResult.OK)
            {
                //textBoxDownLoadFolder.Text = fbd.SelectedPath;
                listBoxShareDirList.Items.Add(fbd.SelectedPath);
            }

        }

        // フォルダリスト削除
        private void buttonDelShareDir_Click(object sender, EventArgs e)
        {
            int idx = listBoxShareDirList.SelectedIndex;
            if (idx >= 0 && idx < listBoxShareDirList.Items.Count)
            {
                listBoxShareDirList.Items.RemoveAt(idx);
            }

        }

        // 共有機能：チェック変更
        private void checkBoxEnableShare_CheckedChanged(object sender, EventArgs e)
        {
            //
            shareCheckChange();
        }
        private void shareCheckChange()
        {
            bool flg = checkBoxEnableShare.Checked;
            labelShare1.Enabled = flg;
            //labelShare2.Enabled = flg;
            //labelShare3.Enabled = flg;
            labelShare4.Enabled = flg;
            labelShare5.Enabled = flg;
            labelShare6.Enabled = flg;
            textBoxSharedPort.Enabled = flg;
            //numericMaxUL.Enabled = flg;
            //numericMaxDL.Enabled = flg;
            textBoxDownLoadFolder.Enabled = flg;
            buttonDownLoadFolder.Enabled = flg;
            checkBoxShareSubfolder.Enabled = flg;
            listBoxShareDirList.Enabled = flg;
            buttonAddShareDir.Enabled = flg;
            buttonDelShareDir.Enabled = flg;
            comboBoxLinkType.Enabled = flg;
        }

        // イグノアリスト追加
        private void buttonIgnoreAdd_Click(object sender, EventArgs e)
        {
            if (this.textBoxIgnore.Text != null && this.textBoxIgnore.Text.Length > 0)
            {
                this.listBoxIgnore.Items.Add(this.textBoxIgnore.Text);
            }

            this.textBoxIgnore.Text = "";
        }

        // イグノアリスト削除
        private void buttonIgnoreDel_Click(object sender, EventArgs e)
        {
            int i = this.listBoxIgnore.SelectedIndex;
            if (i >= 0)
            {
                this.listBoxIgnore.Items.RemoveAt(i);
            }
            this.listBoxIgnore.SelectedIndex = -1;
        }

    }
}
