﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading.Tasks;
using System.Threading;

namespace MoyariClient
{
    public partial class FormPersonalBrowse : Form
    {
        private static Dictionary<String, FormPersonalBrowse> instanceMap = new Dictionary<String, FormPersonalBrowse>();

        private static String getMapKey(ServerInfo serverInfo, String nick)
        {
            return nick + "[" + serverInfo.name + "]"; 
        }

        public static void startNew(ServerInfo serverInfo, String nick)
        {
            String mapKey = getMapKey(serverInfo, nick);

            if (instanceMap.ContainsKey(mapKey) == false)
            {
                lock (instanceMap)
                {
                    if (instanceMap.ContainsKey(mapKey) == false)
                    {
                        FormPersonalBrowse it = new FormPersonalBrowse(mapKey);
                        it.Show();
                        instanceMap.Add(mapKey, it);
                    }
                }
            }
            ServerManager.browseUserRequest(serverInfo, nick);
        }

        public static void addResult(SearchResult searchResultresut)
        {
            String mapKey = getMapKey(searchResultresut.serverInfo, searchResultresut.nick);
            if (instanceMap.ContainsKey(mapKey))
            {
                try
                {
                    instanceMap[mapKey].addInvoke(searchResultresut);
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.Message + e.StackTrace);
                }
            }
        }

        public static void ackComplete(ServerInfo serverInfo, String nick)
        {
            String mapKey = getMapKey(serverInfo, nick);
            if (instanceMap.ContainsKey(mapKey))
            {
                try
                {
                    instanceMap[mapKey].completeInvoke();
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.Message + e.StackTrace);
                }
            }
        }

        public static void closeALL()
        {
            lock (instanceMap)
            {
                KeyValuePair<String,FormPersonalBrowse>[] ary = instanceMap.ToArray();
                foreach (KeyValuePair<String, FormPersonalBrowse> kvp in ary)
                {
                    try
                    {
                        kvp.Value.Close();
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine(e.Message + e.StackTrace);
                    }
                }
            }
        }

        // -------------------------------------------------
        private String mapKey;
        private int addCounter = 0;
        private List<ListViewItem> originalList = new List<ListViewItem>();
        private List<ListViewItem> tmpOrgList = new List<ListViewItem>();

        private bool loadingFlg = true;
        private long timeoutWatch = 0;

        private FilterInfo filterInfo = new FilterInfo("", true, false, true);

        // コンストラクタ
        private FormPersonalBrowse(String mapKey)
        {
            this.mapKey = mapKey;
            InitializeComponent();
            listViewResult.ListViewItemSorter = new FormSearch.SearchListViewColumnSorter();
            this.Text = mapKey;
            listViewResult.ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
            listViewResult.BackColor = AppCommon.setting.colorInfo.colorBackground;
            //textBoxFilter.ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
            //textBoxFilter.BackColor = AppCommon.setting.colorInfo.colorBackground;

            buttonFilter.Enabled = false;
            //textBoxFilter.Enabled = false;
            //checkBoxFilter.Enabled = false;
            //labelFilter.Enabled = false;
        }

        // 表示時
        private void FormPersonalBrowse_Shown(object sender, EventArgs e)
        {
            this.timeoutWatch = DateTime.Now.ToFileTime();
            timeoutWatchThread();
        }

        // タイムアウト監視スレッド
        private void timeoutWatchThread()
        {            
            Task.Factory.StartNew(() =>
            {
                while (loadingFlg)
                {
                    Thread.Sleep(AppCommon.SEARCH_TIMEOUT_WATCH_MS);
                    if (loadingFlg)
                    {
                        long timePast = DateTime.Now.ToFileTime() - this.timeoutWatch;                        
                        if (timePast > AppCommon.SEARCH_TIMEOUT_NS)
                        {   
                            completeInvoke();
                        }
                    }
                }
            });
        }

        // 閉じる時
        private void FormPersonalBrowse_FormClosing(object sender, FormClosingEventArgs e)
        {
            lock (instanceMap)
            {
                instanceMap.Remove(mapKey);
            }
        }

        // 追加
        delegate void addDelegate(SearchResult result);
        private void addInvoke(SearchResult result)
        {
            if (InvokeRequired)
            {
                this.Invoke(new addDelegate(add), new object[] { result });
            }
            else
            {
                add(result);
            }
        }
        private void add(SearchResult result)
        {
            this.timeoutWatch = DateTime.Now.ToFileTime();
            String[] line = { result.filename, result.sizeStr, result.nick, result.linktype, result.serverInfo.name };
            ListViewItem item = new ListViewItem(line);
            item.Tag = result;
            originalList.Add(item);
            tmpOrgList.Add(item);
            addCounter += 1;

            if (addCounter >= AppCommon.SEARCH_FILTER_REFLESH_STEP)
            {
                update();
                addCounter = 0;
            }
        }

        // 表示
        delegate void completeDelegate();
        public void completeInvoke()
        {
            loadingFlg = false;
            if (InvokeRequired)
            {
                this.Invoke(new completeDelegate(update), new object[] { });
            } else {
                update();
            }
        }

        // 表示更新
        private void update()
        {
            listViewResult.Items.AddRange(tmpOrgList.ToArray());
            this.Text = mapKey + "　" + originalList.Count + "件";
            tmpOrgList.Clear();
            if (loadingFlg == false)
            {
                buttonFilter.Enabled = true;
                refleshFilter();
            }
        }

        // カラムクリック
        private void listViewResult_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            ListView listView = listViewResult;
            ListViewColumnSorter _columnSorter = (ListViewColumnSorter)listView.ListViewItemSorter;

            if (e.Column == _columnSorter.SortColumn)
            {
                if (_columnSorter.Order == SortOrder.Ascending)
                {
                    _columnSorter.Order = SortOrder.Descending;
                }
                else
                {
                    _columnSorter.Order = SortOrder.Ascending;
                }
            }
            else
            {
                _columnSorter.SortColumn = e.Column;
                _columnSorter.Order = SortOrder.Ascending;
            }
            listView.Sort();        
        }

        // ダウンロード/リジューム
        private void downloadStart(SearchResult result)
        {
            DownloadManager.addDownloadTaskFromSearchWindow(result.serverInfo, result);
        }

        // 右クリック
        private void listViewSearchResult_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                ListViewItem viewitem = listViewResult.GetItemAt(e.X, e.Y);
                if (viewitem != null)
                {
                    ctxMenuStripRightClick.Show(MousePosition);
                }
            }
        }

        // 右クリック->ダウンロード
        private void toolStripMenuItemDownload_Click(object sender, EventArgs e)
        {
            if (listViewResult.SelectedItems.Count > 0)
            {
                foreach (ListViewItem viewitem in listViewResult.SelectedItems)
                {
                    downloadStart((SearchResult)viewitem.Tag);
                }
            }
        }

        // 右クリック->除外リスト
        private void toolStripMenuItemExclude_Click(object sender, EventArgs e)
        {
            List<String> excludeFileNames = new List<String>();
            foreach (ListViewItem viewitem in listViewResult.SelectedItems)
            {
                SearchResult result = (SearchResult)viewitem.Tag;
                excludeFileNames.Add(result.filename);
            }
            AppCommon.addExcludeFiles(this, excludeFileNames);
            refleshFilter();
        }

        // ダブルクリック
        private void listViewResult_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            ListViewItem viewitem = listViewResult.GetItemAt(e.X, e.Y);
            if (viewitem != null)
            {
                SearchResult tag = (SearchResult)viewitem.Tag;
                downloadStart(tag);
            }
        }

        // 絞込みボタン
        private void buttonFilter_Click(object sender, EventArgs e)
        {
            FormSearchFilter filterForm =
                new FormSearchFilter(
                    this.filterChangeCallBack,
                    this.filterInfo.filterWord,
                    this.filterInfo.withoutSharedFile,
                    this.filterInfo.withoutSharedOnlySize,
                    this.filterInfo.enableDisregardList);
            filterForm.StartPosition = FormStartPosition.CenterParent;
            filterForm.ShowDialog(this);
            filterForm.Dispose();
            //enableFilter();
        }
        // フィルタ設定ダイアログコールバック
        public void filterChangeCallBack(String filterWord, bool filterWithoutSharedFile, bool filterWithoutSharedOnlySize, bool filterEnableDisregardList)
        {
            this.filterInfo =
                new FilterInfo(
                    filterWord,
                    filterWithoutSharedFile,
                    filterWithoutSharedOnlySize,
                    filterEnableDisregardList);

            AppCommon.setting.lastFilterWithoutSharedFile = filterWithoutSharedFile;
            AppCommon.setting.lastFilterWithoutSharedOnlySize = filterWithoutSharedOnlySize;
            AppCommon.setting.lastFilterEnableDisregardList = filterEnableDisregardList;
            AppCommon.save();
            refleshFilter();
        }

        // フィルタ更新
        public void refleshFilter()
        {
            List<ListViewItem> filteredList = FormSearch.filterCheck(this.filterInfo, originalList);
            listViewResult.Items.Clear();
            listViewResult.Items.AddRange(filteredList.ToArray());

            this.Text = mapKey + "　" + filteredList.Count + "/" + originalList.Count + "件";
        }
    }

    

}
