﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;
using System.Threading.Tasks;

namespace MoyariClient
{
    public partial class FormSearch : Form
    {   
        // --------------------------- static
        // インスタンス管理
        public static List<FormSearch> instanceList = new List<FormSearch>();
        private static FormSearch searchingForm = null;
        private static FormSearchBlock blockForm = null;
        private static SearchKeyInfo searchKey = null;
        private static int searchServerNum = 0;
        private static int searchServerCounter = 0;
        private static Dictionary<String, int> searchCountMap = new Dictionary<String, int>();
        
        public const String COLNAME_FNAME = "Filename";
        public const String COLNAME_BYTES = "Bytes";
        public const String COLNAME_USERS = "User";
        public const String COLNAME_SPEED = "Speed";
        public const String COLNAME_SERVE = "Server";

        private static int lastWindowPosX = 0;
        private static int lastWindowPosY = 0;

        // 新規窓開く
        public static FormSearch openNew(MainForm parent)
        {
            if (blockForm != null) //検索中
            {
                return null;
            }

            FormSearch searchForm = new FormSearch(parent);
            searchForm.Show();
            lock (instanceList)
            {
                instanceList.Add(searchForm);
            }
            return searchForm;
        }

        // インスタンスがあればtopに
        public static void windowToTop()
        {
            List<FormSearch> instanceShowList = new List<FormSearch>();
            lock (instanceList)
            {
                foreach(FormSearch fs in instanceList){
                    instanceShowList.Add(fs);
                }                
            }

            instanceShowList.Sort(compareFormSearch);
            foreach (FormSearch fs in instanceShowList)
            {
                fs.TopMost = true;
                fs.TopMost = false;
            }
        }
        private static int compareFormSearch(FormSearch x, FormSearch y)
        {
            return (int)(x.lastActivateTimeLocal - y.lastActivateTimeLocal);
        }

        // ServerThreadからの検索結果受け - 反映対象インスタンスはこっちで判断
        public static void receiveSearchResponse(SearchResult result)
        {
            if (AppCommon.setting.ignoreList.Contains(result.nick))
            {
                Console.WriteLine("イグノア searchResponse [" + result.nick + "][" + result.filepath + "]");
                return;
            }

            if (searchingForm != null)
            {
                if (searchCountMap.ContainsKey(result.serverInfo.index))
                {
                    int n = searchCountMap[result.serverInfo.index] + 1;
                    searchCountMap.Remove(result.serverInfo.index);
                    searchCountMap.Add(result.serverInfo.index, n);
                }
                searchingForm.receiveSearchResponseInvoke(result);
            }
        }

        // ServerThreadからの検索完了受け
        public static void receiveSearchComplete(ServerInfo serverInfo)
        {
            if (searchingForm != null)
            {   
                searchingForm.writeConsoleMessageInvoke("[" + serverInfo.name + "] から検索完了通知");
                searchServerCounter += 1;

                if (searchServerCounter >= searchServerNum)
                {
                    // 全サーバーから帰ってきたら
                    String msg = "検索が完了しました ";
                    foreach (ServerInfo si in searchKey.targetServers)
                    {
                        if (searchCountMap.ContainsKey(si.index))
                        {
                            msg += " [" + si.name + "：" + searchCountMap[si.index] + "件]";
                        }
                    }
                    searchingForm.writeConsoleMessageInvoke(msg);

                    // ロック解除
                    unlockProc(false);
                }
            }
        }

        // 検索実行
        private static void searchExec(FormSearch parent, SearchKeyInfo info)
        {
            //ターゲット設定
            parent.target = true;

            lock (instanceList)
            {
                // 全窓をロック
                foreach (FormSearch f in instanceList)
                {
                    f.setEnable(false);
                }
            }

            searchingForm = parent; //検索中

            searchServerNum = info.targetServers.Length;    // カウンタ
            searchServerCounter = 0;
            searchCountMap.Clear();
            foreach (ServerInfo serverInfo in info.targetServers)
            {
                searchCountMap.Add(serverInfo.index, 0);
            }

            searchKey = info;

            // 鯖管へ検索post
            ServerManager.searchRequest(info);

            // 検索ブロックFormを開く(モードレス、子登録
            FormSearchBlock blockFormLc = new FormSearchBlock();
            blockForm = blockFormLc;
            parent.AddOwnedForm(blockFormLc);
            blockFormLc.StartPosition = FormStartPosition.Manual;
            blockFormLc.Location = new Point(parent.Location.X + (parent.Width - blockFormLc.Width) / 2, parent.Location.Y + (parent.Height - blockFormLc.Height) / 2);
            blockFormLc.Show();
        }

        // ロック解放共通処理
        public static void unlockProc(bool breakFlg)
        {
            // 検索中解除
            searchingForm = null;

            // 検索プログレスFormを閉じる＆リソース解放
            if (blockForm != null)
            {
                blockForm.closeBlockInvoke(false);
                blockForm = null;
            }

            // 全窓ロック解除
            lock (instanceList)
            {
                foreach (FormSearch f in instanceList)
                {
                    if (f.target && breakFlg)
                    {
                        f.writeConsoleMessageInvoke("中断しました　");
                    }

                    f.setEnableInvoke(true);
                }
            }
        }

        // 全部削除
        public static void closeALL()
        {
            // 検索中解除
            searchingForm = null;

            if (blockForm != null)
            {
                blockForm.closeBlockInvoke(false);
                blockForm = null;
            }

            lock (instanceList)
            {
                FormSearch[] rmAry = instanceList.ToArray();
                foreach (FormSearch f in rmAry)
                {
                    f.Close();
                }
            }
        }

        // --------------------------- ------------------------
        private bool target = false;
        private List<ListViewItem> originalResultList = new List<ListViewItem>();
        private List<ListViewItem> searchResultTmpList = new List<ListViewItem>();
        private MainForm parent = null;
        private static int indexcounter = 20;
        private int index = 0;
        public String indexedName = null;
        public ToolStripMenuItem parentTsmi = null;
        private long searchOldTime = 0;
        public long lastActivateTimeLocal = 0;
        public static long lastActivateTime = 0;
        private FilterInfo filterInfo = new FilterInfo("", true, false, true);
        //再フィルタ
        //private String filterWord = "";
        //private bool filterWithoutSharedFile = true;
        //private bool filterWithoutSharedOnlySize = false;
        //private bool filterEnableDisregardList = true;

        // コンストラクタ
        private FormSearch(MainForm parent)
        {
            if (AppCommon.setting.saveWindowPos)
            {
                this.StartPosition = FormStartPosition.Manual;
                int lasX = AppCommon.setting.searchWindowPosX;
                int lasY = AppCommon.setting.searchWindowPosY;

                if (instanceList.Count > 0)
                {
                    lasX = lastWindowPosX + 50;
                    lasY = lastWindowPosY + 50;
                }

                AppCommon.sanitNum(ref lasX, 0, Screen.PrimaryScreen.Bounds.Width , 0);
                AppCommon.sanitNum(ref lasY, 0, Screen.PrimaryScreen.Bounds.Height-100, 0);

                Console.WriteLine("lasX:" + lasX + " lasY" + lasY);
                Console.WriteLine("w:" + Screen.PrimaryScreen.Bounds.Width );
                Console.WriteLine("h:" + Screen.PrimaryScreen.Bounds.Height);

                this.Location = new Point(lasX, lasY);
                lastWindowPosX = lasX;
                lastWindowPosY = lasY;
            }

            InitializeComponent();

            if (AppCommon.setting.saveWindowPos)
            {
                int wNum = AppCommon.setting.searchWindowWidth;
                int hNum = AppCommon.setting.searchWindowHeight;

                AppCommon.sanitNum(ref wNum, 100, Screen.PrimaryScreen.Bounds.Width, 620);
                AppCommon.sanitNum(ref hNum, 50, Screen.PrimaryScreen.Bounds.Height, 300);      

                this.Width = wNum;
                this.Height = hNum;

                foreach (ColumnHeader col in listViewSearchResult.Columns)
                {
                    switch (col.Text)
                    {
                        case COLNAME_FNAME:
                            col.Width = AppCommon.setting.searchListColWidthFname;
                            break;
                        case COLNAME_BYTES:
                            col.Width = AppCommon.setting.searchListColWidthBytes;
                            break;
                        case COLNAME_USERS:
                            col.Width = AppCommon.setting.searchListColWidthUser;
                            break;
                        case COLNAME_SPEED:
                            col.Width = AppCommon.setting.searchListColWidthSpeed;
                            break;
                        case COLNAME_SERVE:
                            col.Width = AppCommon.setting.searchListColWidthServer;
                            break;
                        default:
                            Console.WriteLine("不明なカラム：" + col.Text + "/" + col.Width + "");
                            break;
                    }
                }
            }

            this.parent = parent;
            this.index = indexcounter++;
            this.indexedName = "FormSearch" + this.index.ToString();

            listViewSearchResult.ListViewItemSorter = new SearchListViewColumnSorter();

            textBoxSearchWord.BackColor = AppCommon.setting.colorInfo.colorBackground;
            textBoxSearchWord.ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
            listViewSearchResult.BackColor = AppCommon.setting.colorInfo.colorBackground;
            listViewSearchResult.ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
            textBoxConsole.BackColor = AppCommon.setting.colorInfo.colorBackground;
            textBoxConsole.ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
            //textBoxFilter.BackColor = AppCommon.setting.colorInfo.colorBackground;
            //textBoxFilter.ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;

            //buttonFilterUpdate.Enabled = false;
            buttonFilter.Enabled = false;
            buttonFileDump.Enabled = false;

            //checkBoxFilter.Checked = true;
            //textBoxFilter.Enabled = false;
            //labelFilter.Enabled = false;
            //checkBoxFilter.Enabled = false;

            this.filterInfo = new FilterInfo("", 
                AppCommon.setting.lastFilterWithoutSharedFile, 
                AppCommon.setting.lastFilterWithoutSharedOnlySize, 
                AppCommon.setting.lastFilterEnableDisregardList);
            //this.filterWithoutSharedFile = AppCommon.setting.lastFilterWithoutSharedFile;
            //this.filterWithoutSharedOnlySize = AppCommon.setting.lastFilterWithoutSharedOnlySize;
            //this.filterEnableDisregardList = AppCommon.setting.lastFilterEnableDisregardList;

#if DEBUG
            textBoxSearchWord.Text = "zip";
#endif
        }

        // 表示時
        private void FormSearch_Shown(object sender, EventArgs e)
        {
            parent.openNewSearchWindow(this);
        }

        // アクティブイベント
        private void FormSearch_Activated(object sender, EventArgs e)
        {
            lastActivateTimeLocal = DateTime.Now.ToFileTime();
            lastActivateTime = lastActivateTimeLocal;
        }

        // 閉じる時
        private void FormSearch_FormClosing(object sender, FormClosingEventArgs e)
        {
            lock (instanceList)
            {
                instanceList.Remove(this);
            }
            parent.closeSearchWindow(this);
        }

        // ボタンロック/解放
        delegate void setEnableDelegate(bool enable);
        private void setEnableInvoke(bool enable)
        {
            this.Invoke(new setEnableDelegate(setEnable), new object[] { enable });
        }
        private void setEnable(bool enable)
        {
            buttonSearchExec.Enabled = enable;
            buttonSearchExecNew.Enabled = enable;
           // buttonExSearch.Enabled = enable;            
           //buttonFilterUpdate.Enabled = enable;
            buttonFilter.Enabled = enable;
            buttonFileDump.Enabled = enable;
            textBoxSearchWord.Enabled = enable;
           // textBoxFilter.Enabled = enable;
           // checkBoxFilter.Enabled = enable;
            if (target)
            {
                if (enable)
                {
                    //Console.WriteLine("結果：" + searchResultTmpList.Count);
                    listViewSearchResult.Items.AddRange(searchResultTmpList.ToArray());
                    writeConsoleMessage("絞込み結果：" + searchResultTmpList.Count + "/" + originalResultList.Count + "件");
                    this.TopMost = true;
                    this.TopMost = false;
                    target = false;
                }
            }
            searchResultTmpList.Clear();
        }

        // 検索フォームでEnter
        private void textBoxSearchWord_keydown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == System.Windows.Forms.Keys.Enter)
            {
                e.SuppressKeyPress = true;
                searchStart();                
            }
        }

        // 検索ボタン押下
        private void buttonSearchExec_Click(object sender, EventArgs e)
        {
            searchStart();
        }

        // 検索処理開始
        private void searchStart()
        {
            if (!validation())
            {
                return;
            }
            //this.filterInfo = new FilterInfo(filterWord, filterWithoutSharedFile);
            listViewSearchResult.Items.Clear();
            searchResultTmpList.Clear();
            originalResultList.Clear();

            //parent.renameSearchWindow();
            ServerInfo[] targetServers = ServerManager.getLoginedServerInfoList();
            SearchKeyInfo key = new SearchKeyInfo(targetServers, textBoxSearchWord.Text);
            writeConsoleMessage("検索リクエストを送信します");
            writeConsoleMessage("クエリ="+key.toRequestString());
            String serverNames = "";
            foreach(ServerInfo si in targetServers){
                serverNames += "["+si.name+"]";
            }
            writeConsoleMessage("対象=" + serverNames);
            if (parentTsmi != null)
            {
                parentTsmi.Text = "検索：" + textBoxSearchWord.Text; //TODO 拡張検索での窓表示ワードはかんがえようね
            }
            
            searchExec(this, key);
        }

        // 検索語を設定して検索実行
        public void execSearch(String word)
        {
            this.textBoxSearchWord.Text = word;
            this.searchStart();
        }

        // 新規窓検索
        private void buttonSearchExecNew_Click(object sender, EventArgs e)
        {
            if (!validation())
            {
                return;
            }
            FormSearch newWnd = openNew(parent);
            if (newWnd != null)
            {
                newWnd.execSearch(textBoxSearchWord.Text);
                //newWnd.textBoxSearchWord.Text = textBoxSearchWord.Text;
                //newWnd.searchStart();
            }
        }

        // 検索実行validation
        private bool validation()
        {
            if (textBoxSearchWord.Text.Length == 0)
            {
                return false;
            }

            if (ServerManager.getLoginedServerInfoList().Length == 0)
            {
                MessageBox.Show("接続中のサーバーがありません",
                    "検索エラー",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                return false;
            }

            return true;
        }

        // フィルタ更新
        private void refleshFilter()
        {
            this.target = true;

            lock (instanceList)
            {
                // 全窓をロック
                foreach (FormSearch f in instanceList)
                {
                    f.setEnable(false);
                }
            }

            //this.filterInfo = new FilterInfo(filterWord, filterWithoutSharedFile);
            listViewSearchResult.Items.Clear();
            searchResultTmpList.Clear();

            this.filterThread();
        }

        // フィルタ更新(スレッド)
        private void filterThread()
        {
           Task.Factory.StartNew(() =>
           {
               Console.WriteLine("フィルタ更新：" + filterInfo.logStr());
               foreach (ListViewItem originalResult in originalResultList)
               {
                   SearchResult result = (SearchResult)originalResult.Tag;
                   if (filterInfo.check(result))
                   {
                       searchResultTmpList.Add(originalResult);
                   }
               }

               lock (instanceList)
               {
                   // 全窓ロック解除
                   foreach (FormSearch f in instanceList)
                   {
                       f.setEnableInvoke(true);
                   }
               }
           });
        }

//×
        //// フィルタボタン
        //private void buttonFilter_Click(object sender, EventArgs e)
        //{
        //    refleshFilter();
        //}

        //// 絞込みTextBox
        //private void textBoxFilter_KeyDown(object sender, KeyEventArgs e)
        //{
        //    if (e.KeyCode == System.Windows.Forms.Keys.Enter)
        //    {
        //        e.SuppressKeyPress = true;
        //        refleshFilter();
        //    }
        //}

        // フィルタボタン->フィルタ設定ダイアログ
        private void buttonFilter_Click(object sender, EventArgs e)
        {
            FormSearchFilter filterForm =
                new FormSearchFilter(
                    this.filterChangeCallBack, 
                    this.filterInfo.filterWord, 
                    this.filterInfo.withoutSharedFile, 
                    this.filterInfo.withoutSharedOnlySize, 
                    this.filterInfo.enableDisregardList);
            filterForm.StartPosition = FormStartPosition.CenterParent;
            filterForm.ShowDialog(this);
            filterForm.Dispose();
        }
        // フィルタ設定ダイアログコールバック
        public void filterChangeCallBack(String filterWord, bool filterWithoutSharedFile, bool filterWithoutSharedOnlySize, bool filterEnableDisregardList)
        {
            this.filterInfo =
                new FilterInfo(
                    filterWord, 
                    filterWithoutSharedFile, 
                    filterWithoutSharedOnlySize, 
                    filterEnableDisregardList);

            AppCommon.setting.lastFilterWithoutSharedFile = filterWithoutSharedFile;
            AppCommon.setting.lastFilterWithoutSharedOnlySize = filterWithoutSharedOnlySize;
            AppCommon.setting.lastFilterEnableDisregardList = filterEnableDisregardList;
            AppCommon.save();

            refleshFilter();
        }

        // ファイル出力
        private void buttonFileDump_Click(object sender, EventArgs e)
        {
            //出力先決定
            String dt = DateTime.Now.ToString("yyMMdd");
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.FileName = "["+ dt + "][検索結果][" + textBoxSearchWord.Text + "].csv";
            sfd.InitialDirectory = ".";
            sfd.Filter = "すべてのファイル(*.*)|*.*";
            sfd.FilterIndex = 0;
            sfd.Title = "出力先";
            sfd.RestoreDirectory = true;
            sfd.OverwritePrompt = true;
            sfd.CheckPathExists = true;

            if (sfd.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    StreamWriter writer = new StreamWriter(sfd.FileName, false, AppCommon.ENC_SJIS);
                    //Console.WriteLine(sfd.FileName);
                    //fileDumpThread(sfd.FileName);
                    foreach (ListViewItem viewitem in listViewSearchResult.Items)
                    {
                        SearchResult item = (SearchResult)viewitem.Tag;
                        String line = item.filename + "," + item.nick;
                        writer.WriteLine(line);
                    }

                    writer.Close();
                    MessageBox.Show(this, "検索結果ファイル出力しました", "", MessageBoxButtons.OK, MessageBoxIcon.Information);
                } catch(Exception ex){
                    MessageBox.Show(this, "ファイル作成に失敗しました：" + ex.Message, "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                
            }
        }
     
        // ユーザリストカラムクリック時
        private void listViewSearchResult_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            ListView listView = listViewSearchResult;
            ListViewColumnSorter _columnSorter = (ListViewColumnSorter)listView.ListViewItemSorter;

            if (e.Column == _columnSorter.SortColumn)
            {
                if (_columnSorter.Order == SortOrder.Ascending)
                {
                    _columnSorter.Order = SortOrder.Descending;
                }
                else
                {
                    _columnSorter.Order = SortOrder.Ascending;
                }
            }
            else
            {
                _columnSorter.SortColumn = e.Column;
                _columnSorter.Order = SortOrder.Ascending;
            }
            listView.Sort();            
        }

        // 検索結果反映 Thread合流用
        delegate void receiveSearchResponseDelegate(SearchResult result);
        public void receiveSearchResponseInvoke(SearchResult result)
        {
            this.Invoke(new receiveSearchResponseDelegate(receiveSearchResponseInner), new object[] { result });

        }
        private void receiveSearchResponseInner(SearchResult result)
        {
            String[] line = { result.filename, result.sizeStr, result.nick, result.linktype, result.serverInfo.name };
            ListViewItem item = new ListViewItem(line);
            item.Tag =  result;
            originalResultList.Add(item);
            if (filterInfo.check(result))
            {
                searchResultTmpList.Add(item);
            }

            long nowTime = DateTime.Now.ToFileTime();
            if ((nowTime - searchOldTime) > (AppCommon.SEARCH_REFLESH_RATE_NS))
            {
                int lockStartPos = textBoxConsole.Text.Length;
                textBoxConsole.SelectionStart = lockStartPos;
                if (searchResultTmpList.Count == 1)
                {
                    textBoxConsole.AppendText(System.Environment.NewLine);
                    textBoxConsole.AppendText("検索結果取得中");
                    textBoxConsole.SelectionStart = textBoxConsole.Text.Length;
                    textBoxConsole.ScrollToCaret();
                }
                textBoxConsole.AppendText("．");

                textBoxConsole.Select(lockStartPos, textBoxConsole.Text.Length);
                textBoxConsole.SelectionProtected = true;
                searchOldTime = nowTime;
            }            
        }

        //コンソールメッセージ追記
        delegate void writeConsoleMessageDelegate(String msg);
        public void writeConsoleMessageInvoke(String msg)
        {
            this.Invoke(new writeConsoleMessageDelegate(writeConsoleMessage), new object[] { msg });
        }
        private void writeConsoleMessage(String msg)
        {
            textBoxConsole.SelectionColor = AppCommon.setting.colorInfo.colorTime;
            String timeStr = DateTime.Now.ToString("HH:mm:ss") + " ";
            textBoxConsole.AppendText(System.Environment.NewLine);
            textBoxConsole.AppendText(timeStr);

            textBoxConsole.SelectionColor = AppCommon.setting.colorInfo.colorNormalMsg;
            int lockStartPos = textBoxConsole.Text.Length;
            textBoxConsole.SelectionStart = lockStartPos;
            textBoxConsole.AppendText(msg);
            textBoxConsole.Select(lockStartPos, textBoxConsole.Text.Length);
            textBoxConsole.SelectionProtected = true;

            if (textBoxConsole.Text.Length > AppCommon.PANEL_LOG_DOROP_LIMIT)
            {
                textBoxConsole.Select(0, AppCommon.PANEL_LOG_DOROP_STEP);
                textBoxConsole.SelectionProtected = false;
                textBoxConsole.SelectedText = "";
            }
            textBoxConsole.SelectionStart = textBoxConsole.Text.Length;
            textBoxConsole.ScrollToCaret();
        }

        // 右クリック
        private void listViewSearchResult_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                ListViewItem viewitem = listViewSearchResult.GetItemAt(e.X, e.Y);
                if (viewitem != null)
                {
                    //複数選択時のdisable
                    bool flg = (listViewSearchResult.SelectedItems.Count == 1);
                    toolStripMenuItemUserBrowse.Enabled = flg;

                    ctxMenuStripRightClick.Show(MousePosition);                    
                }
            }
        }

        // ダブルクリック
        private void listViewSearchResult_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            ListViewItem viewitem = listViewSearchResult.GetItemAt(e.X, e.Y);
            if (viewitem != null)
            {
                downloadStart((SearchResult)viewitem.Tag);
            }
        }

        // 右クリック->ダウンロード
        private void toolStripMenuItemDownload_Click(object sender, EventArgs e)
        {
            if (listViewSearchResult.SelectedItems.Count > 0)
            {
                foreach (ListViewItem viewitem in listViewSearchResult.SelectedItems)
                {
                    downloadStart((SearchResult)viewitem.Tag);
                }
            }
        }

        // 右クリック->ブラウズ
        private void toolStripMenuItemUserBrowse_Click(object sender, EventArgs e)
        {
            if (listViewSearchResult.SelectedItems.Count == 1)
            {
                ListViewItem viewitem = listViewSearchResult.SelectedItems[0];
                SearchResult result = (SearchResult)viewitem.Tag;
                FormPersonalBrowse.startNew(result.serverInfo, result.nick);                
            }
        }
        
        // 右クリック->除外
        private void toolStripMenuItemExclude_Click(object sender, EventArgs e)
        {
            List<String> excludeFileNames = new List<String>();
            foreach (ListViewItem viewitem in listViewSearchResult.SelectedItems)
            {
                SearchResult result = (SearchResult)viewitem.Tag;
                excludeFileNames.Add(result.filename);
            }
            AppCommon.addExcludeFiles(this, excludeFileNames);
            refleshFilter();
        }

        // ダウンロード/リジューム
        private void downloadStart(SearchResult result)
        {
            DownloadManager.addDownloadTaskFromSearchWindow(result.serverInfo, result);
        }

        // リサイズ
        private void FormSearch_ResizeEnd(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Normal && AppCommon.setting.saveWindowPos)
            {
                AppCommon.setting.searchWindowWidth = this.Width;
                AppCommon.setting.searchWindowHeight = this.Height;
                AppCommon.save();
            }
        }

        // 移動時
        private void FormSearch_Move(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Normal && AppCommon.setting.saveWindowPos)
            {
                AppCommon.setting.searchWindowPosX = this.Location.X;
                AppCommon.setting.searchWindowPosY = this.Location.Y;
                AppCommon.save();
            }
            lastWindowPosX = this.Location.X;
            lastWindowPosY = this.Location.Y;
        }

        // カラム長変更時
        private void listViewSearchResult_ColumnWidthChanged(object sender, ColumnWidthChangedEventArgs e)
        {
            if (AppCommon.setting.saveWindowPos)
            {
                foreach (ColumnHeader col in listViewSearchResult.Columns)
                {
                    switch (col.Text)
                    {
                        case COLNAME_FNAME:
                            AppCommon.setting.searchListColWidthFname = col.Width;
                            break;
                        case COLNAME_BYTES:
                            AppCommon.setting.searchListColWidthBytes = col.Width;
                            break;
                        case COLNAME_USERS:
                            AppCommon.setting.searchListColWidthUser = col.Width;
                            break;
                        case COLNAME_SPEED:
                            AppCommon.setting.searchListColWidthSpeed = col.Width;
                            break;
                        case COLNAME_SERVE:
                            AppCommon.setting.searchListColWidthServer = col.Width;
                            break;
                        default:
                            Console.WriteLine("不明なカラム："+col.Text+"/"+col.Width+"");
                            break;
                    }
                }
                AppCommon.save();
            }
        }

        // 検索結果ソート
        public class SearchListViewColumnSorter : ListViewColumnSorter
        {
            protected override int compareColumn(ListViewItem listViewX, ListViewItem listViewY)
            {
                if (COLNAME_BYTES == listViewX.ListView.Columns[_columnToSort].Text)
                {
                    String bytesTxtX = listViewX.SubItems[_columnToSort].Text;
                    String bytesTxtY = listViewY.SubItems[_columnToSort].Text;

                    bytesTxtX = bytesTxtX.Replace(",", "");
                    bytesTxtY = bytesTxtY.Replace(",", "");

                    try
                    {
                        int valX = int.Parse(bytesTxtX);
                        int valY = int.Parse(bytesTxtY);

                        return _objectCompare.Compare(valX, valY);
                    }catch(Exception){
                    }
                }
                return base.compareColumn(listViewX, listViewY);                
            }
        }

        // 検索結果のフィルタリスト作成(共用
        public static List<ListViewItem> filterCheck(FilterInfo filterInfo, List<ListViewItem> srcList)
        {
            List<ListViewItem> result = new List<ListViewItem>();            
            foreach (ListViewItem item in srcList)
            {
                SearchResult sr = (SearchResult)item.Tag;
                if (filterInfo.check(sr))
                {
                    result.Add(item);
                }
            }
            return result;
        }
    }

    // フィルタ情報
    public class FilterInfo
    {
        public List<String> filenameFilterList = new List<String>();

        public String filterWord = "";
        public bool withoutSharedFile = true;
        public bool withoutSharedOnlySize = false;
        public bool enableDisregardList = true;

        //public bool withoutSharedFile = true;
        public FilterInfo(String filterWord, bool withoutSharedFile, bool withoutSharedOnlySize, bool enableDisregardList)
        {
            if (filterWord != null)
            {
                String[] parts = filterWord.Split(' ');
                foreach (String s in parts)
                {
                    this.filenameFilterList.Add(s.ToUpper());
                }
            }
            this.filterWord = filterWord;
            this.withoutSharedFile = withoutSharedFile;
            this.withoutSharedOnlySize = withoutSharedOnlySize;
            this.enableDisregardList = enableDisregardList;
        }

        public bool check(SearchResult result)
        {
            if (checkFileNameFilter(result.filename) == false)
            {
                return false;
            }
            if (withoutSharedFile)
            {
                if (withoutSharedOnlySize)
                {
                    if (UploadManager.checkSharedOnlySize(result.sizeNum))
                    {
                        return false;
                    }
                }
                else
                {
                    if (UploadManager.checkShared(result.filename, result.sizeNum))
                    {
                        return false;
                    }
                }
            }
            if (enableDisregardList)
            {
                if (AppCommon.excludeFileList.fileNames.Contains(result.filename))
                {
                    return false;
                }
            }
            return true;
        }

        private bool checkFileNameFilter(String fileName){
            foreach (String filterStr in filenameFilterList)
            {
                if (fileName.ToUpper().Contains(filterStr) == false)
                {
                    return false;
                }
            }
            return true;
        }

        public String logStr()
        {
            String result = "";
            foreach (String filterStr in filenameFilterList)
            {
                result += "[" + filterStr + "]";
            }
            result += withoutSharedFile;
            return result;
        }
    }

}


//てんぷれ
//delegate void **Delegate(ServerInfo serverInfo);
//public void **Invoke(ServerInfo serverInfo)
//{
//    this.Invoke(new **Delegate(**), new object[] { serverInfo });
//}
//private void **(ServerInfo serverInfo)
//{
//}