﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MoyariClient
{
    public partial class FormServerEdit : Form
    {
        private MainForm parent;
        private ServerInfo editInfo = null;
        private bool editMode = false;

        public FormServerEdit(MainForm parent, ServerInfo editInfo)
        {
            InitializeComponent();
            this.parent = parent;
            this.editInfo = editInfo;
            editMode = (editInfo != null);
            if (editMode)
            {
                textBoxName.Text = editInfo.name;
                textBoxHost.Text = editInfo.host;
                textBoxPort.Text = editInfo.port.ToString();
                textBoxUser.Text = editInfo.user;
                textBoxPass.Text = editInfo.pass;
                checkBoxConnect.Checked = editInfo.connectOnStart;

                this.Text = "サーバー情報：更新";
                buttonOk.Text = "更新";
            }
            else
            {
                this.Text = "サーバー情報：追加";
                buttonOk.Text = "追加";
            }
        }
        
        private void buttonOk_Click(object sender, EventArgs e)
        {
            String name = textBoxName.Text;
            String host = textBoxHost.Text;
            String port = textBoxPort.Text;
            String user = textBoxUser.Text;
            String pass = textBoxPass.Text;
            //checkBoxConnect.Checked;

            if (name.Length == 0 || host.Length == 0 || port.Length == 0 || user.Length == 0 || pass.Length == 0)
            {
                MessageBox.Show("未設定の項目があります。","エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            int portnum = -1;
            try
            {
                portnum = int.Parse(port);
            }
            catch (FormatException)
            {
                MessageBox.Show("ポートの設定値が不正です。", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (portnum < 1 || portnum > 65535)
            {
                MessageBox.Show("ポートの設定値が不正です。", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (checkErrorChar(textBoxName.Text))
            {
                MessageBox.Show("名前\n禁則文字が含まれています\n\\/:*?\"<>| (半角スペースも)", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (checkErrorChar(textBoxHost.Text))
            {
                MessageBox.Show("ホスト\n禁則文字が含まれています\n\\/:*?\"<>| (半角スペースも)", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (checkErrorChar(textBoxUser.Text))
            {
                MessageBox.Show("ユーザ\n禁則文字が含まれています\n\\/:*?\"<>| (半角スペースも)", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (textBoxUser.Text.Length > 18)
            {
                MessageBox.Show("ユーザ名は18文字以内で", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (editMode == false)
            {
                editInfo = new ServerInfo();
            }
            editInfo.name = textBoxName.Text;
            editInfo.host = textBoxHost.Text;
            editInfo.port = portnum;
            editInfo.user = textBoxUser.Text;
            editInfo.pass = textBoxPass.Text;
            editInfo.connectOnStart = checkBoxConnect.Checked;
            parent.formServerEditCallback(editInfo, editMode);
            Close();
        }

        private bool checkErrorChar(String str)
        {
            char[] errChars = AppCommon.ERROR_CHARS.ToCharArray();

            foreach (char c in errChars)
            {
                if (str.IndexOf(c) > 0)
                {
                    return true;
                }
            }
            return false;
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void textBoxPort_KeyPress(object sender, KeyPressEventArgs e)
        {
            //×
            //if (char.IsDigit(e.KeyChar) || e.KeyChar == '\b')
            //{
            //    e.Handled = false;
            //    return;
            //}
            //e.Handled = true;
        }
    }
}
