﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Threading;
using System.Threading.Tasks;
using System.Globalization;
using System.Drawing;
using System.Security;
using System.Text.RegularExpressions;
using System.Windows.Forms;


namespace MoyariClient
{
    class Httpd
    {

        private const String CHGCH_KEY = "chgch_";
        private const char CHGCH_SPLIT = '_';
        private const String AUTH_KEY = "AUTHORIZATION";
        private const String VIEW_ROW = "VIEW_ROW";
        private const String INPUT_URL = "/INPUT";
        private const String CHAT_URL = "/CHAT";
        //private const String CSS_FILENAME = "httpd.css";
        private const String CSS_FILENAME = "httpd.css";
        private const String CSS_URL = "/HTTPD.CSS";
        private const String KEY_SEND_TEXT = "SEND_TXT";

        public static Httpd instance = null;
        private static Object lockkey = new Object();

        private Dictionary<String, Dictionary<String, List<HttpdAddMsgLog>>> logstore;

        private bool alive = false;
        private TcpListener listener = null;
        //private CancellationTokenSource serverThreadCts = null;

        //セッションキー IP:key
        private Dictionary<String, HttpSessionKey> sessionKeyMap = new Dictionary<String, HttpSessionKey>();

        // コンストラクタ
        private Httpd(Dictionary<String, Dictionary<String, List<HttpdAddMsgLog>>> httpdLogStore)
        {
            this.logstore = httpdLogStore;
        }

        // インスタンス取得
        public static Httpd getInstance(Dictionary<String, Dictionary<String, List<HttpdAddMsgLog>>> httpdLogStore)
        {
            lock (lockkey)
            {
                if (instance == null)
                {
                    instance = new Httpd(httpdLogStore);
                }
            }
            return instance;
        }

        // スタート
        public void start()
        {
            Console.WriteLine("Httpd.start");
            if (alive)
            {
                Console.WriteLine("HTTPD：実行中");
                return;
            }

            Console.WriteLine("HTTPD：サーバースレッドを開始します");
            int port = AppCommon.setting.httpdPort;

            //serverThreadCts = new CancellationTokenSource();
            alive = true;
            
            Task keepaliveThread = Task.Factory.StartNew(() =>
            {
                while (alive)
                {
                    try
                    {
                        listener = new TcpListener(IPAddress.Any, port);
                        listener.Start();
                        Console.WriteLine("HTTPD：サーバスレッド　Port{0}のListenを開始しました。", port);

                        while (alive)
                        {
                            TcpClient tcpClient = listener.AcceptTcpClient();
                            procClient(tcpClient);
                        }

                        listener.Stop();
                        Console.WriteLine("HTTPD：サーバスレッド　Listenerを閉じました。");
                    }
                    catch (Exception e)
                    {
                        if (alive)
                        {
                            AppCommon.dumpException(e, "HTTPD：サーバスレッドで例外が発生しました");
                            //stop();
                        }
                    }
                    Thread.Sleep(10000);
                }
            });
        }

        // クライアントスレッドの処理
        private void procClient(TcpClient tcpClient){
            Task clientThread = Task.Factory.StartNew(() =>
            {
                bool clientLive = true;
                try
                {
                    String ipAddr = ((IPEndPoint)tcpClient.Client.RemoteEndPoint).Address.ToString();
                    Console.WriteLine("HTTPD：クライアントが接続しました。[" + ipAddr + "]");
                    NetworkStream networkStream = tcpClient.GetStream();

                    //クライアントから送られたデータを受信する
                    MemoryStream ms = new MemoryStream();
                    byte[] resBytes = new byte[256];
                    int resSize;
                    do
                    {
                        //データの一部を受信する
                        resSize = networkStream.Read(resBytes, 0, resBytes.Length);
                        ms.Write(resBytes, 0, resSize);
                    } while (networkStream.DataAvailable);

                    //受信したデータを文字列に変換
                    String reqMsg = AppCommon.ENC_SJIS.GetString(ms.ToArray());
                    ms.Close();

                    RequestData reqData = new RequestData(reqMsg);

                    String sendMsg = null;
                    
                    String headTop = "";
                    headTop = "HTTP/1.0 401 Autorization Required\nWWW-Authenticate: Basic realm=auth\n";
                    if (reqData.paramMap.ContainsKey(AUTH_KEY))
                    {
                        String authStr = reqData.paramMap[AUTH_KEY];
                        sendMsg = "auth ng 2";
                        if (authStr.IndexOf(":") > 0)
                        {
                            String[] authParts = authStr.Split(':');
                            if (authParts.Length >= 2)
                            {
                                if (AppCommon.setting.httpdUserName.CompareTo(authParts[0])==0 && AppCommon.setting.httpdPassword.CompareTo(authParts[1]) == 0)
                                {
                                    headTop = "HTTP/1.0 200 OK\n";

                                    if ("POST".CompareTo(reqData.command) == 0)
                                    {
                                        //POST
                                        //-----POST処理部
                                        //chgchがある
                                        //CHGCH_KEY
                                        // →移転先keyを取得
                                        //  →セッションにカレントキーを載せる
                                        bool chgCh = false;
                                        foreach (String paramKey in reqData.postDataMap.Keys)
                                        {
                                            if (paramKey.StartsWith(CHGCH_KEY))
                                            {
                                                String[] spk = paramKey.Replace(CHGCH_KEY, "").Split(CHGCH_SPLIT);
                                                if (spk.Length > 1)
                                                {
                                                    String skey = spk[0];
                                                    String ckey = spk[1];
                                                    ServerInfo si = ServerManager.getServerInfoByIndex(skey);
                                                    ChannelInfo ch = ServerManager.getChInfoByIndex(skey, ckey);
                                                    if (si != null && ch != null)
                                                    {
                                                        HttpSessionKey sessKey = new HttpSessionKey(skey,ckey,si,ch);
                                                        sessionKeyMap.Remove(ipAddr);
                                                        sessionKeyMap.Add(ipAddr, sessKey);
                                                        chgCh = true;
                                                    }
                                                }
                                            }
                                        }
                                        
                                        //chgchがない
                                        Console.WriteLine("chgCh:" + chgCh);
                                        if (chgCh == false) 
                                        {
                                            if (reqData.postDataMap.ContainsKey(KEY_SEND_TEXT))
                                            {
                                                String sendTxt = reqData.postDataMap[KEY_SEND_TEXT];
                                                if (sendTxt == null || sendTxt.Length == 0)
                                                {
                                                    Console.WriteLine("send_txtに内容がない");
                                                    //  →リロード　なんもなし
                                                }
                                                else
                                                {
                                                    // send_txtに内容がある
                                                    if (sessionKeyMap.ContainsKey(ipAddr))
                                                    {
                                                        HttpSessionKey sessKey = sessionKeyMap[ipAddr];
                                                        ServerManager.c2sPublicMessage(sessKey.serverInfo, sessKey.chInfo, sendTxt);
                                                        //  →書き込み。1秒sleep?
                                                        Thread.Sleep(1000);
                                                    }
                                                    else
                                                    {
                                                        Console.WriteLine("ipAddrに対応するセッションがない:" + ipAddr);
                                                    }
                                                }
                                            }
                                            else
                                            {
                                                Console.WriteLine("KEY_SEND_TEXTがない");

                                            }                                            
                                        }

                                        // VIEW_ROW
                                        if (reqData.postDataMap.ContainsKey(VIEW_ROW))
                                        {
                                            String rowStr = reqData.postDataMap[VIEW_ROW];
                                            //Int16.
                                            int rowNum = 5;
                                            try
                                            {
                                                rowNum = int.Parse(rowStr);
                                                if (sessionKeyMap.ContainsKey(ipAddr))
                                                {
                                                    HttpSessionKey sessKey = sessionKeyMap[ipAddr];
                                                    sessKey.reloadNum = rowNum;
                                                }
                                            }
                                            catch (FormatException)
                                            {
                                                Console.WriteLine("rowStr設定値不正:" + rowStr);
                                            }
                                        }
                                        else
                                        {
                                            Console.WriteLine("");
                                        }

                                    }
                                    Console.WriteLine("<<" + reqData.requrl + ">>");


                                    sendMsg = getProcess(reqData, ipAddr);
                                }
                            }
                        }
                    }
                    else
                    {   
                        headTop = "HTTP/1.0 401 Autorization Required\nWWW-Authenticate: Basic realm=auth\n";
                        sendMsg = "auth ng 1";
                    }

                    byte[] sendMsgBytes = AppCommon.ENC_SJIS.GetBytes(sendMsg);
                    String head = headTop + "Content-Length:" + sendMsgBytes.Length + "\n\n";
                    byte[] headBytes = AppCommon.ENC_SJIS.GetBytes(head);
                    
                    //データを送信する
                    networkStream.Write(headBytes, 0, headBytes.Length);
                    networkStream.Write(sendMsgBytes, 0, sendMsgBytes.Length);
                    networkStream.Flush();

                    clientLive = false;
                    //閉じる
                    networkStream.Close();
                    tcpClient.Close();
                    Console.WriteLine("HTTPD：クライアントを切断しました [" + ipAddr + "]");
                }
                catch (Exception e)
                {
                    if (clientLive)
                    {
                        Console.WriteLine("HTTPD：クライアントスレッドで例外が発生しました [" + e.Message + "]");
                    }
                }
            });
        }

        // heml本体作成処理
        private String getProcess(RequestData reqData,String ipAddr)
        {
             //ログの木からセッションキーリストを取る
            List<HttpSessionKey> sessionKeyList = getKeyListFromLogStore();
            if(sessionKeyList.Count == 0){
                return "閲覧可能なチャンネルがありません";
            }
            //セッション確認
            // キーがない場合はログの木から適当にセッションキーを取る
            //ipAddr
            HttpSessionKey session = null;
            if (sessionKeyMap.ContainsKey(ipAddr))
            {
                session = sessionKeyMap[ipAddr];
            }
            else
            {
                session = sessionKeyList[0];
                sessionKeyMap.Add(ipAddr, session);
            }

            //URLで分岐
            if (INPUT_URL.CompareTo(reqData.requrl) == 0)
            {
                // →post
                //  キーリストから不一致chリスト
                return getInputPage(session, sessionKeyList);
            }
            else if (CHAT_URL.CompareTo(reqData.requrl) == 0)
            {
                // →chat
                //  ログ木からログ取って書く
                return getChatPage(session);
            }
            else if (CSS_URL.CompareTo(reqData.requrl) == 0)
            {
                // css
                return getCssPage(session);
            }
            else
            {
                // →top
                //  固定
                return getTopPage();
            }
        }

        //トップページ取得
        private String getTopPage()
        {
            String result = "";
            result += "<html lang=\"ja-JP\">\n";
            result += "<head>\n";
            result += "<title>chat</title>\n";
            result += "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=Shift_JIS\">\n";
            result += "<meta http-equiv=\"Content-Style-Type\" content=\"text/css\">\n";
            result += "</head>\n";
            result += "<frameset rows=\"" + AppCommon.setting.httpdUpperFrameSize + ",*\" frameborder=0>\n";
            result += "<frame src=\"" + INPUT_URL + "\" name=\"input\">\n";
            result += "<frame src=\"" + CHAT_URL + "\" name=\"chat\">\n";
            result += "</frameset>\n";
            result += "<noframes>このページはフレーム対応のブラウザでご覧ください。</noframes>\n";
            result += "</frameset>\n";
            result += "</body>\n";
            result += "</html>\n";
            return result;
        }

        // 入力ch取得
        private String getInputPage(HttpSessionKey session, List<HttpSessionKey> sessionKeyList)
        {
            String result = ""; 
            result += "<html lang=\"ja-JP\">\n";
            result += "<head>\n";
            result += "<title>chat.</title>\n";
            result += "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=Shift_JIS\">\n";
            result += "<meta http-equiv=\"Content-Style-Type\" content=\"text/css\">\n";
            if (AppCommon.setting.httpdViewPort)
            {
                result += "<meta name=\"viewport\" content=\"" + AppCommon.setting.httpdViewPortValue + "\">";
            }
            result += "<link rel=\"stylesheet\" href=\""+CSS_FILENAME+"\" type=\"text/css\" />\n";
            result += "</head>\n";

            result += "<body class=\"body\" scroll=no>\n";
            result += "<form name=\"chatinput\" method=\"POST\" action=\"/\" target=\"_top\">\n";
            result += "<input class=\"input_send_txt\" type=\"text\" name=\"" + KEY_SEND_TEXT + "\" tabindex=1>\n"; //size=50 
            result += "<input class=\"submit_sendreload\" type=\"submit\" name=\"send_btn\" value=\"送信/リロード\" tabindex=\"2\" onClick=\"disabled = true;chatinput.submit();\">\n";
            result += "<input class=\"input_view_row\" type=\"text\" name=\"" + VIEW_ROW + "\" value=\"" + session.reloadNum + "\" tabindex=4>行\n"; 

            //鯖×ch
            Console.WriteLine("sessionKeyList数:"+sessionKeyList.Count);
            foreach (HttpSessionKey hkl in sessionKeyList)
            {
                if (session.svrIndex.CompareTo(hkl.svrIndex) != 0 || session.chIndex.CompareTo(hkl.chIndex) != 0)
                {
                    if (hkl.chInfo.joined)
                    {
                        String chgChKey = CHGCH_KEY + hkl.svrIndex + CHGCH_SPLIT + hkl.chIndex;
                        String chName = hkl.serverInfo.name + "/" + hkl.chInfo.name;
                        result += "<input class=\"submit_changech\" type=\"submit\" name=\"" + chgChKey + "\" value=\"" + chName + "\">\n";
                    }
                    else
                    {
                        Console.WriteLine("joinしていないチャンネルは回避：" + hkl.serverInfo.name + "/" + hkl.chInfo.name);
                    }
                }
                else
                {
                    Console.WriteLine("一致しているため回避：" + hkl.serverInfo.name + "/" + hkl.chInfo.name  );
                }
            }

            result += "</form>\n";
            result += "<script type=\"text/javascript\">\n";
            result += "<!--\n";
            result += "document.chatinput.send_txt.focus();\n";
            result += "// -->\n";
            result += "</script>\n";
            result += "</body>\n";
            result += "</html>\n";
            return result;
        }

        // chatチャンネル取得
        private String getChatPage(HttpSessionKey session)
        {
            List<HttpdAddMsgLog> logList = null;
            if (logstore.ContainsKey(session.svrIndex))
            {
                if (logstore[session.svrIndex].ContainsKey(session.chIndex))
                {
                    logList = logstore[session.svrIndex][session.chIndex];
                }
                else
                {
                    return "対象のチャンネルログがありません " + session.chInfo.name;
                }
            }
            else
            {
                return "対象のチャンネルログがありません " + session.serverInfo.name;
            }

            String result = "";
            result += "<html lang=\"ja-JP\">\n";
            result += "<head>\n";
            result += "<title>chat</title>\n";
            result += "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=Shift_JIS\">\n";
            result += "<meta http-equiv=\"Refresh\" content=\"30\">\n";
            result += "<meta http-equiv=\"Content-Style-Type\" content=\"text/css\">\n";
            if (AppCommon.setting.httpdViewPort)
            {
                result += "<meta name=\"viewport\" content=\"" + AppCommon.setting.httpdViewPortValue + "\">";
            }
            result += "<link rel=\"stylesheet\" href=\""+CSS_FILENAME+"\" type=\"text/css\" />\n";
            result += "</head>\n";
            result += "<body class=\"body\">\n";
            result += "<tt>\n";
            
            for (int count = 0; count < session.reloadNum && count < logList.Count; count++ )
            {
                HttpdAddMsgLog log = logList[logList.Count - 1 - count];

                result += "<font class=\"msg_time\">" + log.time + "</font>&nbsp;";
                String msg1 = log.message1;
                String msg2 = log.message2;
                if(msg1 != null){
                    msg1 = SecurityElement.Escape(msg1);
                }
                if (msg2 != null)
                {
                    msg2 = SecurityElement.Escape(msg2);
                    msg2 = url2anchor(msg2);
                }
                switch (log.type)
                {
                    
                    case HttpdAddMsgType.Join:
                        result += "<font class=\"msg_join\">" + msg1 + "</font><br>\n";
                        break;
                    case HttpdAddMsgType.Part:
                        result += "<font class=\"msg_part\">" + msg1 + "</font><br>\n";
                        break;
                    case HttpdAddMsgType.Pub:
                        result += "<font class=\"msg_name\">" + msg1 + "&nbsp;&gt;&nbsp;</font><font class=\"msg_talk\">" + msg2 + "</font><br>\n";
                        break;
                    case HttpdAddMsgType.Sys:
                    default:
                        result += "<font class=\"msg_system\">" + msg1 +"</font><br>\n";
                        break;
                }
            }
            result += "</tt>\n";
            result += "</body>\n";
            result += "</html>\n";

            return result;
        }

        // urlをaタグに
        private String url2anchor(String msg)
        {   
            String result = msg;            
            MatchCollection mc =
                Regex.Matches(
                msg, AppCommon.URL_PATTERN);
            foreach (System.Text.RegularExpressions.Match m in mc)
            {
                String repStr = "<a class=\"msg_link\" target=\"_blank\" href=\"" + m.Value + "\">" + m.Value + "</a>";
                result = result.Replace(m.Value, repStr);
            }
            
            return result;
        }

        // cssページ取得
        private String getCssPage(HttpSessionKey session)
        {
            String cssBody = null;
            //css探す
            if (File.Exists(CSS_FILENAME))
            {
                Console.WriteLine("CSSファイル返却");
                // →ある：ファイル返却            
                try
                {
                    StreamReader reader = new StreamReader(CSS_FILENAME, AppCommon.ENC_SJIS);
                    cssBody = reader.ReadToEnd();
                    reader.Close();
                }catch(Exception e){
                Console.WriteLine("CSSファイルロードに失敗："+e.Message);
                }
            }

            // →ないorエラー：
            if (cssBody == null)
            {
                Console.WriteLine("CSSファイル出力");
                cssBody = getCssDefault();
                try
                {
                    StreamWriter writer = new StreamWriter(CSS_FILENAME, true, AppCommon.ENC_SJIS);
                    writer.Write(cssBody);
                    writer.Close();
                }
                catch (Exception e)
                {
                    Console.WriteLine("CSSファイル出力に失敗：" + e.Message);
                }
            }
            return cssBody;
        }

        // デフォルトのcss文字列
        private String getCssDefault()
        {
            String colorTxt = ColorTranslator.ToHtml(AppCommon.setting.colorInfo.colorNormalMsg);
            String colorBack = ColorTranslator.ToHtml(AppCommon.setting.colorInfo.colorBackground);
            String colorJoin = ColorTranslator.ToHtml(AppCommon.setting.colorInfo.colorJoinMsg);
            String colorPart = ColorTranslator.ToHtml(AppCommon.setting.colorInfo.colorPartMsg);
            String colorTime = ColorTranslator.ToHtml(AppCommon.setting.colorInfo.colorTime);
            String colorSys = ColorTranslator.ToHtml(AppCommon.setting.colorInfo.colorSysMsg);
            String colorNick = ColorTranslator.ToHtml(AppCommon.setting.colorInfo.colorNick);

            String result = "";
            result += "\r\n";
            result += "/* 投稿フォーム */\r\n";
            result += ".input_send_txt{\r\n";
            result += "	width:300;\r\n";
            result += "	/*\r\n";
            result += "	height:50px;\r\n";
            result += "	*/\r\n";
            result += "}\r\n";
            result += "\r\n";
            result += "/* 行数指定 */\r\n";
            result += ".input_view_row{\r\n";
            result += "	width:30;\r\n";
            result += "	/*\r\n";
            result += "	height:50px;\r\n";
            result += "	*/\r\n";
            result += "}\r\n";
            result += "\r\n";
            result += "/* 送信/リロードボタン */\r\n";
            result += ".submit_sendreload{\r\n";
            result += "	/*\r\n";
            result += "	width:100px;\r\n";
            result += "	height:50px;\r\n";
            result += "	*/\r\n";
            result += "}\r\n";
            result += "\r\n";
            result += "/* チャンネル変更ボタン */\r\n";
            result += ".submit_changech{\r\n";
            result += "	/*\r\n";
            result += "	width:100px;\r\n";
            result += "	height:50px;\r\n";
            result += "	*/\r\n";
            result += "}\r\n";
            result += "\r\n";
            result += "/* bodyタグ */\r\n";
            result += ".body {\r\n";
            result += "	background:" + colorBack + ";\r\n";
            result += "	font-family:ＭＳ ゴシック;\r\n";
            result += "	font-size: 12px;\r\n";
            result += "	color: " + colorTxt + ";\r\n";
            result += "}\r\n";
            result += "\r\n";
            result += "/* ログ：時刻 */\r\n";
            result += ".msg_time{\r\n";
            result += "	color: " + colorTime + ";\r\n";
            result += "}\r\n";
            result += "\r\n";
            result += "/* ログ：入室 */\r\n";
            result += ".msg_join{\r\n";
            result += "	color: " + colorJoin + ";\r\n";
            result += "}\r\n";
            result += "\r\n";
            result += "/* ログ：退室 */\r\n";
            result += ".msg_part{\r\n";
            result += "	color: " + colorPart + ";\r\n";
            result += "}\r\n";
            result += "\r\n";
            result += "/* ログ：名前 */\r\n";
            result += ".msg_name{\r\n";
            result += "	color: " + colorNick + ";\r\n";
            result += "}\r\n";
            result += "\r\n";
            result += "/* ログ：発言 */\r\n";
            result += ".msg_talk{\r\n";
            result += "	color: " + colorTxt + ";\r\n";
            result += "}\r\n";
            result += "\r\n";
            result += "/* ログ：システムメッセージ */\r\n";
            result += ".msg_system{\r\n";
            result += "	color: " + colorSys + ";\r\n";
            result += "}\r\n";
            result += "\r\n";
            result += "/* ログ：リンク */\r\n";
            result += ".msg_link{\r\n";
            result += "}\r\n";
            result += "\r\n";
            return result;
        }

        //　セッションキークラス
        class HttpSessionKey
        {
            public String svrIndex;
            public String chIndex;
            public ServerInfo serverInfo;
            public ChannelInfo chInfo;
            public int reloadNum = 30;

            public HttpSessionKey(String svrIndex, String chIndex, ServerInfo svrName, ChannelInfo chName)
            {
                this.svrIndex = svrIndex;
                this.chIndex = chIndex;
                this.serverInfo = svrName;
                this.chInfo = chName;
            }
        }

        private List<HttpSessionKey> getKeyListFromLogStore()
        {
            List<HttpSessionKey> result = new List<HttpSessionKey>();
            foreach (String skey in logstore.Keys)
            {
                //Console.WriteLine("skey:" + skey);
                ServerInfo server = ServerManager.getServerInfoByIndex(skey);
                if (server != null)
                {
                    foreach (String ckey in logstore[skey].Keys)
                    {
                        //Console.WriteLine("ckey:" + ckey);
                        ChannelInfo ch = ServerManager.getChInfoByIndex(skey, ckey);
                        if (ch != null)
                        {
                            HttpSessionKey sessionKey = new HttpSessionKey(skey, ckey, server, ch);
                            result.Add(sessionKey);
                        }
                        else
                        {
                            Console.WriteLine("chInfoが取れない:" + skey+":"+ckey);
                        }
                    }
                }
                else
                {
                    Console.WriteLine("serverInfoが取れない:" + skey);
                }
            }
            return result;
        }
        class RequestData
        {
            //public String head;
            public String command = "";
            public String requrl = "";
            public Dictionary<String,String> paramMap = new Dictionary<String,String>();
            public Dictionary<String, String> postDataMap = new Dictionary<String, String>();
            public RequestData(String reqStr)
            {
                reqStr = reqStr.Replace("\r", "");
                String[] parts = reqStr.Split('\n');
                if (parts.Length > 1)
                {
                    if (parts[0].IndexOf(" ") > 0)
                    {
                        String[] reqParts = parts[0].Split(' ');
                        if (reqParts.Length > 1)
                        {
                            command = reqParts[0].ToUpper();
                            requrl = reqParts[1].ToUpper();
                            Console.WriteLine("(" + command + ")(" + requrl + ")");
                        }
                    }                    
                    
                }
                for (int i = 1; i < parts.Length; i++)
                {
                    int idx = parts[i].IndexOf(":");
                    if (idx > 0)
                    {
                        String k = parts[i].Substring(0, idx);
                        k = k.ToUpper();
                        String v = parts[i].Substring(idx + 1);
                        if (v.StartsWith(" "))
                        {
                            v = v.Substring(1);
                        }
                        if (k.CompareTo(AUTH_KEY) == 0)
                        {
                            if (v.Length > 8 && ("BASIC ".CompareTo(v.Substring(0, 6).ToUpper()) == 0))
                            {
                                v = v.Substring(6);
                            }
                            byte[] b = Convert.FromBase64String(v);
                            v = AppCommon.ENC_SJIS.GetString(b);                            
                        }

                        Console.WriteLine("p[" + k + "=" + v + "]");
                        paramMap.Add(k, v);
                    }
                    else
                    {
                        if (parts[i].Length > 0)
                        {
                            String[] tokenParts = {parts[i]};
                            if(parts[i].IndexOf("&") > 0){
                                tokenParts =parts[i].Split('&');
                            }
                            foreach (String token in tokenParts)
                            {
                                String k = token;
                                String v = token;
                                int eqIdx = token.IndexOf("=");
                                if (eqIdx > 0)
                                {
                                    k = token.Substring(0, eqIdx);
                                    v = token.Substring(eqIdx + 1);
                                    v = UrlDecode(v, AppCommon.ENC_SJIS);
                                }
                                Console.WriteLine("b[" + k + "][" + v + "]");
                                postDataMap.Add(k, v);
                            }
                        }
                    }
                }
            }
        }

        private static string UrlDecode(string s, Encoding enc)
        {
            List<byte> bytes = new List<byte>();
            for (int i = 0; i < s.Length; i++)
            {
                char c = s[i];
                if (c == '%')
                    bytes.Add((byte)int.Parse(s[++i].ToString() + s[++i].ToString(), NumberStyles.HexNumber));
                else if (c == '+')
                    bytes.Add((byte)0x20);
                else
                    bytes.Add((byte)c);
            }
            return enc.GetString(bytes.ToArray(), 0, bytes.Count);
        }

        public void stop()
        {
            Console.WriteLine("Httpd.stop");
            alive = false;
            try
            {
                if (listener != null)
                {
                    listener.Stop();
                }
            }catch(Exception){
                //スルー
            }
        }
    }

    //メッセージ用パターン指定
    public enum HttpdAddMsgType
    {
        Sys,
        Join,
        Part,
        Pub
    }

    // 保持用メッセージログクラス
    public class HttpdAddMsgLog
    {
        public ServerInfo serverInfo;
        public ChannelInfo chInfo;
        public HttpdAddMsgType type;
        public String time;
        public String message1;
        public String message2;
        public HttpdAddMsgLog(ServerInfo serverInfo, ChannelInfo chInfo, HttpdAddMsgType type)
        {
            this.serverInfo = serverInfo;
            this.chInfo = chInfo;
            this.type = type;
        }
    }

}
