﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading.Tasks;
using System.Threading;
using System.IO;
using System.Collections;
using System.Globalization;
using System.Text.RegularExpressions;


namespace MoyariClient
{
    public partial class MainForm : Form
    {
        public const String SYSPANE_KEY = "SYS_PANEL_KEY";

        public const String LABEL_NOTCONNECT = "[未接続]";

        // キー= S_鯖Index or C_鯖Index_ChIndex
        private Dictionary<String, RichTextBox> textBoxMap = new Dictionary<String, RichTextBox>();
        private Dictionary<String, ListView> listViewMap = new Dictionary<String, ListView>();

        // 簡易httpd用ログ保持
        public Dictionary<String, Dictionary<String, List<HttpdAddMsgLog>>> httpdLogStore = new Dictionary<String, Dictionary<String, List<HttpdAddMsgLog>>>();

        //現在フォーカスのあるところ
        private ServerInfoTag currentTargetTag = null;

        public MainForm()
        {
            if (AppCommon.setting.saveWindowPos)
            {
                this.StartPosition = FormStartPosition.Manual;
                this.Location = new Point(AppCommon.setting.windowPosX, AppCommon.setting.windowPosY);                                
            }
            
            InitializeComponent();

            inputbox.pasteProc = inputboxPasteProc;// pasteハンドラ

            reflectColorEdit(); // 色反映

            setTransParent();//半透明設定

            if (AppCommon.setting.saveWindowPos)
            {
                this.Width = AppCommon.setting.windowWidth;
                this.Height = AppCommon.setting.windowHeight;
            }

            if (AppCommon.setting.httpdEnabled)
            {
                Httpd.getInstance(httpdLogStore).start();
            }

            notifyIconMain.Text = AppCommon.APPNAME;
            setTrayIconVisible();

            reflectSharedMode();
            menuItemSearchWindowChange();
        }

        // 初期表示時
        private void Form1_Shown(Object sender, EventArgs e)
        {
            textBoxMap.Add(SYSPANE_KEY, defaultTextBox);
            listViewMap.Add(SYSPANE_KEY, userList);

            if (AppCommon.setting != null && AppCommon.setting.infos != null && AppCommon.setting.infos.Length > 0)
            {
                foreach (ServerInfo serverinfo in AppCommon.setting.infos)
                {
                    // 鯖を追加
                    addServer(serverinfo);

                    if (serverinfo.connectOnStart)
                    {
                        //起動時接続を依頼
                        ServerManager.connectServer(serverinfo);
                    }
                }
            }
            else
            {
                //初回起動or設定がない場合の初期化処理
                formServerEditOpen(null);
            }

#if DEBUG
            debugToolStripMenuItem.Visible = true;
#endif

        }

        // 終了時
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            notifyIconMain.Dispose();
        }

        // 半透明設定更新
        public void setTransParent(){
            // 半透明
            if (AppCommon.setting.enableTrans)
            {
                this.Opacity = 0.01 * AppCommon.setting.transVolume;
            }
            else
            {
                //解除
                this.Opacity = 1.0;
            }
        }

        // タスクトレイアイコン
        public void setTrayIconVisible()
        {
            notifyIconMain.Visible = AppCommon.setting.taskTray;
        }

        // トレイアイコンクリック
        private void notifyIconMain_Click(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Minimized)
            {
                this.WindowState = FormWindowState.Normal;
                this.ShowInTaskbar = true;
            }
        }

        //×アクティブイベント
        //private void topToWindow(){
        //    if (FormTransfer.lastActivateTime > FormSearch.lastActivateTime)
        //    {
        //        FormSearch.windowToTop();
        //        FormTransfer.windowToTop();
        //    }
        //    else
        //    {
        //        FormTransfer.windowToTop();
        //        FormSearch.windowToTop();
        //    }
        //    this.TopMost = true;
        //    if (AppCommon.setting.topMost == false)
        //    {
        //        this.TopMost = false;
        //    }
        //}

        // 1行ぶんのメッセージを現在のフォーカスにpost
        private void postInputMessage(String message)
        {
            if (currentTargetTag != null)
            {
                if (currentTargetTag.chInfo == null)
                {
                    //鯖フォーカスの場合
                    addServerMmessage(currentTargetTag.serverInfo, message);
                }
                else
                {
                    if (currentTargetTag.chInfo.isPrivateMsgCh)
                    {
                        //Privメッセージポスト
                        ServerManager.c2sPrivateMessage(currentTargetTag.serverInfo, currentTargetTag.chInfo, message);
                    }
                    else
                    {
                        if (currentTargetTag.chInfo.joined)
                        {
                            //鯖管へメッセージのポスト
                            ServerManager.c2sPublicMessage(currentTargetTag.serverInfo, currentTargetTag.chInfo, message);
                        }
                        else
                        {
                            String msg = "入室していません：" + currentTargetTag.chInfo.name + " [" + message + "]";
                            writeMessage(HttpdAddMsgType.Sys, currentTargetTag.serverInfo, currentTargetTag.chInfo, msg);
                        }
                    }
                }
            }
            else
            {
                Console.WriteLine("inputbox_keydown:対象が取れない");
            }
        }
        // 入力フォームでEnter
        private void inputbox_keydown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == System.Windows.Forms.Keys.Enter)
            {
                e.SuppressKeyPress = true;
                if (inputbox.Text.Length == 0)
                {
                    return;
                }
                postInputMessage(inputbox.Text);
                
                inputbox.Text = ""; //クリア
            }
        }

        // 入力formでPasteイベント
        private void inputboxPasteProc(String msg)
        {
            FormPaste formPst = new FormPaste(msg, inputboxPasteProcCallBack);
            formPst.StartPosition = FormStartPosition.CenterParent;
            formPst.ShowDialog(this);
            formPst.Dispose();

            return;
        }
        // ↑書き込み時コールバック
        private void inputboxPasteProcCallBack(String msg)
        {
            String[] lines = msg.Split('\n');
            foreach (String line in lines)
            {
                String l2 = line.Replace("\r", "");
                postInputMessage(l2);
            }
        }

        // リンククリック時
        private void textBox_linkclick(object sender, LinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start(e.LinkText);
        }

        // キー名作成
        private String getKeyStr(ServerInfo serverInfo, ChannelInfo chInfo)
        {
            if (chInfo != null)
            {
                if (chInfo.isPrivateMsgCh)
                {
                    return "P_" + serverInfo.index + "_" + chInfo.index;
                }
                else
                {
                    return "C_" + serverInfo.index + "_" + chInfo.index;
                }
            }
            else
            {
                return "S_" + serverInfo.index;
            }
        }

        // サーバ追加
        private void addServer(ServerInfo inf)
        {
            TreeNode parent = new TreeNode();
            parent.Text = inf.name + LABEL_NOTCONNECT;
            ServerInfoTag tagInf = new ServerInfoTag(inf, null);
            parent.Tag = tagInf;
           
            parent.Expand();
            chtree.Nodes.Add(parent);

            addPanel(inf, null);

            //初期化時のパネルの移動
            if (textBoxMap[SYSPANE_KEY].Visible)
            {
                selectTreeView(tagInf);
            }

            //鯖管に登録
            ServerManager.addSertverInfo(inf);
        }

        // チャンネルツリービュー要素選択時
        private void selectTreeView(ServerInfoTag tagInfo)
        {
            String panelKey = getKeyStr(tagInfo.serverInfo, tagInfo.chInfo);
            //Console.WriteLine("ツリービュー選択時：" + panelKey);
            if (textBoxMap[panelKey].Visible)
            {
                return; // 表示中の場合
            }

            textBoxMap[panelKey].Visible = true;
            listViewMap[panelKey].Visible = true;
            foreach (String k in textBoxMap.Keys)
            {
                if (k.CompareTo(panelKey) != 0)
                {
                    textBoxMap[k].Visible = false;
                }
            }
            foreach (String k in listViewMap.Keys)
            {
                if (k.CompareTo(panelKey) != 0)
                {
                    listViewMap[k].Visible = false;
                }
            }

            //リストのFocusをなおす
            foreach (TreeNode parentNode in chtree.Nodes)
            {
                //parentNode
                ServerInfoTag parentTagInfo = (ServerInfoTag)parentNode.Tag;
                if (parentTagInfo != null)
                {
                    if (tagInfo.serverInfo.index.CompareTo(parentTagInfo.serverInfo.index) == 0)
                    {
                        if (tagInfo.chInfo == null)
                        {
                            //鯖一致、鯖選択->focus
                            parentNode.BackColor = AppCommon.setting.colorInfo.colorFocusChBack;
                            parentNode.ForeColor = AppCommon.setting.colorInfo.colorFocusChFore;
                            chtree.SelectedNode = parentNode;
                        }
                        else
                        {
                            //鯖一致、ch選択->focus解除
                            parentNode.BackColor = AppCommon.setting.colorInfo.colorBackground;
                            parentNode.ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
                        }
                    }
                    else
                    {
                        //鯖不一致->focus解除
                        parentNode.BackColor = AppCommon.setting.colorInfo.colorBackground;
                        parentNode.ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
                    }
                }

                foreach (TreeNode childNode in parentNode.Nodes)
                {
                    ServerInfoTag childTagInfo = (ServerInfoTag)childNode.Tag;
                    if (tagInfo.serverInfo.index.CompareTo(parentTagInfo.serverInfo.index) == 0)
                    {
                        if (tagInfo.chInfo == null)
                        {
                            //鯖一致、鯖選択->focus解除
                            childNode.BackColor = AppCommon.setting.colorInfo.colorBackground;
                            childNode.ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
                        }
                        else
                        {
                            if (tagInfo.chInfo.index.CompareTo(childTagInfo.chInfo.index) == 0)
                            {
                                //鯖一致、ch一致->focus
                                childNode.BackColor = AppCommon.setting.colorInfo.colorFocusChBack;
                                childNode.ForeColor = AppCommon.setting.colorInfo.colorFocusChFore;
                                chtree.SelectedNode = childNode;
                            }
                            else
                            {
                                //鯖一致、ch不一致->focus解除
                                childNode.BackColor = AppCommon.setting.colorInfo.colorBackground;
                                childNode.ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
                            }
                        }
                    }
                    else
                    {
                        //鯖不一致->focus解除
                        childNode.BackColor = AppCommon.setting.colorInfo.colorBackground;
                        childNode.ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
                    }
                }
            }
            
            changeTitleText(tagInfo.serverInfo, tagInfo.chInfo);

            currentTargetTag = tagInfo;
        }

        // チャンネルツリービュー要素選択時
        private void chtree_AfterSelect(object sender, TreeViewEventArgs e)
        {
            selectTreeView((ServerInfoTag)e.Node.Tag);
        }

        // チャンネルツリービューダブルクリック時
        private void chtree_DoubleClick(object sender, EventArgs e)
        {
            if (currentTargetTag != null)
            {
                // サーバ
                if (ServerManager.checkServerIsConnected(currentTargetTag.serverInfo) == false)
                {
                    ServerManager.connectServer(currentTargetTag.serverInfo);
                }

                // チャンネル
                if (currentTargetTag.chInfo != null && currentTargetTag.chInfo.joined == false)
                {
                    ServerManager.joinCh(currentTargetTag.serverInfo, currentTargetTag.chInfo.name);
                }
            }
        }

        // ユーザリストクリック処理
        private UserInfo rightClickMenuUserInfo = null;
        private void userList_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                if (currentTargetTag != null)
                {
                    if (currentTargetTag.chInfo != null)
                    {
                        //対象がチャンネル
                        String key = getKeyStr(currentTargetTag.serverInfo, currentTargetTag.chInfo);
                        ListView lview =  listViewMap[key];
                        ListViewItem viewitem = lview.GetItemAt(e.X, e.Y);
                        if (viewitem != null)
                        {
                            UserInfo userInfo = (UserInfo)viewitem.Tag;
                            if (userInfo.name.CompareTo(currentTargetTag.serverInfo.user) == 0)
                            {
                                tsMenuItemUserInfo.Enabled = false;
                                tsMenuItemUserMsg.Enabled = false;
                                tsMenuItemUserBrowse.Enabled = false;
                            }
                            else
                            {
                                tsMenuItemUserInfo.Enabled = true;
                                tsMenuItemUserBrowse.Enabled = true;
                                if (currentTargetTag.chInfo.isPrivateMsgCh)
                                {
                                    tsMenuItemUserMsg.Enabled = false;
                                }
                                else
                                {
                                    tsMenuItemUserMsg.Enabled = true;
                                }
                                rightClickMenuUserInfo = userInfo;
                            }
                            ctxMenuUserInfo.Show(MousePosition);
                        }
                        else
                        {
                            //click範囲にitemが無い
                        }
                    }
                    else
                    {
                        // chInfoが無いパネル
                    }
                }
                else
                {
                    // currentTargetTagがnull
                }
            }
        }

        // ユーザリスト：メッセージ送信
        private void tsMenuItemUserMsg_Click(object sender, EventArgs e)
        {
            if (rightClickMenuUserInfo != null && currentTargetTag != null)
            {   
                ServerInfo serverInfo = currentTargetTag.serverInfo;
                addPrivChInfo(serverInfo, rightClickMenuUserInfo.name, true);
            }
        }

        //Privチャットchを鯖に登録、パネルの設定、フォーカス
        private void addPrivChInfo(ServerInfo serverInfo, String userName, bool forceFocus)
        {
            ChannelInfo chInfo = ServerManager.getPrivChatChannelInfo(serverInfo, userName);

            String key = getKeyStr(serverInfo, chInfo);
            if (textBoxMap.ContainsKey(key) == false)
            {
                // パネルとユーザリスト
                addChannel(serverInfo, chInfo);

                //フォーカス移動
                selectTreeView(new ServerInfoTag(serverInfo, chInfo));
            }
            else if (forceFocus)
            {
                selectTreeView(new ServerInfoTag(serverInfo, chInfo));
            }

        }

        // ユーザリスト：ユーザ情報
        private void tsMenuItemUserInfo_Click(object sender, EventArgs e)
        {
            if (rightClickMenuUserInfo != null && currentTargetTag != null)
            {
                //リクエスト
                ServerManager.whoisC2Server(currentTargetTag.serverInfo, rightClickMenuUserInfo.name);                
            }
        }

        // ユーザリスト：ブラウズ
        private void tsMenuItemUserBrowse_Click(object sender, EventArgs e)
        {
            if (rightClickMenuUserInfo != null && currentTargetTag != null)
            {
                //リクエスト
                FormPersonalBrowse.startNew(currentTargetTag.serverInfo, rightClickMenuUserInfo.name);                
            }
        }

        // ユーザリスト：いぐのあ
        private void tsMenuItemUserIgnore_Click(object sender, EventArgs e)
        {
            if (rightClickMenuUserInfo != null && currentTargetTag != null)
            {
                String target = rightClickMenuUserInfo.name;
                //リクエスト
                DialogResult result = MessageBox.Show("無視リストに追加しますか？\n" + target,
                        "確認",
                        MessageBoxButtons.YesNo,
                        MessageBoxIcon.Exclamation,
                        MessageBoxDefaultButton.Button2);

                if (result == DialogResult.Yes)
                {
                    AppCommon.setting.ignoreList.Add(target);
                    AppCommon.save();
                    //リストから消す
                    removeUserIngo(currentTargetTag.serverInfo, currentTargetTag.chInfo, target);

                    writeMessage(HttpdAddMsgType.Sys,currentTargetTag.serverInfo, currentTargetTag.chInfo, target + "を無視リストに追加しました");
                }                
            }
        }

        // チャンネルツリービュークリック処理
        private void chtree_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                chtree.SelectedNode = chtree.GetNodeAt(e.X, e.Y);

                if (currentTargetTag != null)
                {
                    if (currentTargetTag.chInfo != null)
                    {
                        if (currentTargetTag.chInfo.isPrivateMsgCh)
                        {
                            //対象がPriv
                            tsMenuItemChJoin.Enabled = false;
                            tsMenuItemChPart.Enabled = false;
                        }
                        else
                        {
                            if (currentTargetTag.chInfo.joined)
                            {
                                //対象がチャンネル、入室状態
                                tsMenuItemChJoin.Enabled = false;
                                tsMenuItemChPart.Enabled = true;
                            }
                            else
                            {
                                //対象がチャンネル、退室状態
                                tsMenuItemChJoin.Enabled = true;
                                tsMenuItemChPart.Enabled = false;
                            }
                        }
                        ctxMenuChannelEdit.Show(MousePosition);
                    }
                    else
                    {
                        
                        if (ServerManager.checkServerIsConnected(currentTargetTag.serverInfo))
                        {
                            //対象がサーバー、接続状態
                            tsMenuItemServerConnect.Enabled = false;
                            tsMenuItemServerDisconnect.Enabled = true;
                            tsMenuItemServerCancelReconect.Enabled = false;
                            tsMenuItemServerEdit.Enabled = false;
                            tsMenuItemServerDel.Enabled = false;
                        }
                        else
                        {
                            //対象がサーバー、切断状態
                            tsMenuItemServerConnect.Enabled = true;
                            tsMenuItemServerDisconnect.Enabled = false;
                            if (currentTargetTag.serverInfo.reconnectControlFlg)//再接続中
                            {
                                tsMenuItemServerCancelReconect.Enabled = true;
                            }
                            else
                            {
                                tsMenuItemServerCancelReconect.Enabled = false;
                            }
                            tsMenuItemServerEdit.Enabled = true;
                            tsMenuItemServerDel.Enabled = true;
                        }
                        tsMenuItemServerAdd.Enabled = true;
                        ctxMenuServerEdit.Show(MousePosition);
                    }
                }
                else
                {
                    //currentTargetTagが無い->追加のみ
                    tsMenuItemServerConnect.Enabled = false;
                    tsMenuItemServerDisconnect.Enabled = false;
                    tsMenuItemServerAdd.Enabled = true;
                    tsMenuItemServerDel.Enabled = false;
                    tsMenuItemServerEdit.Enabled = false;
                    ctxMenuServerEdit.Show(MousePosition);
                }
            }
        }

        // サーバーメニュー：接続
        private void tsMenuItemServerConnect_Click(object sender, EventArgs e)
        {   
            ServerManager.connectServer(currentTargetTag.serverInfo);
        }

        // サーバーメニュー：切断
        private void tsMenuItemServerDisconnect_Click(object sender, EventArgs e)
        {
            currentTargetTag.serverInfo.reconnectControlFlg = false; //再接続予約を解除
            ServerManager.disconnectC2Server(currentTargetTag.serverInfo);
        }

        // サーバーメニュー：再接続キャンセル
        private void tsMenuItemServerCancelReconect_Click(object sender, EventArgs e)
        {
            currentTargetTag.serverInfo.reconnectControlFlg = false;
            writeMessage(HttpdAddMsgType.Sys, currentTargetTag.serverInfo, null, "再接続予約を取り消しました");
        }

        // サーバーメニュー：追加
        private void tsMenuItemServerAdd_Click(object sender, EventArgs e)
        {
            FormServerEdit serverEditForm = new FormServerEdit(this, null);
            serverEditForm.StartPosition = FormStartPosition.CenterParent;
            serverEditForm.ShowDialog(this);
            serverEditForm.Dispose();
        }

        // サーバーメニュー：削除
        private void tsMenuItemServerDel_Click(object sender, EventArgs e)
        {
            ServerInfo serverInfo = currentTargetTag.serverInfo;

            DialogResult result = MessageBox.Show(
                "" + serverInfo.name + "\n削除して良いですか？",
                "",
                MessageBoxButtons.YesNo,
                MessageBoxIcon.Exclamation,
                MessageBoxDefaultButton.Button2);
            if (result == DialogResult.Yes)
            {
                // 繋がってれば切断
                if (ServerManager.checkServerIsConnected(serverInfo))
                {
                    ServerManager.disconnectC2Server(serverInfo);
                }

                // 鯖情報ツリーから削除
                foreach (TreeNode parent in chtree.Nodes)
                {
                    ServerInfoTag parentTag = (ServerInfoTag)parent.Tag;
                    if (serverInfo.index.CompareTo(parentTag.serverInfo.index) == 0)
                    {
                        // チャンネル用のパネル、ユーザリストを削除
                        foreach (TreeNode child in parent.Nodes)
                        {
                            ServerInfoTag childTag = (ServerInfoTag)child.Tag;
                            String chKey = getKeyStr(childTag.serverInfo, childTag.chInfo);
                            lock (textBoxMap)
                            {
                                if (textBoxMap.ContainsKey(chKey))
                                {
                                    textBoxMap[chKey].Visible = false;
                                    textBoxMap.Remove(chKey);
                                }
                            }
                            lock (listViewMap)
                            {
                                if (listViewMap.ContainsKey(chKey))
                                {
                                    listViewMap[chKey].Visible = false;
                                    listViewMap.Remove(chKey);
                                }
                            }
                        }
                        // サーバ用のパネル、ユーザリストを削除
                        String serverkey = getKeyStr(serverInfo, null);
                        lock (textBoxMap)
                        {
                            if (textBoxMap.ContainsKey(serverkey))
                            {
                                textBoxMap[serverkey].Visible = false;
                                textBoxMap.Remove(serverkey);
                            }
                        }
                        lock (listViewMap)
                        {
                            if (listViewMap.ContainsKey(serverkey))
                            {
                                listViewMap[serverkey].Visible = false;
                                listViewMap.Remove(serverkey);
                            }
                        }

                        //ツリービューから削除
                        chtree.Nodes.Remove(parent);
                        break;
                    }
                }

                // サーバー情報を削除
                ServerManager.removeSertverInfo(serverInfo);

                // 削除情報を設定保存
                List<ServerInfo> sInfoList = new List<ServerInfo>();
                foreach (ServerInfo info in AppCommon.setting.infos)
                {
                    if (info.index.CompareTo(serverInfo.index) != 0)
                    {
                        sInfoList.Add(info);
                    }
                }
                AppCommon.setting.infos = sInfoList.ToArray();
                AppCommon.save();
            }
        }

        // サーバーメニュー：編集
        private void tsMenuItemServerEdit_Click(object sender, EventArgs e)
        {
            ServerInfo serverInfo = currentTargetTag.serverInfo;
            formServerEditOpen(serverInfo);
        }

        // サーバーメニュー：入室
        private void tsMenuItemChJoin_Click(object sender, EventArgs e)
        {
            if (currentTargetTag != null)
            {
                if (currentTargetTag.chInfo != null)
                {
                    if (currentTargetTag.chInfo.joined == false)
                    {
                        ServerManager.joinCh(currentTargetTag.serverInfo, currentTargetTag.chInfo.name);
                    }
                }
            }
        }

        // サーバーメニュー：退室
        private void tsMenuItemChPart_Click(object sender, EventArgs e)
        {
            if (currentTargetTag != null)
            {
                if (currentTargetTag.chInfo != null)
                {
                    if (currentTargetTag.chInfo.joined)
                    {
                        ServerManager.partChC2Server(currentTargetTag.serverInfo, currentTargetTag.chInfo.name);
                    }
                }
            }
        }

        // パネル追加(鯖名,チャンネル名[null可)
        private void addPanel(ServerInfo serverInfo, ChannelInfo chInfo)
        {
            String key = getKeyStr(serverInfo, chInfo);
            // ビュー用テキストボックス
            RichTextBox subTextBox = new RichTextBox();
            subTextBox.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                        | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            subTextBox.Location = new System.Drawing.Point(0, 0);
            subTextBox.Margin = new System.Windows.Forms.Padding(0);
            subTextBox.Name = key;
            subTextBox.Size = new System.Drawing.Size(431, 339);
            subTextBox.TabIndex = 5;
            subTextBox.LinkClicked += new System.Windows.Forms.LinkClickedEventHandler(this.textBox_linkclick);
            subTextBox.Visible = false;
            subTextBox.KeyPress += new System.Windows.Forms.KeyPressEventHandler(this.defaultTextBox_KeyPress);
            subTextBox.ContextMenuStrip = this.ctxMenuStripTxtPanel;
            subTextBox.BackColor = AppCommon.setting.colorInfo.colorBackground;
            subTextBox.Font = AppCommon.setting.txtFont;
            //splitContainer1.Panel1.Controls.Add(subTextBox);
            tableLayoutPanel1.Controls.Add(subTextBox, 0, 0);

            // リストビュー
            ColumnHeader user = new ColumnHeader();
            ColumnHeader files = new ColumnHeader();
            ColumnHeader speed = new ColumnHeader();
            user.Text = "User";
            user.Width = 100;
            files.Text = "Files";
            speed.Text = "Speed";

            ListView userlist = new System.Windows.Forms.ListView();
            userlist.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] { user, files, speed });
            userlist.Dock = System.Windows.Forms.DockStyle.Fill;
            userlist.FullRowSelect = true;
            userlist.LabelWrap = false;
            userlist.Location = new System.Drawing.Point(0, 0);
            userlist.Margin = new System.Windows.Forms.Padding(0);
            userlist.Name = key;
            userlist.Size = new System.Drawing.Size(145, 201);
            userlist.TabIndex = 0;
            userlist.UseCompatibleStateImageBehavior = false;
            userlist.View = System.Windows.Forms.View.Details;
            userlist.MouseDown += new System.Windows.Forms.MouseEventHandler(this.userList_MouseDown);
            userlist.BackColor = AppCommon.setting.colorInfo.colorBackground;
            userlist.ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
            userlist.Font = AppCommon.setting.txtFont;
            userlist.Visible = false;

            userlist.ListViewItemSorter = new ListViewColumnSorter();
            userlist.ColumnClick += new System.Windows.Forms.ColumnClickEventHandler(this.userList_ColumnClick);


            splitContainer2.Panel1.Controls.Add(userlist);

            String sn = serverInfo.name;
            if (chInfo != null)
            {
                sn += ":" + chInfo.name;
            }
            Console.WriteLine("パネル追加 [" + key + "]["+sn+"]");
            lock (listViewMap)
            {
                listViewMap.Add(key, userlist);
            }
            lock (textBoxMap)
            {
                textBoxMap.Add(key, subTextBox);
            }
            
            String initMsg = serverInfo.name;            
            if (chInfo != null)
            {
                initMsg += " " + chInfo.name;
            }
            writeMessage(HttpdAddMsgType.Sys, serverInfo, chInfo, initMsg, null, true);
        }

        // ユーザリストカラムクリック時
        private void userList_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            if (currentTargetTag != null)
            {
                String key = getKeyStr(currentTargetTag.serverInfo, currentTargetTag.chInfo);

                if (listViewMap.ContainsKey(key) == false)
                {
                    return;
                }
                ListView listView = listViewMap[key];
                ListViewColumnSorter _columnSorter = (ListViewColumnSorter)listView.ListViewItemSorter;

                if (e.Column == _columnSorter.SortColumn)
                {
                    if (_columnSorter.Order == SortOrder.Ascending)
                    {
                        _columnSorter.Order = SortOrder.Descending;
                    }
                    else
                    {
                        _columnSorter.Order = SortOrder.Ascending;
                    }
                }
                else
                {
                    _columnSorter.SortColumn = e.Column;
                    _columnSorter.Order = SortOrder.Ascending;
                }
                listView.Sort();
            }
        }

        //ログ出力
        private void writeLog(ServerInfo serverInfo, ChannelInfo chInfo, String message)
        {
            if (AppCommon.setting.enableLog == false)
            {
                return;
            }
            //Application.StartupPath + "\\" + AppCommon.setting.logDir;
            String logDirPath =  AppCommon.setting.logDir;
            if (Directory.Exists(logDirPath) == false)
            {
                try
                {
                    Directory.CreateDirectory(logDirPath);
                }
                catch (Exception e)
                {
                    // フォルダが無くてファイルがある
                    MessageBox.Show("ログ出力フォルダが作成できません。ログ出力設定を解除します。\n\n" + e.Message, "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    AppCommon.setting.enableLog = false;
                    AppCommon.save();
                    return;
                }
            }

            // 鯖名_ch名_日付.log or 鯖名_日付.log
            String timeStr = DateTime.Now.ToString("yyyyMMdd");
            String fileName = serverInfo.name;
            if (chInfo == null)
            {
                fileName += "_" + timeStr+".log"; //fam_fam_20121103.log
            }
            else
            {
                fileName += "_" + chInfo.name + "_" + timeStr + ".log"; //fam_#xxx_20121106.log
            }

            String logFilePath = logDirPath + "\\" + fileName;
            try
            {
                StreamWriter writer = new StreamWriter(logFilePath, true, AppCommon.ENC_SJIS);
                writer.WriteLine(message);
                writer.Close();
            }
            catch (Exception e)
            {
                MessageBox.Show("ログ出力に失敗しました。ログ出力設定を解除します。\n\n" + e.Message, "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                AppCommon.setting.enableLog = false;
                AppCommon.save();
                return;
            }
        }

        // パネルにメッセージ追記 △△△
        private void writeMessage(HttpdAddMsgType msgType, ServerInfo serverInfo, ChannelInfo chInfo, String message1, String message2 = null, Boolean nolog = false)
        {
            String key = getKeyStr(serverInfo, chInfo);
            if (textBoxMap.ContainsKey(key) == false)
            {
                Console.WriteLine("異常系　メッセージ追記対象が無い");
                return;
            }
            RichTextBox targetBox = textBoxMap[key];

            int lockStartPos = targetBox.Text.Length;
            targetBox.SelectionStart = lockStartPos;
            //if (msgType != AddMsgType.Init)//最初だけ改行しない
            //{
            //    targetBox.AppendText(System.Environment.NewLine);
            //}

            String logMessage = "";
            //時刻
            targetBox.SelectionColor = AppCommon.setting.colorInfo.colorTime;
            String timeStr = DateTime.Now.ToString("HH:mm:ss") + " ";
            targetBox.AppendText(timeStr);
            logMessage += timeStr;

            switch(msgType) {                    
                case HttpdAddMsgType.Join:
                    //join
                    targetBox.SelectionColor = AppCommon.setting.colorInfo.colorJoinMsg;
                    targetBox.AppendText(message1);
                    logMessage += message1;
                    break;
                case HttpdAddMsgType.Part:
                    //part
                    targetBox.SelectionColor = AppCommon.setting.colorInfo.colorPartMsg;
                    targetBox.AppendText(message1);
                    logMessage += message1;
                    break;
                case HttpdAddMsgType.Pub:
                    //nick
                    targetBox.SelectionColor = AppCommon.setting.colorInfo.colorNick;
                    targetBox.AppendText(message1 + " > ");
                    //メッセージ
                    targetBox.SelectionColor = AppCommon.setting.colorInfo.colorNormalMsg;
                    targetBox.AppendText(message2);
                    checkURLMessage(message2, message1); //URL履歴用
                    logMessage += message1 + " > " + message2;
                    break;
                case HttpdAddMsgType.Sys:
                default:
                    //他
                    targetBox.SelectionColor = AppCommon.setting.colorInfo.colorSysMsg;
                    targetBox.AppendText(message1);
                    logMessage += message1;
                    break;
            }

            if (nolog == false)
            {
                writeLog(serverInfo, chInfo, logMessage);
            }

            targetBox.AppendText(System.Environment.NewLine);

            ////ロック
            targetBox.Select(lockStartPos, targetBox.Text.Length);
            targetBox.SelectionProtected = true;

            if (targetBox.Text.Length > AppCommon.PANEL_LOG_DOROP_LIMIT)
            {
                targetBox.Select(0, AppCommon.PANEL_LOG_DOROP_STEP);
                targetBox.SelectionProtected = false;
                targetBox.SelectedText = "";
            }

            targetBox.SelectionStart = targetBox.Text.Length;
            targetBox.ScrollToCaret();

            // 呼び出し小窓(対象はmessage2のみ
            if (AppCommon.setting.enableCalling && message2 != null)
            {
                checkCalling(serverInfo, timeStr, message1, message2);
            }

            // httpd用ロギング
            HttpdAddMsgLog log = new HttpdAddMsgLog(serverInfo, chInfo,msgType);
            log.time = timeStr;
            log.message1 = message1;
            log.message2 = message2;
            if(httpdLogStore.ContainsKey(serverInfo.index)==false){
                httpdLogStore.Add(serverInfo.index, new Dictionary<String, List<HttpdAddMsgLog>>());
            }
            //LOGSTORE_KEY_SYSTEM 
            String chkey = AppCommon.HTTP_LOGSTORE_KEY_CHANNNEL;
            if (chInfo != null)
            {
                chkey = chInfo.index;
            }
            if (httpdLogStore[serverInfo.index].ContainsKey(chkey) == false)
            {
                httpdLogStore[serverInfo.index].Add(chkey, new List<HttpdAddMsgLog>());
            }
            httpdLogStore[serverInfo.index][chkey].Add(log);
            if (httpdLogStore[serverInfo.index][chkey].Count > AppCommon.HTTP_LOGSTORE_MAX)
            {
                httpdLogStore[serverInfo.index][chkey].RemoveAt(0);
            }
        }
        // 呼び出し小窓処理
        private void checkCalling(ServerInfo serverInfo, String timeStr, String message1, String message2)
        {
            if (AppCommon.setting.callByNick)
            {
                int matchIdx1 = message2.ToUpper().IndexOf(serverInfo.user.ToUpper());                
                if (matchIdx1 >= 0)
                {
                    FormCalling.proc(this, timeStr, serverInfo.user, message1, message2, matchIdx1);
                    addCallingHistry(timeStr, message1, message2, serverInfo.user, matchIdx1);
                    return;
                }
            }
            foreach (String kwd in AppCommon.setting.appendKeywords)
            {
                int matchIdx2 = message2.ToUpper().IndexOf(kwd.ToUpper());
                if (matchIdx2 >= 0)
                {
                    FormCalling.proc(this, timeStr, kwd, message1, message2, matchIdx2);
                    addCallingHistry(timeStr, message1, message2, kwd, matchIdx2);
                    return;
                }
            }
        }
        // 呼出履歴保持
        private void addCallingHistry(String timeStr, String message1, String message2, String matchword, int matchIdx)
        {
            String prefix = DateTime.Now.ToString("MM/dd HH:mm:ss") + " " + message1 + " ";
            String showStr = message2;//表示用
            if (showStr.Length > AppCommon.setting.historyTextShowLimit)
            {
                showStr = showStr.Substring(0, AppCommon.setting.historyTextShowLimit - 2) + "~~";
            }
            ToolStripMenuItem tsmi = new ToolStripMenuItem();
            tsmi.Name = showStr;
            tsmi.Size = new System.Drawing.Size(152, 22);
            tsmi.Text = prefix + showStr;
            tsmi.Tag = new CallingHistoryTag(timeStr, matchword, message1, message2, matchIdx);
            tsmi.Click += new System.EventHandler(this.tsMenuItemHistoryCalling_Click);
            this.tsMenuItemHistoryCall.DropDownItems.Insert(0, tsmi);

            if (this.tsMenuItemHistoryCall.DropDownItems.Count > AppCommon.setting.historySizeCall)
            {
                this.tsMenuItemHistoryCall.DropDownItems.RemoveAt(this.tsMenuItemHistoryCall.DropDownItems.Count - 1);
            }
        }

        // URL履歴保持
        private void checkURLMessage(String message, String msgfrom)
        {
            String prefix = DateTime.Now.ToString("MM/dd HH:mm:ss") + " " + msgfrom + " ";
            MatchCollection mc = Regex.Matches(message, AppCommon.URL_PATTERN);
            foreach (System.Text.RegularExpressions.Match m in mc)
            {
                String showUrl = m.Value;//表示用
                if (showUrl.Length > AppCommon.setting.historyTextShowLimit)
                {
                    showUrl = showUrl.Substring(0, AppCommon.setting.historyTextShowLimit - 2) + "~~";
                }
                ToolStripMenuItem tsmi = new ToolStripMenuItem();
                tsmi.Name = showUrl;
                tsmi.Size = new System.Drawing.Size(152, 22);
                tsmi.Text = prefix + showUrl; 
                tsmi.Tag = m.Value;
                tsmi.Click += new System.EventHandler(this.tsMenuItemHistoryURL_Click);
                this.tsMenuItemHistoryURL.DropDownItems.Insert(0, tsmi);
                
                if (this.tsMenuItemHistoryURL.DropDownItems.Count > AppCommon.setting.historySizeURL)
                {
                    this.tsMenuItemHistoryURL.DropDownItems.RemoveAt(this.tsMenuItemHistoryURL.DropDownItems.Count - 1);
                }
            }
        }

        // サーバメッセージの追記(鯖名, メッセージ)※受けスレッド用
        delegate void addServerMmessageDelegate(ServerInfo serverInfo, String message);
        public void addServerMmessageInvoke(ServerInfo serverInfo, String message)
        {
            this.Invoke(new addServerMmessageDelegate(addServerMmessage), new object[] { serverInfo, message });
        }
        // サーバメッセージの追記内部
        private void addServerMmessage(ServerInfo serverInfo, String message)
        {
            writeMessage(HttpdAddMsgType.Sys, serverInfo, null, message);
        }

        // チャンネルメッセージの追記(鯖名, チャンネル名, ユーザ名, メッセージ)※受けスレッド用
        delegate void addChMmessageDelegate(ServerInfo serverInfo, ChannelInfo chInfo, String userName, String message,bool isPriv);
        public void addChMmessageInvoke(ServerInfo serverInfo, ChannelInfo chInfo, String userName, String message, bool isPriv)
        {
            this.Invoke(new addChMmessageDelegate(addChMmessage), new object[] { serverInfo, chInfo, userName, message, isPriv });
        }
        // チャンネルメッセージの追記(内部
        private void addChMmessage(ServerInfo serverInfo, ChannelInfo chInfo, String userName, String message, bool isPriv)
        {
            if (AppCommon.setting.ignoreList.Contains(userName))
            {
                Console.WriteLine("イグノア chMmessage [" + userName + "][" + message + "]");
                return;
            }

            if (isPriv)
            {
                //privの場合は初回パネル追加の可能性
                addPrivChInfo(serverInfo, chInfo.name, true);
            }

            //Privの場合はパネルがない場合がある
            writeMessage(HttpdAddMsgType.Pub, serverInfo, chInfo, userName, message);
        }

        //ユーザ追加(鯖名, チャンネル名, ユーザ名)
        delegate void userJoinedDelegate(ServerInfo serverInfo, ChannelInfo chInfo, UserInfo userInfo, bool isInit);
        public void userJoinedInvoke(ServerInfo serverInfo, ChannelInfo chInfo, UserInfo userInfo, bool isInit)
        {
            this.Invoke(new userJoinedDelegate(userJoined), new object[] { serverInfo, chInfo, userInfo, isInit });
        }
        private void userJoined(ServerInfo serverInfo, ChannelInfo chInfo, UserInfo userInfo, bool isInit)
        {
            if (AppCommon.setting.ignoreList.Contains(userInfo.name))
            {
                Console.WriteLine("イグノア userJoined [" + userInfo.name + "]");
                return;
            }

            if (chInfo == null || userInfo == null)
            {
                Console.WriteLine("異常系　状態不備？ [" + chInfo + "][" + userInfo + "]");
                return;
            }
            foreach(UserInfo u in chInfo.userList) {
                if (u.name.CompareTo(userInfo.name) == 0)
                {
                    Console.WriteLine("異常系　重複ユーザ[" + u.name + "]");
                    return;
                }
            }
            
            String key = getKeyStr(serverInfo, chInfo);
            if (isInit == false)
            {
                writeMessage(HttpdAddMsgType.Join, serverInfo, chInfo, userInfo.name + "が入室しました。");
            }

            //ユーザリスト登録
            chInfo.userList.Add(userInfo);

            String[] line = { userInfo.name, userInfo.sharing, userInfo.linktype };
            ListViewItem item = new ListViewItem(line);
            item.Tag = userInfo;
            listViewMap[key].Items.Add(item);
        }

        //共有数更新
        delegate void updateSharedFileListDelegate(string userName, string shareNum);
        public void updateSharedFileListInvoke(string userName, string shareNum)
        {
            this.Invoke(new updateSharedFileListDelegate(updateSharedFileList), new object[] { userName, shareNum });
        }
        private void updateSharedFileList(string userName, string shareNum)
        {

            foreach (String key in listViewMap.Keys)
            {
                foreach (ListViewItem item in listViewMap[key].Items)
                {
                    if (item.SubItems.Count >= 2)
                    {
                        if (userName == item.SubItems[0].Text)
                        {
                            item.SubItems[1].Text = shareNum;
                            if (item.Tag != null && item.Tag is UserInfo)
                            {
                                UserInfo uinf = (UserInfo)item.Tag;
                                uinf.sharing = shareNum;
                            }
                        }   
                    }
                }
            }
        }

        //ユーザ削除(鯖名, チャンネル名, ユーザ名)
        delegate void userPartedDelegate(ServerInfo serverInfo, ChannelInfo chInfo, String userName);
        public void userPartedInvoke(ServerInfo serverInfo, ChannelInfo chInfo, String userName)
        {
            this.Invoke(new userPartedDelegate(userParted), new object[] { serverInfo, chInfo, userName });
        }
        private void userParted(ServerInfo serverInfo, ChannelInfo chInfo, String userName)
        {
            if (AppCommon.setting.ignoreList.Contains(userName))
            {
                Console.WriteLine("イグノア userParted [" + userName + "]");
                return;
            }

            if (chInfo == null)
            {
                Console.WriteLine("異常系　状態不備？ [" + chInfo + "]");
                return;
            }


            writeMessage(HttpdAddMsgType.Part, serverInfo, chInfo, userName + "が退室しました。");
            removeUserIngo(serverInfo, chInfo, userName);
        }
        private void removeUserIngo(ServerInfo serverInfo, ChannelInfo chInfo, String userName)
        {
            String key = getKeyStr(serverInfo, chInfo);
            UserInfo r = null;
            foreach (UserInfo u in chInfo.userList)
            {
                if (u.name.CompareTo(userName) == 0)
                {
                    r = u;
                    break;
                }
            }
            if (r == null)
            {
                Console.WriteLine("異常系　削除対象無し[" + userName + "]");
                return;
            }
            chInfo.userList.Remove(r);

            ListViewItem ritem = null;
            foreach (ListViewItem item in listViewMap[key].Items)
            {
                UserInfo ui = (UserInfo)item.Tag;
                if (ui.index.CompareTo(r.index) == 0)
                {
                    ritem = item;
                    break;
                }
            }
            if (ritem == null)
            {
                Console.WriteLine("異常系　削除対象無し２[" + userName + "]");
                return;
            }
            listViewMap[key].Items.Remove(ritem);
        }

        // channnel情報更新追加
        delegate void updateChannelListEntryDelegate(ServerInfo serverInfo, ChannelInfo chInfo);
        public void updateChannelListEntryInvoke(ServerInfo serverInfo, ChannelInfo chInfo)
        {
            this.Invoke(new updateChannelListEntryDelegate(updateChannelListEntry), new object[] { serverInfo , chInfo});
        }
        private void updateChannelListEntry(ServerInfo serverInfo, ChannelInfo chInfo)
        {
            String tag = getKeyStr(serverInfo, chInfo);
            if (textBoxMap.ContainsKey(tag) == false) 
            {
                addChannel(serverInfo, chInfo);
            }

            updateChannnelText();
        }

        //チャンネル追加(パネルとツリーリストに追加
        private void addChannel(ServerInfo serverInfo, ChannelInfo chInfo)
        {
            String tag = getKeyStr(serverInfo, chInfo);
            if (textBoxMap.ContainsKey(tag))
            {
                return; //重複回避
            }
            
            addPanel(serverInfo, chInfo); //パネル追加
                
            foreach (TreeNode parent in chtree.Nodes)
            {
                ServerInfoTag parentTag = (ServerInfoTag)parent.Tag;
                if (serverInfo.index.CompareTo(parentTag.serverInfo.index) == 0)
                {
                    TreeNode chNode = new TreeNode();
                    chNode.Text = chInfo.name;
                    chNode.Tag = new ServerInfoTag(serverInfo, chInfo);
                    parent.Nodes.Add(chNode);
                }
            }
            
        }

        //login成功
        delegate void loginAckDelegate(ServerInfo serverInfo);
        public void loginAckInvoke(ServerInfo serverInfo)
        {
            this.Invoke(new loginAckDelegate(loginAck), new object[] { serverInfo });
        }
        private void loginAck(ServerInfo serverInfo)
        {
            foreach (TreeNode parent in chtree.Nodes)
            {
                ServerInfoTag parentTagInfo = (ServerInfoTag)parent.Tag;
                if (serverInfo.index == parentTagInfo.serverInfo.index)
                {
                    parent.Text = serverInfo.name;
                }
            }
            //serverInfo.connected = true; ※削除保留※

            String key = getKeyStr(serverInfo, null);
            writeMessage(HttpdAddMsgType.Sys, serverInfo, null, serverInfo.name + "　ログインしました");            
        }

        //join成功※受けスレッド用
        delegate void joinAckDelegate(ServerInfo serverInfo, ChannelInfo chInfo);
        public void joinAckInvoke(ServerInfo serverInfo, ChannelInfo chInfo)
        {
            this.Invoke(new joinAckDelegate(joinAck), new object[] { serverInfo, chInfo });
        }
        //join成功(内部
        private void joinAck(ServerInfo serverInfo, ChannelInfo chInfo)
        {
            String tag = getKeyStr(serverInfo, chInfo);
            if (textBoxMap.ContainsKey(tag) == false)
            {
                //チャンネルがなければ追加する
                addChannel(serverInfo, chInfo);
            }

            //ツリービューの表示を挿し直す
            foreach (TreeNode parentNode in chtree.Nodes)
            {
                ServerInfoTag parentTagInfo = (ServerInfoTag)parentNode.Tag;
                if (serverInfo.index.CompareTo(parentTagInfo.serverInfo.index) == 0)
                {
                    foreach (TreeNode childNode in parentNode.Nodes)
                    {
                        ServerInfoTag childTagInfo = (ServerInfoTag)childNode.Tag;
                        if (chInfo.index.CompareTo(childTagInfo.chInfo.index) == 0)
                        {   
                      //      childNode.Text = chInfo.name + "[入室]";
                            parentNode.Expand();
                        }
                    }
                }
            }

            //フォーカスが鯖ならパネル切り替え
            String parentKey = getKeyStr(serverInfo, null);
            if (textBoxMap[parentKey].Visible)
            {
                selectTreeView(new ServerInfoTag(serverInfo, chInfo));
            }

            //入室情報がなければ保存
            List<String> tmpCoSChs = new List<String>();
            foreach (ServerInfo server in AppCommon.setting.infos)
            {
                if (server.index.CompareTo(serverInfo.index) == 0)
                {
                    if (server.channels.Contains(chInfo.name) == false)
                    {
                        foreach (String chs in server.channels)
                        {
                            tmpCoSChs.Add(chs);
                        }
                        tmpCoSChs.Add(chInfo.name);
                        server.channels = tmpCoSChs.ToArray();
                        AppCommon.save();
                    }
                    break;
                }
            }

            // sysメッセージ
            writeMessage(HttpdAddMsgType.Join, serverInfo, chInfo, "チャット" + chInfo.name + "に入室しました。");

            chInfo.joined = true;

            updateChannnelText();//表記更新
        }

        //タイトル変更
        private void changeTitleText(ServerInfo serverInfo, ChannelInfo chInfo)
        {
            String title = AppCommon.APPNAME + " - ";
            if (chInfo == null)
            {
                title += serverInfo.name;
            }
            else
            {
                title += chInfo.name + ":" + chInfo.topic;
            }
            this.Text = title;
        }

        //Topic変更
        delegate void channelTopicDelegate(ServerInfo serverInfo, ChannelInfo chInfo);
        public void channelTopicInvoke(ServerInfo serverInfo, ChannelInfo chInfo)
        {
            this.Invoke(new channelTopicDelegate(channelTopic), new object[] { serverInfo, chInfo });
        }
        private void channelTopic(ServerInfo serverInfo, ChannelInfo chInfo)
        {
            String panelKey = getKeyStr(serverInfo, chInfo);
            if (textBoxMap[panelKey].Visible)
            {
                // 表示中チャンネルの場合
                changeTitleText(serverInfo, chInfo);
            }

            // メッセージ表示
            writeMessage(HttpdAddMsgType.Sys, serverInfo, chInfo, "トピック：" + chInfo.topic);
        }

        // 切断
        delegate void disconnectDelegate(ServerInfo serverInfo);
        public void disconnectInvoke(ServerInfo serverInfo)
        {
            this.Invoke(new disconnectDelegate(disconnect), new object[] { serverInfo });
        }
        private void disconnect(ServerInfo serverInfo)
        {
            String key = getKeyStr(serverInfo, null);
            //serverInfo.connected = false;    ※削除保留※        

            // 切断時既にパネルが無い場合がある
            foreach (TreeNode parent in chtree.Nodes)
            {
                ServerInfoTag parentTag = (ServerInfoTag)parent.Tag;
                if (serverInfo.index.CompareTo(parentTag.serverInfo.index) == 0)
                {
                    // チャンネル用のパネル、ユーザリストを削除
                    foreach (TreeNode child in parent.Nodes)
                    {
                        ServerInfoTag childTag = (ServerInfoTag)child.Tag;
                        String chKey = getKeyStr(childTag.serverInfo, childTag.chInfo);
                        lock(textBoxMap)
                        {
                            if (textBoxMap.ContainsKey(chKey))
                            {
                                textBoxMap[chKey].Visible = false;
                                textBoxMap.Remove(chKey);
                            }
                        }

                        lock (listViewMap)
                        {
                            if (listViewMap.ContainsKey(chKey))
                            {
                                listViewMap[chKey].Visible = false;
                                listViewMap.Remove(chKey);
                            }
                        }
                    }
                    
                    // treeviewの子要素をクリア、未接続に
                    parent.Nodes.Clear();
                    parent.Text = serverInfo.name + LABEL_NOTCONNECT;

                    //切断された場合を考慮して再フォーカス
                    selectTreeView(parentTag);
                    writeMessage(HttpdAddMsgType.Sys, serverInfo, null, serverInfo.name + "　切断しました。");
                }
            }

            //再接続
            if (AppCommon.setting.autoRetry && serverInfo.reconnectControlFlg)
            {
                //再接続待ち時間
                //660秒以上 -> 10秒
                //40 - 660秒 -> 600秒
                //15 - 40秒 -> 60秒
                //0 - 15秒 -> 30秒
                DateTime now = DateTime.Now;
                double subSec = now.Subtract(serverInfo.reconnectLastTime).TotalSeconds;
                String timeStr = DateTime.Now.ToString("HH:mm:ss");
                int sleepSec = 10;
                if (subSec < 15)
                {
                    sleepSec = 30 ;
                }
                else if (subSec < 40)
                {
                    sleepSec = 60  ;
                }
                else if (subSec < 660)
                {
                    sleepSec = 600 ;
                }
                
                serverInfo.reconnectLastTime = now;

                writeMessage(HttpdAddMsgType.Sys, serverInfo, null, "再接続予約します 待機時間:" + sleepSec + "秒");
                serverInfo.reconnectReserveKey = timeStr;
                Task readThread = Task.Factory.StartNew(() =>
                {
                    Thread.Sleep(sleepSec * 1000);
                    if (serverInfo.reconnectReserveKey != null && timeStr == serverInfo.reconnectReserveKey)
                    {
                        serverInfo.reconnectReserveKey = null;
                        ServerManager.connectServer(serverInfo);
                    }
                    else
                    {
                        Console.WriteLine("再接続キー相違のため破棄[" + timeStr + "/"+serverInfo.reconnectReserveKey+"]");
                    }
                });
            }
        }

        //チャンネル退室
        delegate void partChAckDelegate(ServerInfo serverInfo,ChannelInfo chInfo);
        public void partChAckInvoke(ServerInfo serverInfo, ChannelInfo chInfo)
        {
            this.Invoke(new partChAckDelegate(partChAck), new object[] { serverInfo, chInfo });
        }
        private void partChAck(ServerInfo serverInfo, ChannelInfo chInfo)
        {   
            //foreach (TreeNode parent in chtree.Nodes)
            //{
            //    TagInfo parentTag = (TagInfo)parent.Tag;
            //    if (serverInfo.index.CompareTo(parentTag.serverInfo.index) == 0)
            //    {
            //        foreach (TreeNode child in parent.Nodes)
            //        {
            //            TagInfo childTag = (TagInfo)child.Tag;
            //            if(chInfo.index.CompareTo(childTag.chInfo.index) == 0){
                            // 切断状態にする
                           // childTag.chInfo.joined = false;
                           // child.Text = childTag.chInfo.name; // 表記の切り替え

                            // 切断メッセージ
                            String key = getKeyStr(serverInfo, chInfo);
                            writeMessage(HttpdAddMsgType.Part, serverInfo, chInfo, "チャット" + chInfo.name + "を退室しました。");
                            if (listViewMap.ContainsKey(key)) 
                            {
                                listViewMap[key].Items.Clear(); //ユーザリストクリア
                            }
                            chInfo.userList.Clear();
            //            }
            //        }
            //    }
            //}

            updateChannnelText();

            //切断情報の保存
            List<String> tmpCoSChs = new List<String>();
            foreach (ServerInfo server in AppCommon.setting.infos)
            {
                if (server.index.CompareTo(serverInfo.index) == 0)
                {
                    foreach (String chs in server.channels)
                    {
                        if (chInfo.name.CompareTo(chs) != 0)
                        {
                            tmpCoSChs.Add(chs);
                        }
                    }
                    server.channels = tmpCoSChs.ToArray();
                    AppCommon.save();
                    break;
                }
            }

        }

        // メニュー->設定
        private void configToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormConfig configForm = new FormConfig(this);
            configForm.StartPosition = FormStartPosition.CenterParent;
            configForm.ShowDialog(this);
            configForm.Dispose();
        }

        // 共有機能ドロップダウン開いた時
        private void tsMenuItemShareTool_DropDownOpening(object sender, EventArgs e)
        {
            tsMenuItemUpdateShareList.Enabled = false;
            if(UploadManager.isRefleshOk()){
                if (ServerManager.getLoginedServerInfoList().Count() > 0)
                {
                    tsMenuItemUpdateShareList.Enabled = true;
                }
            }
        }

        // 共有機能 -> 検索窓新規
        private void tsMenuItemSearchWindowOpenNew_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem mi = (ToolStripMenuItem)sender;
            if (mi.Tag != null)
            {
                FormSearch searchForm = (FormSearch)mi.Tag;
                searchForm.TopMost = true;
                searchForm.TopMost = false;
            }
            else
            {
                FormSearch.openNew(this);
            }
        }

        // 共有機能 -> 転送窓
        private void tsMenuItemTransWindow_Click(object sender, EventArgs e)
        {
            FormTransfer.openAndReflesh();
        }

        // 共有機能 -> 共有リスト更新
        private void tsMenuItemUpdateShareList_Click(object sender, EventArgs e)
        {
            UploadManager.initSharedFileInfo();
        }

//×        // メニュー->サーバー追加
//        private void addServerToolStripMenuItem_Click(object sender, EventArgs e)
//        {
//            formServerEditOpen(null);
//        }
        private void formServerEditOpen(ServerInfo info){
            FormServerEdit serverEditForm = new FormServerEdit(this, info);
            serverEditForm.StartPosition = FormStartPosition.CenterParent;
            serverEditForm.ShowDialog(this);
            serverEditForm.Dispose();
        }
        //サーバー追加/編集ダイアログの戻り
        public void formServerEditCallback(ServerInfo serverInfo, bool editMode)
        {
            if (editMode)
            {
                // 画面に反映
                foreach (TreeNode parent in chtree.Nodes)
                {
                    ServerInfoTag parentTag = (ServerInfoTag)parent.Tag;
                    if (serverInfo.index.CompareTo(parentTag.serverInfo.index) == 0)
                    {
                        parent.Text = serverInfo.name + LABEL_NOTCONNECT;
                    }
                }
            }
            else
            {
                ServerInfo[] newInfos = new ServerInfo[AppCommon.setting.infos.Length + 1];
                Array.Copy(AppCommon.setting.infos, newInfos, AppCommon.setting.infos.Length);
                newInfos[newInfos.Length - 1] = serverInfo;
                AppCommon.setting.infos = newInfos;

                // 鯖を追加
                addServer(serverInfo);

                if (serverInfo.connectOnStart)
                {
                    ServerManager.connectServer(serverInfo);
                }
            }

            AppCommon.save();
        }

        // 履歴 -> 呼出履歴
        private void tsMenuItemHistoryCalling_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem mi = (ToolStripMenuItem)sender;
            CallingHistoryTag tag = (CallingHistoryTag)mi.Tag;
            FormCalling.proc(this, tag.timeStr, tag.pattern, tag.message1, tag.message2, tag.matchStartIdx);
        }

        // 履歴 -> URL履歴
        private void tsMenuItemHistoryURL_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem mi = (ToolStripMenuItem)sender;            
            System.Diagnostics.Process.Start((String)mi.Tag);
        }

        // ヘルプ->バージョン
        private void versionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox1 formVersion = new AboutBox1();
            formVersion.StartPosition = FormStartPosition.CenterParent;
            formVersion.ShowDialog(this);
            formVersion.Dispose();
        }

        // メインパネルでキーボード押下(beep音抑止
        private void defaultTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }

        // メインパネル右クリックメニュー開く時
        private void ctxMenuStripTxtPanel_Opening(object sender, CancelEventArgs e)
        {
            tsMenuItemTxtPanelSearch.Enabled = AppCommon.setting.enableShare;
        }

        // メインパネルで右クリックコピー
        private void tsMenuItemTxtPanelCopy_Click(object sender, EventArgs e)
        {
            if (currentTargetTag != null)
            {
                String key = getKeyStr(currentTargetTag.serverInfo, currentTargetTag.chInfo);
                if (textBoxMap.ContainsKey(key))
                {
                    RichTextBox text = textBoxMap[key];
                    text.Copy();
                }
            }
        }

        // メインパネルで右クリック共有検索
        private void tsMenuItemTxtPanelSearch_Click(object sender, EventArgs e)
        {
            if (currentTargetTag != null)
            {
                String key = getKeyStr(currentTargetTag.serverInfo, currentTargetTag.chInfo);
                if (textBoxMap.ContainsKey(key))
                {
                    RichTextBox text = textBoxMap[key];
                    //text.Copy();
                    FormSearch searchWnd = FormSearch.openNew(this);
                    searchWnd.execSearch(text.SelectedText);
                    //Console.WriteLine(text.SelectedText);
                }
            }
        }

        // メインFormリサイズ
        private void MainForm_Resize(object sender, EventArgs e)
        {
            // 最小化イベントでタスクトレイ格納する場合
            if (this.WindowState == FormWindowState.Minimized)
            {
                if (AppCommon.setting.taskTray)
                {
                    this.ShowInTaskbar = false;
                }
            }
        }

        // メインFormリサイズ
        private void MainForm_ResizeEnd(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Normal && AppCommon.setting.saveWindowPos)
            {
                AppCommon.setting.windowWidth = this.Width;
                AppCommon.setting.windowHeight = this.Height;
                AppCommon.save();
            }
        }

        // メインForm移動
        private void MainForm_Move(object sender, EventArgs e)
        {
            if (AppCommon.setting.saveWindowPos)
            {
                AppCommon.setting.windowPosX = this.Location.X;
                AppCommon.setting.windowPosY = this.Location.Y;
                AppCommon.save();
            }
        }

        // 色設定を反映する
        public void reflectColorEdit()
        {
            this.inputbox.BackColor = AppCommon.setting.colorInfo.colorBackground;
            this.inputbox.ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
            this.inputbox.Font = AppCommon.setting.txtFont;
            this.chtree.BackColor = AppCommon.setting.colorInfo.colorBackground;
            this.chtree.ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
            this.chtree.Font = AppCommon.setting.txtFont;
            this.defaultTextBox.BackColor = AppCommon.setting.colorInfo.colorBackground;
            this.defaultTextBox.ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
            this.defaultTextBox.Font = AppCommon.setting.txtFont;
            this.userList.BackColor = AppCommon.setting.colorInfo.colorBackground;
            this.userList.ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
            this.userList.Font = AppCommon.setting.txtFont;

            foreach (String key in textBoxMap.Keys)
            {
                textBoxMap[key].BackColor = AppCommon.setting.colorInfo.colorBackground;
                textBoxMap[key].Font = AppCommon.setting.txtFont;
                textBoxMap[key].ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
            }
            foreach (String key in listViewMap.Keys)
            {
                listViewMap[key].BackColor = AppCommon.setting.colorInfo.colorBackground;
                listViewMap[key].Font = AppCommon.setting.txtFont;
                listViewMap[key].ForeColor = AppCommon.setting.colorInfo.colorNormalMsg;
            }
        }

        //  共有モード変更反映
        public void reflectSharedMode()
        {
            if (AppCommon.setting.enableShare)
            {
                tsMenuItemShareTool.Visible = true;
            }
            else
            {
                tsMenuItemShareTool.Visible = false;

                //開いてたら閉じる
                FormSearch.closeALL();
                FormTransfer.stopALL();
                FormPersonalBrowse.closeALL();
            }
        }

        //ユーザ情報表示
        delegate void showUserInfoDelegate(String[] infos);
        public void showUserInfoInvoke(String[] infos)
        {
            this.Invoke(new showUserInfoDelegate(showUserInfo), new object[] { infos });
        }
        private void showUserInfo(String[] infos)
        {
            //<nick> "<user-level>" <time> "<channels>" "<status>" <shared> <downloads> <uploads> <link-type> "<client-info>" [ <total downloads> <total_uploads> <ip> <connecting port> <data port> <email> ]
            //1626<-[000066][604][user2 User 2219 "#moyare " "Active" 0 0 0 0 "OpenNapChat 1.05.622"]
            //Console.WriteLine("whois戻り:" + message);            
            FormUserInfo uiform = new FormUserInfo(infos);
            uiform.StartPosition = FormStartPosition.CenterParent;
            this.AddOwnedForm(uiform);
            uiform.Show();

            if (infos.Length >= 6)
            {
                updateSharedFileList(infos[0], infos[5]);
            }

        }

        // 検索窓追加時に呼ばれる
        public void openNewSearchWindow(FormSearch searchForm)
        {
            ToolStripMenuItem tsmi = new ToolStripMenuItem();
            tsmi.Name = searchForm.indexedName;
            tsmi.Size = new System.Drawing.Size(152, 22);
            tsmi.Text = "未検索";
            tsmi.Tag = searchForm;
            tsmi.Click += new System.EventHandler(this.tsMenuItemSearchWindowOpenNew_Click);
            this.tsMenuItemSearchWindow2.DropDownItems.Add(tsmi);
            searchForm.parentTsmi = tsmi;
            menuItemSearchWindowChange();
        }

        // 検索窓閉じる時に呼ばれる
        public void closeSearchWindow(FormSearch searchForm)
        {
            this.tsMenuItemSearchWindow2.DropDownItems.Remove(searchForm.parentTsmi);
            menuItemSearchWindowChange();
        }

        // 検索窓メニューの表示制御
        private void menuItemSearchWindowChange()
        {
            if (FormSearch.instanceList.Count > 0)
            {                
                tsMenuItemSearchWindow1.Visible = false;
                tsMenuItemSearchWindow2.Visible = true;
            }
            else
            {
                tsMenuItemSearchWindow1.Visible = true;
                tsMenuItemSearchWindow2.Visible = false;
            }
        }

        // チャンネルツリーの表記更新
        public void updateChannnelText()
        {
            //ツリービューの表示を挿し直す
            foreach (TreeNode parentNode in chtree.Nodes)
            {
                ServerInfoTag parentTagInfo = (ServerInfoTag)parentNode.Tag;
             
                foreach (TreeNode childNode in parentNode.Nodes)
                {
                    ServerInfoTag childTagInfo = (ServerInfoTag)childNode.Tag;

                    String chText = childTagInfo.chInfo.name;

                    chText += "["+childTagInfo.chInfo.userNum+"人]";
                    
                    if (childTagInfo.chInfo.joined)
                    {
                        chText += "[入室]";
                    }

                    if (childNode.Text != chText)
                    {
                        childNode.Text = chText;
                    }
                }
            }

        }

        // サーバ情報タグ
        private class ServerInfoTag
        {
            public ServerInfo serverInfo = null;
            public ChannelInfo chInfo = null;

            public ServerInfoTag(ServerInfo serverInfo, ChannelInfo chInfo)
            {
                this.serverInfo = serverInfo;
                this.chInfo = chInfo;
            }
        }

        // 呼出履歴復帰用タグ
        private class CallingHistoryTag
        {
            public String timeStr;
            public String pattern;
            public String message1;
            public String message2;
            public int matchStartIdx;
            public CallingHistoryTag(String timeStr, String pattern, String message1, String message2, int matchStartIdx)
            {
                this.timeStr = timeStr;
                this.pattern = pattern;
                this.message1 = message1;
                this.message2 = message2;
                this.matchStartIdx = matchStartIdx;
            }
        }

        // ----------------------------- デバッグ用 -----------------------------
        private void message2CurrentPanel(String message)
        {
            if (currentTargetTag != null)
            {
                writeMessage(HttpdAddMsgType.Sys, currentTargetTag.serverInfo, currentTargetTag.chInfo, message);
            }
        }

        private void testItem1_Click(object sender, EventArgs e)
        {
            AppCommon.setting.disableDebugProtocol = (AppCommon.setting.disableDebugProtocol == false);
            Console.WriteLine("C2Smessage:" + AppCommon.setting.disableDebugProtocol);
            AppCommon.save();
        }

        private void test2ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Console.WriteLine("デバッグ2★");

        }

   }
}


//てんぷれ
//delegate void **Delegate(ServerInfo serverInfo);
//public void **Invoke(ServerInfo serverInfo)
//{
//    this.Invoke(new **Delegate(**), new object[] { serverInfo });
//}
//private void **(ServerInfo serverInfo)
//{
//}


//FormSearchBlock blockFormLc = new FormSearchBlock();
//blockFormLc.StartPosition = FormStartPosition.CenterParent;
//blockFormLc.ShowDialog(parent);
//blockFormLc.Dispose();


//FormSearchBlock blockFormLc = new FormSearchBlock();
//parent.AddOwnedForm(blockFormLc);
//blockFormLc.StartPosition = FormStartPosition.Manual;
//blockFormLc.Location = new Point(parent.Location.X + (parent.Width - blockFormLc.Width) / 2, parent.Location.Y + (parent.Height - blockFormLc.Height) / 2);
//blockFormLc.Show();