﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Net.Sockets;
using System.IO;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace MoyariClient
{
    // staticサーバ管理
    public class ServerManager
    {
        public static Encoding ENC_SJIS = Encoding.GetEncoding("Shift_JIS");

        // 画面
        public static MainForm mainForm = null;

        // サーバ接続保持　key=serverIndex
        private static Dictionary<String, ServerConnection> serverConnectMap = new Dictionary<String, ServerConnection>();
        
        // メッセージ処理SC(メッセージ)
        public static void s2cMessage(ServerInfo serverInfo, MsgType type, byte[] body, int msgType = 0)
        {
            String msg = ENC_SJIS.GetString(body);

#if DEBUG
            if (AppCommon.setting.disableDebugProtocol == false)
            {
                Console.WriteLine("S2Cメッセージ[" + serverInfo.name + "][" + type + "][" + msg + "]");
            }
#endif

            ServerConnection con = serverConnectMap[serverInfo.index];
            if (con.isConnected == false)
            {
                return;
            }
            try
            {
                switch (type)
                {
                    case MsgType.LOGIN_ACK:
                        //ログイン成功
                        mainForm.loginAckInvoke(serverInfo);
                        con.isLogined = true;
                        con.sendMessage(MsgType.LIST_CHANNEL, ""); // 接続時チャンネルリスト要求

                        // 共有有効な場合
                        if (AppCommon.setting.enableShare)
                        {
                            UploadManager.sendSharedFileList(serverInfo);
                        }
                        break;
                    case MsgType.CHANNEL_ENTRY:
                        //Format: <channel-name> <number-of-users> <topic>
                        String[] ceParts = stringSplitter(msg, 3);
                        String chName = ceParts[0];

                        ChannelInfo chInfo = con.getChannel(chName);
                        chInfo.userNum = ceParts[1];
                        chInfo.topic = ceParts[2];

                        mainForm.updateChannelListEntryInvoke(serverInfo, chInfo);

                        if (serverInfo.channels != null)
                        {
                            foreach (String ccName in serverInfo.channels)
                            {
                                if (ccName == chName && chInfo.joined == false) //自動初回join
                                {
                                    joinCh(serverInfo, chName);
                                }
                            }
                        }

                        break;
                    case MsgType.JOIN_ACK:
                        mainForm.joinAckInvoke(serverInfo, con.getChannel(msg));
                        break;
                    case MsgType.MESSAGE_OF_DAY:
                        mainForm.addServerMmessageInvoke(serverInfo, msg);
                        break;
                    case MsgType.SERVER_STATUS:
                        //server: <users> <# files> <size>　使うならユーザ数が取れる　keepAliveの更新をする？
                        break;
                    case MsgType.LIST_CHANNEL:
                        //reqの戻り keepAliveの更新をする？
                        break;
                    case MsgType.END_CH_USER_LIST:
                        //捨ててよし
                        break;
                    case MsgType.CHANNEL_USER_LIST:
                        //<channel> <user> <sharing> <link-type>
                        String[] ulParts = stringSplitter(msg, 4);
                        ChannelInfo ulChInfo = con.getChannel(ulParts[0]);
                        UserInfo ulUser = new UserInfo(ulParts[1], ulParts[2], ulParts[3]);
                        mainForm.userJoinedInvoke(serverInfo, ulChInfo, ulUser, true);
                        break;
                    case MsgType.CHANNEL_TOPIC:
                        //<channel> <topic>
                        String[] ctParts = stringSplitter(msg, 2);
                        ChannelInfo ctChInfo = con.getChannel(ctParts[0]);
                        ctChInfo.topic = ctParts[1];
                        mainForm.channelTopicInvoke(serverInfo, ctChInfo);
                        break;
                    case MsgType.USER_JOINED_CH:
                        //<channel> <user> <sharing> <link-type>
                        String[] ujParts = stringSplitter(msg, 4);
                        ChannelInfo ujChInfo = con.getChannel(ujParts[0]);
                        UserInfo ujUser = new UserInfo(ujParts[1], ujParts[2], ujParts[3]);
                        mainForm.userJoinedInvoke(serverInfo, ujChInfo, ujUser, false);
                        break;
                    case MsgType.USER_PARTED_CH:
                        //<channel> <nick> <sharing> <linespeed>
                        String[] upParts = stringSplitter(msg, 4);
                        ChannelInfo upChInfo = con.getChannel(upParts[0]);
                        mainForm.userPartedInvoke(serverInfo, upChInfo, upParts[1]);
                        break;
                    case MsgType.PUBLIC_MESSAGE:
                        //<channel> <nick> <text>
                        String[] mpParts = stringSplitter(msg, 3);
                        ChannelInfo mpChInfo = con.getChannel(mpParts[0]);
                        mainForm.addChMmessageInvoke(serverInfo, mpChInfo, mpParts[1], mpParts[2], false);
                        break;
                    case MsgType.PART_CHANNEL:
                        // <channel-name>
                        ChannelInfo pcChInfo = con.getChannel(msg);
                        pcChInfo.joined = false;
                        //pcChInfo.joinRequested = false;
                        mainForm.partChAckInvoke(serverInfo, pcChInfo);
                        break;
                    case MsgType.ERROR:
                        mainForm.addServerMmessageInvoke(serverInfo, "ERROR:" + msg);
                        break;
                    case MsgType.LOGIN_ATTENPT:
                        mainForm.addServerMmessageInvoke(serverInfo, "ログイン状態が割り込まれました");
                        break;
                    case MsgType.ERROR_MESSAGE:
                        mainForm.addServerMmessageInvoke(serverInfo, "エラーメッセージ：" + msg);
                        break;
                    case MsgType.CLIENT_DISCONNECT:
                        disconnectC2Server(serverInfo);
                        break;
                    case MsgType.PRIVATE_MESSAGE:
                        //<nick> <message>
                        String[] pmParts = stringSplitter(msg, 2);

                        if (AppCommon.UL_QUEUE_MSG.ToUpper() == pmParts[1].ToUpper())
                        {
                            UploadManager.wantQueueRecv(pmParts[0]);
                        }
                        else
                        {
                            ChannelInfo pmChInfo = con.getPrivChannel(pmParts[0]);
                            mainForm.addChMmessageInvoke(serverInfo, pmChInfo, pmParts[0], pmParts[1], true);
                        }
                        break;
                    case MsgType.WHOIS_RESPONSE:
                        //<nick> "<user-level>" <time> "<channels>" "<status>" <shared> <downloads> <uploads> <link-type> "<client-info>" [ <total downloads> <total_uploads> <ip> <connecting port> <data port> <email> ]
                        //1626<-[000066][604][user2 User 2219 "#moyare " "Active" 0 0 0 0 "OpenNapChat 1.05.622"]                    
                        String[] wrInfos = AppCommon.splitMessageToString(msg, 10);
                        mainForm.showUserInfoInvoke(wrInfos);
                        break;
                    case MsgType.SEARCH_RESPONCE:
                        String[] srInfos = AppCommon.splitMessageToString(msg, 10);
                        //	"<filename>" <md5> <size> <bitrate> <frequency> <length> <nick> <ip> <link-type> [weight]                    
                        FormSearch.receiveSearchResponse(new SearchResult(serverInfo, srInfos[0], srInfos[2], srInfos[6], srInfos[8]));
                        break;
                    case MsgType.SEARCH_COMPLETE:
                        FormSearch.receiveSearchComplete(serverInfo);
                        break;
                    case MsgType.DOWNLOAD_ACK:
                        //<nick> <ip> <port> "<filename>" <md5> <linespeed>
                        String[] daParts = AppCommon.splitMessageToString(msg, 6);
                        DownloadManager.downloadAcked(serverInfo, new DownloadAckInfo(daParts));
                        break;
                    case MsgType.QUEUE_LIMIT_S2C:
                        //<nick> "<filename>" <filesize> <digit>
                        String[] qlParts = AppCommon.splitMessageToString(msg, 4);
                        DownloadManager.downloadQueueLimit(serverInfo, qlParts[0], qlParts[1], qlParts[2], qlParts[3]);
                        break;
                    case MsgType.CLEAR_IGNORE_LIST:
                        //server: <count>
                        Console.WriteLine("CLEAR_IGNORE_LIST receive:" + msg);
                        break;
                    case MsgType.UPLOAD_REQUEST:
                        // <nick> "<filename>" <speed>
                        String[] urParts = AppCommon.splitMessageToString(msg, 3);
                        UploadManager.receiveUploadRequest(serverInfo, urParts[0], urParts[1], urParts[2]);
                        break;
                    case MsgType.ALTERNATE_DL_ACK:
                        // <nick> <ip> <port> "<filename>" <md5> <speed>
                        String[] adaParts = AppCommon.splitMessageToString(msg, 6);
                        UploadManager.alternateDownloadAckThread(serverInfo, adaParts[0], adaParts[1], adaParts[2], adaParts[3], adaParts[4], adaParts[5]);
                        break;
                    case MsgType.BROWSE_USER_RESPONSE:
                        String[] burParts = AppCommon.splitMessageToString(msg, 7);
                        //<nick> "<filename>" <md5> <size> <bitrate> <frequency> <time>
                        //<nick> is the user contributing the file
                        //<filename> is the mp3 file contributed
                        //<md5> is the has of the mp3 file
                        //<size> is the file size in bytes
                        //<bitrate> is the mp3 bitrate in kbps
                        //<frequence> is the sampling frequency in Hz
                        //<time> is the play time in seconds
                        //ServerInfo serverInfo, String user, String filePath, String fileSizeStr)

                        //mainForm.openBrowseWindow(new SearchResult(serverInfo, burParts[1], burParts[3], burParts[0], "0"));

                        FormPersonalBrowse.addResult(new SearchResult(serverInfo, burParts[1], burParts[3], burParts[0], "0"));
                        break;
                    case MsgType.BROWSE_USER_COMPLETE:
                        String[] bucParts = AppCommon.splitMessageToString(msg, 2);
                        //213 (0xd5)	end of browse list [SERVER]
                        //<nick> [ip]
                        FormPersonalBrowse.ackComplete(serverInfo, bucParts[0]);
                        break;
                    case MsgType.PONG_RESPONSE:
                        //nothing
                        break;
                    default: //UNKNOWN
                        mainForm.addServerMmessageInvoke(serverInfo, "不明なメッセージ種別(" + msgType.ToString() + "):" + msg);
                        break;
                }
            }
            catch (Exception e)
            {
                if (con.isConnected)
                {
                    Task keepaliveThread = Task.Factory.StartNew(() =>
                   {
                        AppCommon.dumpException(e, "ハンドルできない致命的な例外 type:" + type + " msg:" + msg);
                        MessageBox.Show("ハンドルできない致命的な例外\ntype:" + type + " msg:" + msg + "\n" + e.Message + "\n" + e.StackTrace,
                            "エラー",
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error);                       
                   });
                }
            }
        }

        // Serverエラーメッセージ
        public static void serverErrMsgToScreen(ServerInfo serverInfo, String msg, Exception e)
        {
            String errMsg = msg + "\n" + e.Message + "\n" + e.StackTrace;
            Console.WriteLine(errMsg);
            mainForm.addServerMmessageInvoke(serverInfo, errMsg);
        }

        // 文字列操作補助
        private static String[] stringSplitter(String str, int num)
        {
            String[] result = new String[num];
            for (int i = 0; i < num - 1; i++)
            {
                int len = str.IndexOf(' ');
                if (len >= 0)
                {
                    result[i] = str.Substring(0, len);
                    str = str.Substring(len + 1);
                }
            }
            result[num - 1] = str;
            return result;
        }

        //  メッセージ処理CS(パブリックメッセージ) privは別
        public static void c2sPublicMessage(ServerInfo serverInfo, ChannelInfo chInfo, String message)
        {
            Console.WriteLine("メッセージ処理CS [" + serverInfo.name + "](" + chInfo.name + ")[" + message + "]");
            ServerConnection con = serverConnectMap[serverInfo.index];
            if (con != null)
            {
                if (con.isConnected)
                {
                    //con.publicMessage(chInfo, message);
                    con.sendMessage(MsgType.SEND_PUBLIC_MESSAGE, chInfo.name + " " + message);
                }
                else
                {
                    mainForm.addServerMmessageInvoke(serverInfo, "エラー　メッセージ送信対象のServerConnectionが未接続");
                }
            }
            else
            {
                mainForm.addServerMmessageInvoke(serverInfo, "エラー　接続対象のServerConnectionが取れない");
            }
        }

        //  接続開始 formから呼ばれる
        public static void connectServer(ServerInfo serverInfo)
        {
            if (serverInfo.reconnectReserveKey != null)
            {
                mainForm.addServerMmessageInvoke(serverInfo, "再接続予約を取り消します");
                serverInfo.reconnectReserveKey = null;
            }

            Task connectThread = Task.Factory.StartNew(() =>
            {
                Console.WriteLine("接続開始依頼 [" + serverInfo.name + "]");
                serverInfo.reconnectControlFlg = true; //再接続予約              
                ServerConnection con = serverConnectMap[serverInfo.index];
                if (con != null)
                {
                    if (con.isConnected == false)
                    {
                        mainForm.addServerMmessageInvoke(serverInfo, serverInfo.name + "　接続します");
                        con.connect();
                    }
                    else
                    {
                        mainForm.addServerMmessageInvoke(serverInfo, "エラー　対象は接続済み");
                    }
                }
                else
                {
                    mainForm.addServerMmessageInvoke(serverInfo, "エラー　接続対象のServerConnectionが取れない");
                }
            });
        }

        // 接続状態確認
        public static bool checkServerIsConnected(ServerInfo serverInfo)
        {   
            ServerConnection con = serverConnectMap[serverInfo.index];
            if (con != null)
            {
                return con.isConnected;
            }
            return false;
        }

        //  切断依頼(クラ->サバ
        public static void disconnectC2Server(ServerInfo serverInfo)
        {
            ServerConnection con = serverConnectMap[serverInfo.index];
            con.disConnect();
            //Task connectThread = Task.Factory.StartNew(() =>
            //{
            //    ServerConnection con = serverConnectMap[serverInfo.index];
            //    if (con != null)
            //    {
            //        //if (con.isConnected)
            //        //{
            //        con.disConnect();
            //        //}
            //        //else
            //        //{
            //        //    Console.WriteLine("切断対象のServerConnectionが切断済み");
            //        //}
            //    }
            //    else
            //    {
            //        mainForm.addServerMmessageInvoke(serverInfo, "エラー　切断対象のServerConnectionが取れない");
            //    }
            //});
        }
        // 切断(サバ->クラ情報反映
        public static void disconnectS2Client(ServerInfo serverInfo)
        {
            mainForm.disconnectInvoke(serverInfo);
        }

        //  サーバ削除(鯖情報) formから呼ばれる
        public static void removeSertverInfo(ServerInfo serverInfo)
        {
            if (serverInfo.reconnectReserveKey != null)
            {
                mainForm.addServerMmessageInvoke(serverInfo, "再接続予約を取り消します");
                serverInfo.reconnectReserveKey = null;
            }

            if (serverConnectMap.ContainsKey(serverInfo.index))
            {
                lock (serverConnectMap)
                {
                    serverConnectMap.Remove(serverInfo.index);
                }
            }
        }

        //  サーバ追加(鯖情報) formから呼ばれる
        public static void addSertverInfo(ServerInfo serverInfo)
        {
            Console.WriteLine("鯖情報追加 [" + serverInfo.name + "]");
            serverConnectMap.Add(serverInfo.index, new ServerConnection(serverInfo));
        }

        // チャンネルに入る(リクエストのみ)
        public static void joinCh(ServerInfo serverInfo, String chName)
        {
            ServerConnection con = serverConnectMap[serverInfo.index];
            con.sendMessage(MsgType.JOIN_CHANNEL, chName); 
        }

        // チャンネルを出る(リクエストのみ)
        public static void partChC2Server(ServerInfo serverInfo, String chName)
        {
            ServerConnection con = serverConnectMap[serverInfo.index];
            con.sendMessage(MsgType.PART_CHANNEL, chName); 
        }        

        // whois(リクエストのみ)
        public static void whoisC2Server(ServerInfo serverInfo, String userName)
        {
            ServerConnection con = serverConnectMap[serverInfo.index];
            con.sendMessage(MsgType.WHOIS_REQUEST, userName); 
        }

        // Privチャット用チャンネル登録・取得
        public static ChannelInfo getPrivChatChannelInfo(ServerInfo serverInfo, String userName)
        {
            ServerConnection con = serverConnectMap[serverInfo.index];
            return con.getPrivChannel(userName);
        }

        // Privateメッセージ送信
        public static void c2sPrivateMessage(ServerInfo serverInfo, ChannelInfo chInfo, String message)
        {
            //Privはエコーバックしないので自前
            mainForm.addChMmessageInvoke(serverInfo, chInfo, serverInfo.user, message, true);
            ServerConnection con = serverConnectMap[serverInfo.index];
            //con.privateMessage(chInfo, message);
            con.sendMessage(MsgType.PRIVATE_MESSAGE, chInfo.name + " " + message);
        }

        // 鯖key->鯖Info
        public static ServerInfo getServerInfoByIndex(String serverIndex)
        {
            if (serverConnectMap.ContainsKey(serverIndex))
            {
                return serverConnectMap[serverIndex].serverInfo;
            }
            return null;
        }

        // chKey->chInfo
        public static ChannelInfo getChInfoByIndex(String serverIndex,String chIndex)
        {
            if (serverConnectMap.ContainsKey(serverIndex))
            {
                return serverConnectMap[serverIndex].getChannelIndex(chIndex);
            }
            return null;
        }

        // いきてるサーバ数
        public static ServerInfo[] getLoginedServerInfoList()
        {
            List<ServerInfo> result = new List<ServerInfo>();
            foreach (String key in serverConnectMap.Keys)
            {
                if (serverConnectMap[key].isLogined)
                {
                    result.Add(serverConnectMap[key].serverInfo);
                }
            }
            return result.ToArray();
        }
        // 検索実行
        public static void searchRequest(SearchKeyInfo info)
        {
//TODO拡張検索時に拡張(鯖指定)
            foreach(String key in serverConnectMap.Keys){
                //serverConnectMap[key].searchRequest(info);
                serverConnectMap[key].sendMessage(MsgType.SEARCH_REQUEST, info.toRequestString());
            }
        }

        // DLリクエスト
        public static void donwloadRequest(DownloadTaskInfo task)
        {   
            ServerInfo serverInfo = task.serverInfo;            
            ServerConnection con = serverConnectMap[serverInfo.index];
            con.sendMessage(MsgType.DOWNLOAD_REQUEST, task.searchResult.nick + " \""+ task.searchResult.filepath + "\""); 
        }

        // DL開始
        public static void donwloadStart(DownloadTaskInfo task)
        {
            ServerInfo serverInfo = task.serverInfo;
            ServerConnection con = serverConnectMap[serverInfo.index];
            con.sendMessage(MsgType.DOWNLOAD_START, "");//bodyなし
        }

        // DL完了
        public static void donwloadComplete(DownloadTaskInfo task)
        {
            ServerInfo serverInfo = task.serverInfo;
            ServerConnection con = serverConnectMap[serverInfo.index];
            con.sendMessage(MsgType.DOWNLOAD_COMPLETE, "");//bodyなし
        }

        // DL完了共有追加
        public static void notifyShareFile(ServerInfo serverInfo, String filePath, long fileSize)
        {
            //Format: "<filename>" <md5> <size> <bitrate> <frequency> <time>
            //["filepath" 00000000000000000000000000000000 8847359 24 16000 600]
            ServerConnection con = serverConnectMap[serverInfo.index];
            String body = "\"" + filePath + "\" " + AppCommon.DUMMY_MD5 + " " + fileSize + " 24 16000 600"; 
            con.sendMessage(MsgType.NOTIFY_SHARED_FILE_, body);            
        }

        // remove file
        public static void removeFile(ServerInfo serverInfo, String filePath)
        {
            ServerConnection con = serverConnectMap[serverInfo.index];
            con.sendMessage(MsgType.REMOVE_SHARED_FILE_, "\"" + filePath + "\"");
        }

        // 110unshare all files [CLIENT] UNSHARE_ALL_FILES
        public static void unShareAllFiles_(ServerInfo serverInfo)
        {
            ServerConnection con = serverConnectMap[serverInfo.index];            
            con.sendMessage(MsgType.UNSHARE_ALL_FILES, "");            
        }

        //change link speed 
        public static void changeLinkSpeed(ServerInfo serverInfo)
        {
            ServerConnection con = serverConnectMap[serverInfo.index];            
            con.sendMessage(MsgType.CHANGE_LINK_SPEED, AppCommon.setting.linktype);                           
        }

        //change data port
        public static void changeDataPort(ServerInfo serverInfo)
        {
            String portStr = AppCommon.setting.shareWaitPort.ToString();
            ServerConnection con = serverConnectMap[serverInfo.index];                
            con.sendMessage(MsgType.CHANGE_DATA_PORT, portStr);
        }

        // 共有ファイル追加(870
        //ADD_SHARED_FILE
        public static void sendShareFileList(ServerInfo serverInfo, String message)
        {
            ServerConnection con = serverConnectMap[serverInfo.index];
            con.sendMessage(MsgType.ADD_FILE_BY_DIR, message);         
        }

        // wantQueue
        public static void wantQueueRequest(DownloadTaskInfo task)
        {
            ServerInfo serverInfo = task.serverInfo;
            ServerConnection con = serverConnectMap[serverInfo.index];
            con.sendMessage(MsgType.PRIVATE_MESSAGE, task.searchResult.nick + " //WantQueue");
        }

        // accept upload request
        public static void acceptUploadRequest(UploadTaskInfo task)
        {
            ServerInfo serverInfo = task.serverInfo;
            ServerConnection con = serverConnectMap[serverInfo.index];
            //task
            con.sendMessage(MsgType.UPLOAD_REQUEST_ACK, task.user + " \"" + task.filePath + "\"");
        }

        public static void uploadingStart(UploadTaskInfo task)
        {
            //220 (0xdc)	uploading file [CLIENT] no body.
            ServerInfo serverInfo = task.serverInfo;
            ServerConnection con = serverConnectMap[serverInfo.index];
            con.sendMessage(MsgType.UPLOAD_START, "");
        }

        public static void uploadComplete(UploadTaskInfo task)
        {
            //221 (0xdd)	upload complete [CLIENT]			no body.
            ServerInfo serverInfo = task.serverInfo;
            ServerConnection con = serverConnectMap[serverInfo.index];
            con.sendMessage(MsgType.UPLOAD_COMPLETE, "");
        }

        //Queue可能な場合
        public static void uploadQueueLimit(ServerInfo serverInfo, String nick, String filePath, String waitNum)
        {
            //752 (0x2f0)	pong response [CLIENT, SERVER]
            //619 (0x26b)	queue limit [CLIENT]
            //	Format: <nick> "<filename>" <n>
            ServerConnection con = serverConnectMap[serverInfo.index];
            con.sendMessage(MsgType.PONG_RESPONSE, nick);
            con.sendMessage(MsgType.QUEUE_LIMIT_C2S, nick + " \"" + filePath + "\" " + waitNum);// + waitNum
        }

        public static void altDownloadRequest(ServerInfo serverInfo, DownloadAckInfo ackInfo)
        {
            //500 (0x1f4)	alternate download request [CLIENT] <nick> "<filename>"
            ServerConnection con = serverConnectMap[serverInfo.index];
            String msg = ackInfo.nick + " \"" + ackInfo.filepath + "\"";
            con.sendMessage(MsgType.ALTERNATE_DL_REQ, msg);
        }

        public static void browseUserRequest(ServerInfo serverInfo, String nick)
        {
            ServerConnection con = serverConnectMap[serverInfo.index];
            con.sendMessage(MsgType.BROWSE_USER_REQ, nick);
        }
        
    }

    // サーバ接続    ------------------------------------------------------------------------------------------------------------------------
    class ServerConnection
    {
       
        public bool isConnected = false;
        public ServerInfo serverInfo = null;

        // チャンネルリストエントリーを受けてから入る
        private Dictionary<String, ChannelInfo> publicChannels = new Dictionary<String, ChannelInfo>();
        private Dictionary<String, ChannelInfo> privateChannels = new Dictionary<String, ChannelInfo>(); 

        //接続中のみ
        NetworkStream stream = null;
        private CancellationTokenSource keepaliveThreadCts = null;
        private CancellationTokenSource readThreadCts = null;
        public bool isLogined = false;
        private MemoryStream readMemoryStream = null;
        
        //コンストラクタ
        public ServerConnection(ServerInfo serverInfo)
        {
            this.serverInfo = serverInfo;
        }

        // publicチャンネル情報取得
        public ChannelInfo getChannel(String chName)
        {
            if (publicChannels.ContainsKey(chName) == false)
            {
                ChannelInfo ch = new ChannelInfo(chName);
                publicChannels.Add(chName, ch);
            }
            return publicChannels[chName];
        }

        // indexからチャンネルを取得
        public ChannelInfo getChannelIndex(String index)
        {
            foreach (String name in publicChannels.Keys)
            {
                if (index.CompareTo(publicChannels[name].index) == 0 )
                {
                    return publicChannels[name];
                }
            }
            return null;
        }

        //Priv用のchInfo取得(無ければ登録)
        public ChannelInfo getPrivChannel(String userName)
        {
            if (privateChannels.ContainsKey(userName) == false)
            {                
                ChannelInfo ch = new ChannelInfo(userName);
                ch.isPrivateMsgCh = true;
                privateChannels.Add(userName, ch);
            }
            return privateChannels[userName];
        }

        // チャンネル追加(追加したらreturn true
        //public void addChannel(String chName)
        //{
        //    if (publicChannels.ContainsKey(chName))
        //    {
        //        return;
        //    }

        //    ChannelInfo ch = new ChannelInfo(chName);
        //    publicChannels.Add(chName, ch);
        //    return;
        //}

        //public bool addChannel(String chName, String userNum, String topic)
        //{
        //    if (channels.ContainsKey(chName))
        //    {
        //        channels[chName].userNum = userNum;
        //        channels[chName].topic = topic;
        //        return false;
        //    }

        //    ChannelInfo ch = new ChannelInfo(chName, userNum, topic);
        //    channels.Add(chName, ch);
        //    return true;
        //}

        // 送信byte配列作成
        private byte[] toSendBytes(MsgType type, String body)
        {
            byte[] typeBytes = BitConverter.GetBytes((int)type);
            byte[] bodyBytes = ServerManager.ENC_SJIS.GetBytes(body);
            byte[] lengBytes = BitConverter.GetBytes(bodyBytes.Length);            
            MemoryStream mem = new MemoryStream();
            mem.Write(lengBytes, 0, 1);
            mem.Write(lengBytes, 1, 1);
            mem.Write(typeBytes, 0, 1);
            mem.Write(typeBytes, 1, 1);
            mem.Write(bodyBytes, 0, bodyBytes.Length);
            return mem.ToArray();
        }

        // メッセージ送信(同期
        private Object sendLock = new Object(); 
        public void sendMessage(MsgType type, String body, bool beforeLoginMsg = false)
        {
            if (isConnected == false || stream == null)
            {
                return;
            }
            byte[] sendBytes = toSendBytes(type, body);
         //   Console.WriteLine("");
            lock (sendLock)
            {
                //ダブルロック
                if (isConnected == false || stream == null || (isLogined == false && beforeLoginMsg == false))
                {
                    return;
                }
                stream.Write(sendBytes, 0, sendBytes.Length);
                stream.Flush();
#if DEBUG
                if (AppCommon.setting.disableDebugProtocol == false)
                {
                    Console.WriteLine("C2Sメッセージ[" + serverInfo.name + "][" + type + "][" + body + "]");
                }
#endif
            }
        }

        //接続開始()
        public void connect()
        {
            try
            {
                isConnected = true;
                TcpClient sock = new TcpClient(serverInfo.host, serverInfo.port);
                stream = sock.GetStream();

                String waitport = "0";
                String linkType = "0";
                if (AppCommon.setting.enableShare)
                {
                    waitport = AppCommon.setting.shareWaitPort.ToString();
                    linkType = AppCommon.setting.linktype;
                }
                //Format: <nick> <password> <port> "<client-info>" <link-type> [ <num> ]
                //0001->[000038][002][user pass 0 "OpenNapChat 1.05.622" 0]
                String loginMsg = serverInfo.user + " " + serverInfo.pass + " " + waitport + " \"" + AppCommon.APPNAME + " " + AppCommon.APPVER + "\" " + linkType;
                sendMessage(MsgType.LOGIN_REQ, loginMsg, true);
                
                //keep-alive
                keepaliveThreadCts = new CancellationTokenSource();
                Task keepaliveThread = Task.Factory.StartNew(() =>
                {
                    try
                    {
                        // Thread.CurrentThread.Name = "Kee-Alive-Thread";
                        while (isConnected)
                        {
                            Thread.Sleep(AppCommon.KEEP_ALIVE_INTERVAL);
                            if (isLogined)
                            {
                                //0035->[000000][214][]
                                sendMessage(MsgType.SERVER_STATUS, "");
                                //0027->[000000][617][]
                                sendMessage(MsgType.LIST_CHANNEL, "");
                            }
                        }
                    }
                    catch (Exception e)
                    {
                        if (isConnected)
                        {
                            ServerManager.serverErrMsgToScreen(serverInfo, "エラー(3)：", e);
                            disConnect();
                        }
                    }
                }, keepaliveThreadCts.Token);
            
                byte[] buffer = new byte[1024*100];
                int len = -1;
                readMemoryStream = new MemoryStream();
            
                //readスレッド
                readThreadCts = new CancellationTokenSource();
                Task readThread = Task.Factory.StartNew(() =>
                {
                    //Thread.CurrentThread.Name = "Read-Thread";
                    try
                    {
                        while (isConnected)
                        {
                            len = stream.Read(buffer, 0, buffer.Length);
                            if (len <= 0)
                            {
                                Console.WriteLine("readスレッドdebug＠len=" + len);
                                break;
                            }
                            //Console.WriteLine("readスレッドdebug＠len:" + len);
                            //Console.WriteLine("readスレッドdebug＠write直前サイズ:" + readMemoryStream.Length);
                            //int capa = (int)readMemoryStream.Length + len;
                            //Console.WriteLine("readスレッドdebug＠capa:" + capa + " streamのcapa:" + readMemoryStream.Capacity);                          
                            readMemoryStream.Seek(0, SeekOrigin.End);
                            readMemoryStream.Write(buffer, 0, len);
                            //Console.WriteLine("readスレッドdebug＠write直後サイズ:" + readMemoryStream.Length);
                            if (isConnected)// read-block回避
                            {
                                checkReadMsg();
                            }
                        }
                    }
                    catch (Exception e)
                    {
                        if (isConnected)
                        {

                            ServerManager.serverErrMsgToScreen(serverInfo, "エラー(1)：", e );
                            disConnect();
                        }
                    }
                    Console.WriteLine("readスレッドdebug＠readスレッド終了");
                }, readThreadCts.Token);

            }
            catch (Exception e)
            {
                ServerManager.serverErrMsgToScreen(serverInfo, "エラー(2)：", e);
                disConnect();
            }
        }
        
        //S2Cメッセージの解析
        private void checkReadMsg()
        {
            while (readMemoryStream.Length >= 4)
            {
                readMemoryStream.Seek(0, SeekOrigin.Begin);
                byte[] headBytes = new byte[4];                
                readMemoryStream.Read(headBytes, 0, headBytes.Length);                

                byte[] lenBuf = { headBytes[0], headBytes[1], 0x00, 0x00 };
                int readTmpLength = BitConverter.ToInt32(lenBuf, 0);
                byte[] typBuf = { headBytes[2], headBytes[3], 0x00, 0x00 };
                int tmpTypeVal = BitConverter.ToInt32(typBuf, 0);

                if (readTmpLength > readMemoryStream.Length - 4)
                {
                    Console.WriteLine("readスレッドdebug＠メッセージ不足で戻る　type[" + tmpTypeVal + "] size:" + readTmpLength + "/" + readMemoryStream.Length);
                    return;
                }

                MsgType readTmpType = MsgType.UNKNOWN;
                foreach (int typeVal in Enum.GetValues(typeof(MsgType)))
                {
                    if (tmpTypeVal == typeVal)
                    {
                        readTmpType = (MsgType)Enum.Parse(typeof(MsgType), typeVal.ToString());
                    }
                }

                byte[] msgBody = new byte[readTmpLength];
                readMemoryStream.Read(msgBody, 0, msgBody.Length);

                ServerManager.s2cMessage(serverInfo, readTmpType, msgBody, tmpTypeVal);

               // Console.WriteLine("readスレッドdebug＠余りの引継ぎ： size:" + readMemoryStream.Length + "-" + readTmpLength + "=" + (readMemoryStream.Length-readTmpLength));

                MemoryStream switchMemStream = new MemoryStream();
                byte[] buff = new byte[1024];
                int len = -1;
                while ((len = readMemoryStream.Read(buff, 0, buff.Length)) > 0)
                {
                    switchMemStream.Write(buff, 0, len);
                }
                readMemoryStream = switchMemStream;
               // Console.WriteLine("readスレッドdebug＠ループ出る直前の引継ぎサイズ:" + readMemoryStream.Length);
            }
        }

        // 切断
        public void disConnect()
        {
            //※削除保留：多重切断抑止は必要？頭で？後で？※
            //if (isConnected == false)
            //{
            //    return;
            //}
            lock (sendLock)
            {
                isConnected = false;
            }
            isLogined = false;
            publicChannels.Clear();

            if (stream != null)
            {
                stream.Close();
                stream = null;
            }

            // キープアライブスレッドを停止
            if (keepaliveThreadCts != null)
            {
                keepaliveThreadCts.Cancel();
                keepaliveThreadCts = null;
            }
            if (readThreadCts != null)
            {
                readThreadCts.Cancel();
                readThreadCts = null;
            }

            ServerManager.disconnectS2Client(serverInfo);
        }        
    }

    // 検索キー情報　構造情報も中に持つ
    public class SearchKeyInfo
    {
        //FILENAME CONTAINS "aaa"
        //LINESPEED "AT LEAST" 8
        //MAX_RESULTS 75
        public String[] keywords = null;
        public ServerInfo[] targetServers = null;

        // 単純検索用コンストラクタ
        public SearchKeyInfo(ServerInfo[] targetServers, String keyword)
        {
            this.keywords = new String[] { keyword };
            this.targetServers = targetServers;
        }

        // 送信用文字列
        //TODO 拡張検索時に拡張
        public String toRequestString()
        {
            String result = "";
            foreach (String k in keywords)
            {
                if (result.Length > 0)
                {
                    result += " ";
                }
                result += "FILENAME CONTAINS \"" + k + "\"";
            }
            return result;
        }
    }

    // 検索結果情報
    public class SearchResult
    {
        public ServerInfo serverInfo;
        //public String server; // 個別
        public String filepath;
        public String filename;
        //public String md5;
        public long sizeNum = 1;
        public String sizeStr;
       // public String bitrate;
       // public String frequency;
       // public String length;
        public String nick;
        //public String ip;
        public String linktype;

        //public SearchResult(String[] infos, ServerInfo serverInfo)
        public SearchResult(ServerInfo serverInfo, String filePath, String fileSizeStr, String nick, String linkTypeCode)
        {
            this.serverInfo = serverInfo;
            this.filepath = filePath;
            this.filename = AppCommon.filePath2FileName(filePath);
          //  this.md5 = md5;
            this.sizeStr = fileSizeStr;
            try
            {
                sizeNum = long.Parse(sizeStr);
                sizeStr = sizeNum.ToString(AppCommon.FILE_SITE_FOMMAT);
            }
            catch (FormatException)
            {
                Console.WriteLine("不正サイズ：" + sizeStr);
            }            
            //bitrate = infos[3];
            //frequency = infos[4];
            //length = infos[5];
            this.nick = nick;
            //ip = infos[7];
            this.linktype = AppCommon.code2LinkType(linkTypeCode);
           // this.server = serverInfo.name;
        }
    }

    // DL Ack 情報
    public class DownloadAckInfo
    {
        public String nick;
        public String ip;
        public String port;
        public int portNum = -1;
        public String filepath;
        public String md5;
        public String linespeed;
        public DownloadAckInfo(String[] msg)
        {
            this.nick = msg[0];
            this.ip = msg[1];
            this.port = msg[2];
            //this.portNum = long.Parse(this.port);
            this.filepath = msg[3];
            this.md5 = msg[4];
            this.linespeed = AppCommon.code2LinkType(msg[5]);
        }
    }

    // メッセージタイプ
    public enum MsgType
    {
        UNKNOWN = -1 , //未定義
        ERROR = 0,
        LOGIN_REQ = 2,
        LOGIN_ACK = 3,
        NOTIFY_SHARED_FILE_ = 100,
        REMOVE_SHARED_FILE_ = 102,
        UNSHARE_ALL_FILES = 110,
        SEARCH_REQUEST = 200,
        SEARCH_RESPONCE = 201,
        SEARCH_COMPLETE = 202,
        DOWNLOAD_REQUEST = 203,
        DOWNLOAD_ACK = 204,
        PRIVATE_MESSAGE = 205,
        BROWSE_USER_REQ = 211,
        BROWSE_USER_RESPONSE = 212,
        BROWSE_USER_COMPLETE = 213,
        SERVER_STATUS = 214,
        DOWNLOAD_START = 218,
        DOWNLOAD_COMPLETE = 219,
        UPLOAD_START = 220,
        UPLOAD_COMPLETE = 221,
        CLIENT_DISCONNECT = 316,
        CLEAR_IGNORE_LIST = 326,
        JOIN_CHANNEL = 400,
        PART_CHANNEL = 401,
        SEND_PUBLIC_MESSAGE = 402,
        PUBLIC_MESSAGE = 403,
        ERROR_MESSAGE = 404,
        JOIN_ACK = 405,
        USER_JOINED_CH = 406,
        USER_PARTED_CH = 407,
        CHANNEL_USER_LIST = 408,
        END_CH_USER_LIST = 409,
        CHANNEL_TOPIC = 410,
        ALTERNATE_DL_REQ = 500,
        ALTERNATE_DL_ACK = 501,
        WHOIS_REQUEST = 603,//<nick>
        WHOIS_RESPONSE = 604,
        UPLOAD_REQUEST = 607,
        UPLOAD_REQUEST_ACK = 608,
        LIST_CHANNEL = 617,
        CHANNEL_ENTRY = 618,
        QUEUE_LIMIT_C2S = 619, //queue limit(client
        QUEUE_LIMIT_S2C = 620, //queue limit(server
        MESSAGE_OF_DAY = 621,
        CHANGE_LINK_SPEED = 700,
        CHANGE_DATA_PORT = 703,
        LOGIN_ATTENPT = 748,
        PONG_RESPONSE = 752, //752 (0x2f0)	pong response
        ADD_FILE_BY_DIR = 870,
    }



}
