﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Net.Sockets;
using System.Net;
using System.Threading;

namespace MoyariClient
{
    class UploadManager
    {
        // 共有ファイル <ファイルパス,情報>
        private static Dictionary<String, SharedFileInfo> sharedInfoMap = new Dictionary<String, SharedFileInfo>();
        // 共有ファイルキー情報リスト( ファイルサイズ + ":" + ファイル名 )
        private static List<String> sharedFileKeyList = new List<String>(); 
        // 共有ファイルキー情報リスト( ファイルサイズ )
        private static List<long> sharedFileSizeList = new List<long>(); 

        private static List<ShareFileSendThread> sendingThreadList = new List<ShareFileSendThread>();
        private static ShareFileLoadThread loadingThread = new ShareFileLoadThread();
        public static Object sharedLock = new Object();

        // wantQueueスタック
        private static Dictionary<String, int> wantQueueCommandStack = new Dictionary<String, int>();

        // タスクキュー
        private static List<UploadTaskInfo> taskQueue = new List<UploadTaskInfo>();

        // 鯖
        private static UploadServerThread serverThread = null;

        // ------------------------------------------------ 設定初期化関連
        // 設定のロード
        public static void initSharedFileInfo()
        {
            lock (sharedLock)
            {
                // 送信スレッドが居たら止める
                loadingThread.kill();
                lock (sendingThreadList)
                {
                    foreach (ShareFileSendThread sendingThread in sendingThreadList)
                    {
                        sendingThread.kill();
                    }
                    sendingThreadList.Clear();
                }

                ShareFileLoadThread t = new ShareFileLoadThread();
                t.start();
                loadingThread = t;
            }
        }

        // 共有ファイルリスト差し替え
        public static void switchSharedInfoMap(Dictionary<String, SharedFileInfo> loadedSharedInfoMap)
        {
            ServerInfo[] serverInfos = ServerManager.getLoginedServerInfoList();

            Dictionary<String, SharedFileInfo> oldMap = null;
            List<String> newSharedFileKeyList = new List<String>();
            List<long> newSharedFileSizeList = new List<long>();
            foreach (String key in loadedSharedInfoMap.Keys)
            {
                newSharedFileKeyList.Add(loadedSharedInfoMap[key].getConflictKey());
                newSharedFileSizeList.Add(loadedSharedInfoMap[key].fileSize);
            }

            lock (sharedInfoMap)
            {
                oldMap = sharedInfoMap;
                sharedInfoMap = loadedSharedInfoMap;
                sharedFileKeyList = newSharedFileKeyList;
                sharedFileSizeList = newSharedFileSizeList;
            }
            foreach(ServerInfo serverInfo in serverInfos){
                sendSharedFileList(serverInfo, true, oldMap);
            }            
        }

        // 共有ファイル個別追加(DL完了時用)
        public static void addShareLocalFilePath(String filePath)
        {
            SharedFileInfo info = new SharedFileInfo(new FileInfo(filePath));
            lock (sharedInfoMap)
            {
                sharedInfoMap.Add(filePath, info);
                sharedFileKeyList.Add(info.getConflictKey());
                sharedFileSizeList.Add(info.fileSize);
            }
        }

        // 共有ファイル一致チェック
        public static bool checkShared(String fileName, long fileSize)
        {
            return sharedFileKeyList.Contains(SharedFileInfo.makeConflistKey(fileName, fileSize));            
        }

        // 共有ファイル一致チェック(サイズのみ)
        public static bool checkSharedOnlySize(long fileSize)
        {
            return sharedFileSizeList.Contains(fileSize);
        }

        // 共有ファイルリスト送信
        public static void sendSharedFileList(ServerInfo serverInfo, bool relativeFlg = false, Dictionary<String, SharedFileInfo> oldMap = null)
        {
            // ファイルロード中ならなにもしない。
            // relativeFlgはロードスレッドから呼ばれる場合
            if (loadingThread.alive && relativeFlg == false)
            {
                return;            
            }

            lock (sendingThreadList)
            {
                ShareFileSendThread sendThread = new ShareFileSendThread(serverInfo, oldMap);
                sendThread.start();
                sendingThreadList.Add(sendThread);
            }
        }
        
        // リストにして取得
        public static List<SharedFileInfo> getSharedFileList()
        {
            List<SharedFileInfo> sfList = new List<SharedFileInfo>();
            lock (sharedInfoMap)
            {
                foreach (String key in sharedInfoMap.Keys)
                {
                    sfList.Add(sharedInfoMap[key]);
                }
            }
            return sfList;
        }

        // 共有リスト更新中チェック
        public static bool isRefleshOk()
        {
            if (loadingThread.alive)
            {
                return false;
            }

            lock (sendingThreadList)
            {
                foreach (ShareFileSendThread shareThread in sendingThreadList)
                {
                    if (shareThread.alive)
                    {
                        return false;
                    }
                }
            }

            if (sharedInfoMap.Count > 0)
            {
                return true;
            }

            return false;
        }


        // ------------------------------------------UPタスク関連

        // 衝突回避用
        public static UploadTaskInfo[] getTaskAry()
        {
            UploadTaskInfo[] result = null;
            lock (taskQueue)
            {
                result = taskQueue.ToArray();
            }
            return result;
        }

        // アップロードタスクの削除
        public static void removeUploadTasks(UploadTaskInfo removeTask, bool reflesh)
        {
            removeUploadTasks(new UploadTaskInfo[] { removeTask }, reflesh);
        }
        public static void removeUploadTasks(UploadTaskInfo[] removeTaskAry, bool reflesh)
        {
            lock (taskQueue)
            {
                foreach (UploadTaskInfo rt in removeTaskAry)
                {
                    rt.cancel();
                    taskQueue.Remove(rt);
                }
            }

            if (reflesh)
            {
                FormTransfer.refleshUploadListALL(); //リスト作り直し
            }
        }

        // アップロードリクエスト受信
        public static void receiveUploadRequest(ServerInfo serverInfo, String nick, String filePath, String speed)
        {
            if (AppCommon.setting.ignoreList.Contains(nick))
            {
                Console.WriteLine("イグノア UploadRequest [" + nick + "][" + filePath + "]");
                return;//ignore
            }

            if (sharedInfoMap.ContainsKey(filePath))
            {
                //limit 
                int activeDLnum = 0; //アクティブなDL数(キュー除く)
                int activeSameUserDLnum = 0; //アクティブな同一ユーザDL数(キュー除く)
                int queueNum = 0; //キュー入してるタスク数
                UploadTaskInfo[] ctasks = getTaskAry();
                foreach (UploadTaskInfo ctask in ctasks)
                {
                    if (ctask.state == UploadTaskState.UPLOADING || (ctask.state == UploadTaskState.WAIT && ctask.queueing == false))
                    {
                        activeDLnum += 1;
                        if (serverInfo.index == ctask.serverInfo.index && ctask.user == nick)
                        {
                            activeSameUserDLnum += 1;
                        }
                    }
                    if (ctask.queueing == true && ctask.state == UploadTaskState.WAIT)
                    {
                        queueNum += 1;
                    }
                }
                
                if (activeDLnum >= AppCommon.setting.maxUploadNum || activeSameUserDLnum >= AppCommon.setting.maxUploadNumAtUser)
                {
                    int num = activeDLnum - AppCommon.setting.maxUploadNum;
                    int num2 = activeSameUserDLnum - AppCommon.setting.maxUploadNumAtUser;                         
                    if (num < num2){
                        num = num2;
                    }
                    num += queueNum;

                    if (wantQueueCommandStack.ContainsKey(nick) && (wantQueueCommandStack[nick] > 0))
                    {
                        //Queueコマンドの場合
                        wantQueueCommandStack[nick] -= 1;

                        //752-619 EnterQueueの返しは待ち数付き 
                        ServerManager.uploadQueueLimit(serverInfo, nick, filePath, (num + 1).ToString());

                        //waitで入れる ackは返さない
                        UploadTaskInfo task = new UploadTaskInfo(serverInfo, nick, filePath, speed);
                        task.state = UploadTaskState.WAIT;
                        task.uploading = true;
                        task.queueing = true;

                        lock (taskQueue)
                        {
                            taskQueue.Add(task);
                        }

                        FormTransfer.refleshUploadListALL();
                    }
                    else
                    {
                        //752-619 キュー可Busyの返しは0
                        ServerManager.uploadQueueLimit(serverInfo, nick, filePath, "0");
                    }
                }
                else
                {
                    UploadTaskInfo task = new UploadTaskInfo(serverInfo, nick, filePath, speed);
                    task.state = UploadTaskState.WAIT;
                    task.uploading = true;

                    lock (taskQueue)
                    {
                        taskQueue.Add(task);
                    }

                    // 608 ack
                    ServerManager.acceptUploadRequest(task);

                    FormTransfer.refleshUploadListALL();
                }
            }
            else
            {
                // なにもしない
                FormTransfer.addConsoleMessage("不明なファイルがULリクエストされました " +nick + "[" + serverInfo.name + "]" + filePath);
            }
        }

        // サーバースレッド起動/更新
        public static void startUploadServer()
        {
            //serverThread
            if (serverThread != null){
                serverThread.stop();
            }
            serverThread = new UploadServerThread(AppCommon.setting.shareWaitPort);
            serverThread.start();
        }

        // サーバースレッド停止
        public static void stopUploadServer()
        {
            if (serverThread != null)
            {
                serverThread.stop();
                serverThread = null;
            }
            removeUploadTasks(getTaskAry(), true);
        }

        // 共有解除
        public static void unshareALL()
        {
            lock (sharedInfoMap)
            {
                sharedInfoMap.Clear();
            }
            sharedFileKeyList.Clear();
            sharedFileSizeList.Clear();
        }

        // wantqueue受信
        public static void wantQueueRecv(String user)
        {
            if (wantQueueCommandStack.ContainsKey(user) == false)
            {
                wantQueueCommandStack.Add(user, 0);
            }
            wantQueueCommandStack[user] += 1;
        }

        // 実行可能なqueueがあればackする
        public static void startQueuedTask()
        {
            UploadTaskInfo[] taskAry = getTaskAry();
            
            Dictionary<String, int> activeUserMap = new Dictionary<String, int>();
            int totalAct = 0;
            foreach (UploadTaskInfo ctask in taskAry)
            {
                if ((ctask.state == UploadTaskState.WAIT && ctask.queueing == false) || ctask.state == UploadTaskState.UPLOADING)
                {
                    if (activeUserMap.ContainsKey(ctask.user) == false)
                    {
                        activeUserMap.Add(ctask.user, 0);
                    }
                    activeUserMap[ctask.user] += 1;
                    totalAct += 1;
                }
            }

            if (totalAct >= AppCommon.setting.maxUploadNum)
            {
                return;
            }

            foreach (UploadTaskInfo task in taskAry)
            {
                if (task.state == UploadTaskState.WAIT && task.queueing)
                {
                    if (activeUserMap.ContainsKey(task.user) == false || activeUserMap[task.user] < AppCommon.setting.maxUploadNumAtUser)
                    {
                        task.queueing = false;
                        // 608 ack
                        ServerManager.acceptUploadRequest(task);
                        FormTransfer.refleshUploadListViewItem(task);
                        return;
                    }
                }
            }
        }

        // ALTERNATE_DL_ACK 対応　逆アップロード
        public static void alternateDownloadAckThread(ServerInfo serverInfo, String nick, String ip, String port, String filePath, String md5, String speed)
        {
            Task.Factory.StartNew(() =>
            {
                UploadTaskInfo[] taskAry = getTaskAry();
                UploadTaskInfo task = null;
                foreach (UploadTaskInfo ctask in taskAry)
                {
                    if (ctask.state == UploadTaskState.WAIT)
                    {
                        if (ctask.user == nick && ctask.filePath == filePath)
                        {
                            task = ctask;
                        }
                    }
                }
                if (task == null)
                {
                    FormTransfer.addConsoleMessage("ファイルアップロードでエラー(不明なポート０アップロードファイル要求):");
                    return;
                }

                // 状態整合確認
                if (sharedInfoMap.ContainsKey(task.filePath) == false)
                {
                    FormTransfer.addConsoleMessage("ファイルアップロード(ポート０)でエラー(ファイルが共有対象ではない):" + nick + " " + filePath);
                    return;
                }
                SharedFileInfo fileInfo = sharedInfoMap[task.filePath];

                //FileStream localFileStream = null;
                NetworkStream clientStream = null;
                try
                {
                    task.state = UploadTaskState.UPLOADING;
                    FormTransfer.refleshUploadListViewItem(task);

                    //接続
                    String hostStr = AppCommon.ipStr2hostStr(ip);
                    int portNum = int.Parse(port);

                    TcpClient sock = new TcpClient(hostStr, portNum);
                    sock.SendTimeout = AppCommon.setting.sockTimeoutSec * AppCommon.SEC_TO_MILLSEC_RATE;
                    sock.ReceiveTimeout = AppCommon.setting.sockTimeoutSec * AppCommon.SEC_TO_MILLSEC_RATE;

                    clientStream = sock.GetStream();

                    // ヘッダ確認
                    int headbyte = clientStream.ReadByte();
                    if (headbyte != AppCommon.DOWNLOAD_START_BYTE)
                    {
                        //ackbyte不正
                        task.state = UploadTaskState.DEAD;
                        task.errorString = "エラー中断";
                        FormTransfer.addConsoleMessage("ファイルアップロード(ポート０)　エラー中断します(不正な接続先)");
                        return;
                    }

                    //SEND
                    byte[] sendBytes = AppCommon.DOWNLOAD_SEND_BYTES;
                    clientStream.Write(sendBytes, 0, sendBytes.Length);
                    clientStream.Flush();

                    //コマンドライン
                    String command = serverInfo.user + " \"" + filePath + "\" " + fileInfo.fileSize;
                    sendBytes = AppCommon.ENC_SJIS.GetBytes(command);
                    clientStream.Write(sendBytes, 0, sendBytes.Length);
                    clientStream.Flush();

                    // 転送開始サイズを1パケットで取得
                    byte[] readBuff = new byte[255];
                    int readlen = clientStream.Read(readBuff, 0, readBuff.Length);

                    MemoryStream sizems = new MemoryStream();
                    sizems.Write(readBuff, 0, readlen);
                    sizems.Close();
                    String startSizeStr = AppCommon.ENC_SJIS.GetString(sizems.ToArray());
#if DEBUG
                    Console.WriteLine("転送開始サイズ:" + startSizeStr);
#endif
                    long uploadStartPos = long.Parse(startSizeStr);
                    
                    normalUploadBody(task, clientStream, uploadStartPos, fileInfo);
#if DEBUG
                    Console.WriteLine("ポト0アップロード正常終了 " + nick + " ： " + filePath);
#endif
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.StackTrace);
                    if (task != null && task.state == UploadTaskState.UPLOADING)
                    {
                        String msg = "アップロードが中断されました　";
                        msg += task.user + "[" + task.serverInfo.name + "]：" + task.fileName + " " + e.Message;
                        FormTransfer.addConsoleMessage(msg);
                        task.state = UploadTaskState.DEAD;
                        task.errorString = "中断されました";
                        FormTransfer.refleshUploadListViewItem(task);
                    }
                    else
                    {
                        FormTransfer.addConsoleMessage("ファイルアップロードで不明なエラー(2)：" + e.Message + "\n" + e.StackTrace);
                    }
                }
                finally
                {
                    try
                    {
                        if (clientStream != null)
                        {
                            clientStream.Close();
                        }
                    }catch(Exception){
                    }
                }

            });
        }

        // 通常アップロード(共通部)
        public static void normalUploadBody(UploadTaskInfo task, NetworkStream clientStream, long uploadStartPos, SharedFileInfo fileInfo)
        {
            FileStream localFileStream = null;
            try
            {
                localFileStream = new FileStream(task.filePath, FileMode.Open, FileAccess.Read);
                localFileStream.Seek(uploadStartPos, SeekOrigin.Begin);

                task.completeSize = uploadStartPos;
                task.fileSize = fileInfo.fileSize;

                FormTransfer.addConsoleMessage("アップロード要求を受け付けました　" + task.user + "[" + task.serverInfo.name + "]：" + task.fileName);

                ServerManager.uploadingStart(task); // ------------- ここから開始                    

                task.state = UploadTaskState.UPLOADING;
                FormTransfer.refleshUploadListViewItem(task);

                //byte[] lenStrBytes = AppCommon.ENC_SJIS.GetBytes(fileInfo.fileSize.ToString());
                //clientStream.Write(lenStrBytes, 0, lenStrBytes.Length);
                //clientStream.Flush();

                byte[] writeBuff = new byte[AppCommon.UL_BUFFER_SIZE];
                int writeLen = -1;
                long startTime = DateTime.Now.ToFileTime();
                long oldTime = startTime;
                bool completeFlg = false;

                while (task.uploading)
                {
                    writeLen = localFileStream.Read(writeBuff, 0, writeBuff.Length);
                    if (writeLen <= 0)
                    {
                        completeFlg = true;
                        break;
                    }
#if DEBUG
                    Thread.Sleep(30);
#endif
                    clientStream.Write(writeBuff, 0, writeLen);
                    clientStream.Flush();
                    long nowTime = DateTime.Now.ToFileTime();
                    task.completeSize += writeLen;
                    if ((nowTime - oldTime) > AppCommon.DL_REFLESH_RATE_NS)
                    {
                        float pastSec = (nowTime - startTime) / AppCommon.DL_REFLESH_RATE_NS;
                        float kbPerSec = (((float)task.completeSize) / pastSec) / 1000f;
                        task.ulSpeed = kbPerSec.ToString("0.00");
                        task.compPct = ((float)task.completeSize) / ((float)task.fileSize);
                        task.compPctString = (((int)(task.compPct * 100)).ToString() + "%");
                        FormTransfer.refleshUploadListViewItem(task);
                        oldTime = nowTime;
                    }
                }

                // 正常終了
                if (completeFlg)
                {
                    task.state = UploadTaskState.COMPLETE;
                    FormTransfer.addConsoleMessage("アップロード完了しました　" + task.user + "[" + task.serverInfo.name + "]：" + task.fileName);
                }
                else
                {
                    if (task.state == UploadTaskState.CANCELED)
                    {
                        // キャンセルされた場合->何もしない
                    }
                    else
                    {
                        task.state = UploadTaskState.DEAD;
                        task.errorString = "切断されました";
                        FormTransfer.addConsoleMessage("アップロードが切断されました　" + task.user + "[" + task.serverInfo.name + "]：" + task.fileName);
                    }
                }
                FormTransfer.refleshUploadListViewItem(task);
            }
            finally
            {
                ServerManager.uploadComplete(task);

                if (localFileStream != null)
                {
                    try
                    {
                        localFileStream.Close();
                    }
                    catch (Exception)
                    {
                    }
                }
            }
        }



        // ------------------------------------------------------------------
        // アップロードサーバスレッド(と子スレッドの管理)
        class UploadServerThread
        {
            private int port = 0;
            private bool running = false;
            private List<UploadClientThread> clThreadList = new List<UploadClientThread>();
            private TcpListener listener = null;

            // コンストラクタ
            public UploadServerThread(int port)
            {
                this.port = port;
                this.running = true;
            }

            // 全停止
            public void stop()
            {
                running = false;
                disposeListener();

                UploadClientThread[] clAry = null;                
                lock (clThreadList)
                {   
                    clAry = clThreadList.ToArray();
                }
                foreach (UploadClientThread clt in clAry)
                {
                    clt.stop();
                }
            }

            private void disposeListener()
            {
                try
                {
                    if (listener != null)
                    {
                        listener.Stop();
                    }
                }
                catch (Exception)
                {
                    //スルー
                }
                finally
                {
                    listener = null;
                }
            }

            // 対象のタスクと一致するクライアントスレッドを停止(外から
            //public void stopClientThrad(UploadTaskInfo task)
            //{
            //    if (running == false)
            //    {
            //        return;
            //    }
            //    UploadClientThread target = null;
            //    lock (clThreadList)
            //    {
            //        foreach (UploadClientThread clt in clThreadList)
            //        {
            //            if (clt.task != null && clt.task.index == task.index)
            //            {
            //                target = clt;
            //            }
            //        }
            //    }
            //    if (target != null)
            //    {
            //        target.stop();
            //    }
            //}

            // クライアントスレッドを削除(クラから呼ばれる
            public void removeClientThread(UploadClientThread cl)
            {
                if (clThreadList.Contains(cl))
                {
                    lock (clThreadList)
                    {
                        if (clThreadList.Contains(cl))
                        {
                            clThreadList.Remove(cl);
                        }
                    }
                }
            }

            // スレッド開始
            public void start()
            {
                FormTransfer.addConsoleMessage("ファイルアップロード待受開始します　ポート:" + port);
                Task.Factory.StartNew(() =>
                {
                    try
                    {
                        listener = new TcpListener(IPAddress.Any, port);
                        listener.Start();

                        while (running)
                        {
                            TcpClient tcpclient = listener.AcceptTcpClient();
                            tcpclient.SendTimeout = AppCommon.setting.sockTimeoutSec * AppCommon.SEC_TO_MILLSEC_RATE;
                            tcpclient.ReceiveTimeout = AppCommon.setting.sockTimeoutSec * AppCommon.SEC_TO_MILLSEC_RATE;
                            UploadClientThread ult = new UploadClientThread(this,tcpclient);
                            lock (clThreadList)
                            {
                                if (running)
                                {
                                    clThreadList.Add(ult);
                                    ult.start();
                                }
                            }
                        }
                    }
                    catch (Exception e)
                    {
                        if (running)
                        {
                            FormTransfer.addConsoleMessage("エラー：サーバーを正常に起動できません(port番号衝突等)。 "+e.Message);
                        }
                    }
                    finally 
                    {
                        disposeListener();
                        FormTransfer.addConsoleMessage("ファイルアップロード待受を終了します　ポート:" + port);
                    }
                });
            }
        }

        //アップロードクライアント対応スレッド
        class UploadClientThread
        {
            public TcpClient tcpclient = null;
            private bool running = false;
            private UploadServerThread parent;
            
            // コンストラクタ
            public UploadClientThread(UploadServerThread parent, TcpClient tcpclient)
            {
                this.tcpclient = tcpclient;
                this.parent = parent;
                this.running = true;
            }

            // 停止
            public void stop()
            {
                this.running = false;
                disposeClient();
            }

            // client切断
            private void disposeClient()
            {
                try
                {
                    if (tcpclient != null)
                    {
                        tcpclient.Close();
                    }
                }
                catch (Exception)
                {
                    //nothing
                }
                finally
                {
                    tcpclient = null;
                }
            }

            // 開始
            public void start()
            {   
                NetworkStream clientStream = null;
                Task.Factory.StartNew(() =>
                {
                    try
                    {
                        clientStream = tcpclient.GetStream();

                        clientStream.WriteByte(AppCommon.DOWNLOAD_START_BYTE);
                        clientStream.Flush();
                                                
                        byte[] getBuf = new byte[3];
                        int getLen = -1;
                        getLen = clientStream.Read(getBuf, 0, getBuf.Length);

                        if (getLen != getBuf.Length)
                        {
                            return;
                        }

                        byte[] GETbyte = AppCommon.DOWNLOAD_GET_BYTES;
                        byte[] SENDbyte = AppCommon.DOWNLOAD_SEND_BYTES;
                        bool getRequest = false;
                        if (getBuf[0] == GETbyte[0] && getBuf[1] == GETbyte[1] && getBuf[2] == GETbyte[2])
                        {
                            // 通常UL
                            getRequest = true;
                            Console.WriteLine("GETリクエスト");
                        }
                        else if (getBuf[0] == SENDbyte[0] && getBuf[1] == SENDbyte[1] && getBuf[2] == SENDbyte[2])
                        {
                            // ポト0 ダウンロード接続
                            byte dByte = (byte)clientStream.ReadByte(); //もう1byte切り出す
                            getRequest = false;
                            Console.WriteLine("SENDリクエスト");
                        }
                        else
                        {
                            // エラー
                            FormTransfer.addConsoleMessage("ファイルアップロードでエラー(不明な接続):[" + AppCommon.ENC_SJIS.GetString(getBuf)+"]");
                            return;
                        }

                        getBuf = new byte[255*2]; //必ず1発で
                        getLen = clientStream.Read(getBuf, 0, getBuf.Length);

                        MemoryStream commandLineMs = new MemoryStream();
                        commandLineMs.Write(getBuf, 0, getLen);
                        commandLineMs.Close();

                        String commandLine = AppCommon.ENC_SJIS.GetString(commandLineMs.ToArray());
#if DEBUG
                        Console.WriteLine("リクエストコマンド[" + commandLine + "]");
#endif

                        if (getRequest)
                        {
                            normalUpload(clientStream, commandLine);
                        }
                        else
                        {
                            port0download(clientStream, commandLine);
                        }
                    }
                    catch (Exception e)
                    {
                        if (running)
                        {
                            FormTransfer.addConsoleMessage("ファイルアップロードで不明なエラー(1)："+e.Message+"\n"+e.StackTrace);                            
                        }
                    }
                    finally
                    {
                        if (clientStream != null)
                        {
                            try
                            {
                                clientStream.Close();
                            }
                            catch (Exception)
                            {
                            }                            
                        }

                        disposeClient();

                        parent.removeClientThread(this);

                        startQueuedTask();
                    }
                });
            }

            // 通常アップロード - 準備と例外管理
            private void normalUpload(NetworkStream clientStream, String commandLine)
            {
                UploadTaskInfo task = null;
                try
                {
                    String[] commandParts = AppCommon.splitMessageToString(commandLine, 3);

                    UploadTaskInfo[] taskAry = getTaskAry();
                    foreach (UploadTaskInfo ctask in taskAry)
                    {
                        if (ctask.state == UploadTaskState.WAIT)
                        {
                            if (ctask.user == commandParts[0] && ctask.filePath == commandParts[1])
                            {
                                task = ctask;
                            }
                        }
                    }
                    if (task == null)
                    {
                        FormTransfer.addConsoleMessage("ファイルアップロードでエラー(不明なファイル要求):" + commandLine);
                        return;
                    }
                    if (task.uploading == false)//キャンセル済み
                    {
                        return;
                    }

                    long uploadStartPos = long.Parse(commandParts[2]);

                    // 状態整合確認
                    if (sharedInfoMap.ContainsKey(task.filePath) == false)
                    {
                        FormTransfer.addConsoleMessage("ファイルアップロードでエラー(ファイルが共有対象ではない):" + commandLine);
                        return;
                    }
                    SharedFileInfo fileInfo = sharedInfoMap[task.filePath];
                    //if (fileInfo.fileSize < uploadStartPos)
                    //{
                    //    FormTransfer.addConsoleMessage("ファイルアップロードでエラー(不正なリジューム要求):(" + fileInfo.fileSize + ")" + commandLine);
                    //    return;
                    //}

                    // 転送開始サイズを送る
                    byte[] lenStrBytes = AppCommon.ENC_SJIS.GetBytes(fileInfo.fileSize.ToString());
                    clientStream.Write(lenStrBytes, 0, lenStrBytes.Length);
                    clientStream.Flush();
                    
                    normalUploadBody(task, clientStream, uploadStartPos, fileInfo); //前処理後 - 共通処理
                }
                catch (Exception e)
                {
                    if (running)
                    {
                        if (task != null)
                        {
                            String msg = "アップロードが中断されました　";
                            msg += task.user + "[" + task.serverInfo.name + "]：" + task.fileName;
                            FormTransfer.addConsoleMessage(msg);
                            task.state = UploadTaskState.DEAD;
                            task.errorString = "中断されました";
                            FormTransfer.refleshUploadListViewItem(task);
                        }
                        else
                        {
                            FormTransfer.addConsoleMessage("ファイルアップロードで不明なエラー(2)：" + e.Message + "\n" + e.StackTrace);
                        }
                    }
                }
                finally
                {
                    task.uploading = false;
                }
            }
            
            // ポト0ダウンロード
            private void port0download(NetworkStream clientStream, String commandLine)
            {
                //DownloadTaskInfo task = null;
                DownloadTaskInfo task = null;
                FileStream fsLocal = null;
                try
                {
                    String[] commandParts = AppCommon.splitMessageToString(commandLine, 3);
                    //commandParts[0]; nick
                    //commandParts[1]; filePath
                    //commandParts[2]; totalSize
                    
                    //Taskを取ってくる
                    DownloadTaskInfo[] taskAry = DownloadManager.getTaskAry();
                    foreach (DownloadTaskInfo dltask in taskAry)
                    {
                        if (dltask.portZeroDownloadFlg)
                        {
                            if (commandParts[0] == dltask.searchResult.nick && commandParts[1] == dltask.searchResult.filepath)
                            {
                                task = dltask;
                            }
                        }
                    }
                    if (task == null)
                    {
                        FormTransfer.addConsoleMessage("ポート０ダウンロードエラー(対象不明)：" + commandLine);
                        return;
                    }

                    //DLスタートコマンド
                    ServerManager.donwloadStart(task);

                    //対象を取得:dl中
                    task.state = DownloadTaskState.DONWLOADING;

                    // 保存先ファイルを確定、stream開いてlock                    
                    if (task.isResume)
                    {
                        fsLocal = new FileStream(task.localWorkFilePath, FileMode.Open);
                        fsLocal.Seek(0, SeekOrigin.End);
                        task.completeSize = task.resumeLength;
                    }
                    else
                    {
                        fsLocal = new FileStream(task.localWorkFilePath, FileMode.Create);
                    }

                    DownloadManager.addResumeInfo(task.searchResult.filename, task.searchResult.sizeNum, task.localWorkFileName);
                    
                    //ウンロード開始byteを送る
                    byte[] lenStrBytes = AppCommon.ENC_SJIS.GetBytes(task.resumeLength.ToString());
                    clientStream.Write(lenStrBytes, 0, lenStrBytes.Length);
                    clientStream.Flush();

                    // ダウンロード共通部
                    DownloadManager.normalDownloadBody(task, clientStream, fsLocal);
                }
                catch (Exception e)
                {
                    if (running)
                    {
                        if (task.downloading)
                        {
                            task.state = DownloadTaskState.DEAD;
                            task.errorString = "エラー";
                            FormTransfer.addConsoleMessage("エラー発生。中断します：" + task.logInfo() + "\n" + e.Message + "\n" + e.StackTrace);
                        }
                    }
                }
                finally
                {
                    task.downloading = false;

                    try
                    {
                        if (fsLocal != null)
                        {
                            fsLocal.Close();
                            fsLocal = null;
                        }
                    }
                    catch (Exception)
                    {
                        //do nothing
                    }

                    ServerManager.donwloadComplete(task);// コマンド：DL完了 (成否問わず
                    DownloadManager.startDownloadIfAble(true);// 次へ
                }
            }
        }


    }
    // ------------------------------------------------------------------------------------------------------------------------------------------------

    // 共有ファイル情報
    public class SharedFileInfo
    {
        public String filePath = null;
        public String fileName = null;
        public String dirPath = null;
        public long fileSize = 0;

        public SharedFileInfo(FileInfo fi)
        {
            this.filePath = fi.FullName;
            this.dirPath = fi.Directory.FullName + Path.DirectorySeparatorChar;
            this.fileName = fi.Name;
            this.fileSize = fi.Length;
        }

        public String getConflictKey()
        {
            return makeConflistKey(fileName, fileSize);
        }

        public static String makeConflistKey(String fileName, long fileSize)
        {
            return fileSize.ToString() + ":" + fileName.ToUpper();
        }
    }

    // 共有ファイル送信スレッド
    public class ShareFileSendThread
    {
        public bool alive = false;
        private ServerInfo serverInfo = null;
        private Dictionary<String, SharedFileInfo> oldMap = null;　//差し替えの場合の旧ファイル

        public ShareFileSendThread(ServerInfo serverInfo, Dictionary<String, SharedFileInfo> oldMap)
        {
            this.serverInfo = serverInfo;
            this.oldMap = oldMap;
        }
        
        public void kill()
        {
            if (alive)
            {
                FormTransfer.addConsoleMessage("共有ファイル情報送信を中断[" + serverInfo.name + "]");
            }
            alive = false;
        }

        public void start()
        {
            alive = true;

            Task.Factory.StartNew(() =>
            {
                try
                {
                    if (oldMap == null || oldMap.Count == 0)
                    {
                        //Console.WriteLine("★★SendNew");
                        sendNew();
                    }
                    else
                    {
                        //Console.WriteLine("★★さしかえ");
                        sendSwitch(); // さしかえ
                    }
                }
                catch (Exception e)
                {
                    FormTransfer.addConsoleMessage("エラー：共有ファイル送信に失敗:" + e.Message + "\n" + e.StackTrace);
                }
                finally
                {
                    alive = false;
                }
            });
        }

        // 新規
        private void sendNew()
        {
            FormTransfer.addConsoleMessage("共有ファイル情報を送信します[" + serverInfo.name + "]");
            List<SharedFileInfo> sfList = UploadManager.getSharedFileList();

            //フォルダでグループ化
            Dictionary<String, List<SharedFileInfo>> folderMap = new Dictionary<String, List<SharedFileInfo>>();
            foreach (SharedFileInfo fi in sfList)
            {
                if (folderMap.ContainsKey(fi.dirPath) == false)
                {
                    folderMap.Add(fi.dirPath, new List<SharedFileInfo>());
                }
                folderMap[fi.dirPath].Add(fi);
            }

            //フォルダごとに容量上限で送信メッセージリストを作る
            List<String> messageList = new List<String>();
            foreach (String dirPath in folderMap.Keys)
            {
                String md5 = AppCommon.DUMMY_MD5;
                List<SharedFileInfo> filist = folderMap[dirPath];
                String message = "\"" + dirPath + "\"";
                foreach (SharedFileInfo finfo in filist)
                {
                    String fileParts = " \"" + finfo.fileName + "\" " + md5 + "-" + finfo.fileSize + " " + finfo.fileSize + " 24 16000 600";
                    if ((message.Length + fileParts.Length) > 1024) //limit 2048byte
                    {
                        messageList.Add(message);
                        message = "\"" + dirPath + "\"";
                    }
                    message += fileParts;
                }
                messageList.Add(message);
            }

            if (alive == false)
            {
                return;
            }

            foreach (String message in messageList)
            {
                if (alive == false)
                {
                    return;
                }
                ServerManager.sendShareFileList(serverInfo, message);
            }

            ServerManager.mainForm.updateSharedFileListInvoke(serverInfo.user, sfList.Count.ToString());  
            FormTransfer.addConsoleMessage("共有ファイル情報を送信しました[" + serverInfo.name + "] " + sfList.Count + "files");

        }

        //差し替え
        private void sendSwitch()
        {
            FormTransfer.addConsoleMessage("共有ファイル差し替え情報を送信します[" + serverInfo.name + "]");

            int adCnt = 0;
            int rmCnt = 0;

            List<SharedFileInfo> currentFileInfoList = UploadManager.getSharedFileList();
            List<String> currentFilePathList = new List<String>();
            foreach (SharedFileInfo fileInfo in currentFileInfoList)
            {
                if (oldMap.ContainsKey(fileInfo.filePath) == false)
                {
                    // 追加
                    if (alive == false)
                    {
                        return;
                    }
                    ServerManager.notifyShareFile(serverInfo, fileInfo.filePath, fileInfo.fileSize);
                    adCnt += 1;
                }
                else
                {
                    oldMap.Remove(fileInfo.filePath);
                }
                currentFilePathList.Add(fileInfo.filePath);
            }

            foreach (String filepath in oldMap.Keys)
            {   
                // 削除
                if (alive == false)
                {
                    return;
                }
                ServerManager.removeFile(serverInfo, filepath);
                rmCnt += 1;                               
            }

            ServerManager.mainForm.updateSharedFileListInvoke(serverInfo.user, currentFileInfoList.Count.ToString());  
            FormTransfer.addConsoleMessage("共有ファイル差し替え情報を送信しました[" + serverInfo.name + "] 追加：" + adCnt + " 削除："+rmCnt);                
        }
    }


    // 共有ファイル読み込みスレッド
    public class ShareFileLoadThread
    {
        public bool alive = false;
        private Dictionary<String, SharedFileInfo> sharedInfoMap = new Dictionary<String, SharedFileInfo>();

        public void kill()
        {
            if (alive)
            {
                FormTransfer.addConsoleMessage("共有ファイル情報初期化を中断");
            }
            alive = false;
        }

        public void start()
        {
            Task.Factory.StartNew(() =>
            {
                try
                {
                    alive = true;
                    FormTransfer.addConsoleMessage("共有ファイル情報を初期化します");
                    loadDirInfo(AppCommon.setting.shareFolderList);
                    loadDirInfo(new String[]{ AppCommon.setting.downloadFolder });

                    if (alive)
                    {
                        lock (UploadManager.sharedLock)
                        {
                            if (alive)
                            {
                                UploadManager.switchSharedInfoMap(sharedInfoMap);
                            }
                        }
                    }
                }
                catch (Exception e)
                {
                    FormTransfer.addConsoleMessage("エラー：共有ファイル初期化に失敗:" + e.Message + "\n" + e.StackTrace);
                }
                finally
                {
                    if (alive)
                    {
                        FormTransfer.addConsoleMessage("共有ファイル情報を初期化しました " + sharedInfoMap.Count+"files");
                    }
                    alive = false;
                }
            });
        }

        private void loadDirInfo(String[] dirPathList)
        {
            if (dirPathList == null)
            {
                return;
            }

            foreach (String dirPath in dirPathList)
            {
                if (alive == false)
                {
                    return;
                }

                try
                {
                    if (Directory.Exists(dirPath) == false)
                    {
                        continue;
                    }

                    String[] files = Directory.GetFiles(dirPath);
                    foreach (String filePath in files)
                    {
                        if (alive == false)
                        {
                            return;
                        }

                        if (filePath.EndsWith(AppCommon.DL_WORK_FILE_EXP))
                        {
                            continue;
                        }

                        SharedFileInfo info = new SharedFileInfo(new FileInfo(filePath));
                        if (sharedInfoMap.ContainsKey(info.filePath) == false)
                        {
                            sharedInfoMap.Add(info.filePath, info);
                        }
                    }

                    if (AppCommon.setting.enableSubFolder)
                    {
                        string[] dirs = Directory.GetDirectories(dirPath);
                        loadDirInfo(dirs);
                    }
                }
                catch (Exception e)
                {
                    FormTransfer.addConsoleMessage("フォルダ情報の取得に失敗 Path:" + dirPath + " Message:" + e.Message);
                }
            }
        }
    }

    //

    public class UploadTaskInfo
    {
        private static int indexcounter = 40;
        public int index = 0;
        public bool uploading = false; //リクエストで上げる。タスク生存期間判定フラグ
        public long completeSize = 0;
        public long fileSize = 0;
        public String fileName;
        public String filePath;
        public String user;
        public String linktype;
        public ServerInfo serverInfo;
        public UploadTaskState state;
        public String errorString;
        public float compPct;
        public String ulSpeed;
        public String compPctString;
        public bool queueing = false;

        public UploadTaskInfo(ServerInfo serverInfo, String nick, String filePath, String speed)
        {
            this.filePath = filePath;
            this.fileName = AppCommon.filePath2FileName(this.filePath);
            this.serverInfo = serverInfo;
            this.user = nick;
            this.linktype = AppCommon.code2LinkType(speed);
            this.state = UploadTaskState.DEAD;
            indexcounter += 1;
            this.index = indexcounter;
        }
        public void cancel()
        {
            this.uploading = false;
            this.state = UploadTaskState.CANCELED;     
        }

    }

    public enum UploadTaskState
    {
        DEAD,
        CANCELED,
        WAIT,
        UPLOADING, //active
        COMPLETE,
    }
}
